useAnt = true;
if (useAnt) {
    // Get the script attributes
    hostStr = attributes.get("host");
    portStr = attributes.get("port");
    userStr = attributes.get("user");
    passwordStr = attributes.get("password");
    repositoryStr = attributes.get("repository");

    groupName = attributes.get("groupname");
    pageName = attributes.get("pagename");
    pageDescription = attributes.get("pagedescription");
    shareType = attributes.get("sharetype");
    profile = attributes.get("profile");
    role = attributes.get("role");
    pageRank = attributes.get("pagerank");
    page = attributes.get("page");
    properties = attributes.get("properties");
    collectionData = attributes.get("collectiondata");

    layoutType = attributes.get("layouttype");
    columnWidthsStr = attributes.get("columnwidths");
    jsr168Preferences = attributes.get("jsr168preferences");
    jsr168PortletStatus = attributes.get("jsr168portletstatus");

    rcPropName = attributes.get("resultsproperty");

    // Define the logger
    ant = new AntBuilder();
    logger = { level, msg ->
        ant.echo(level:level, message:msg);
    }

    // Get the utilities directory to locate JomaWrapper
    //utilsDir = project.getProperty("bsh.src.dir");
    utilsDir = project.getProperty("install.cfgwizard.utilities.dir");
} else {
    // Get the script attributes
    hostStr = args[0];
    portStr = args[1];
    userStr = args[2];
    passwordStr = args[3];
    repositoryStr = args[4];

    groupName = args[5];
    pageName = args[6];
    pageDescription = args[7];
    shareType = args[8];
    profile = args[9];
    role = args[10];
    pageRank = args[11];
    page = args[12];
    properties = args[13];
    collectionData = args[14];

    layoutType = args[15];
    columnWidthsStr = args[16];
    jsr168preferences = args[17];
    jsr168portletStatus = args[18];

    // Define the logger
    logger = { level, msg ->
        println(msg);
    }

    // Get the utilities directory to locate JomaWrapper
    utilsDir = ".";
}

def getNumberOfPortletsInColumn(portletData, col) {
    int retval = 0;

    // Loop through each row and check if a valid portlet is in the specified portlet
    for (row in 1..portletData.size()) {
        if (portletData[row-1][col-1].name != null) {
            retval++;
        }
    }

    return retval;
}

def loadPage(jomaWrapper, groupName, pageName, pageDescription, shareType, profile, role, pageRank, page, properties, collectionData, layoutType, columnWidthsStr, jsr168Preferences, jsr168PortletStatus, log) {
    pageHelper = evaluate(new File("$utilsDir/PageHelper.groovy").getText("UTF-8"));
    def rc = pageHelper.RC_OK;

    // Verify that the page does not exist
    def pageJoma = null;
    if (shareType.equalsIgnoreCase("Available")) {
        pageJoma = jomaWrapper.getMetadataObject("PSPortalPage", "@Name='$pageName'][Trees/Tree[@Name='$groupName Permissions Tree']");
    } else {
        pageJoma = jomaWrapper.getMetadataObject("PSPortalPage", "@Name='$pageName'][Trees/Tree[@Name='$shareType']/ParentTree/Tree[@Name='$profile']/ParentTree/Tree[@Name='$role']/ParentTree/Tree[@Name='RolesTree']/ParentTree/Tree[@Name='$groupName Permissions Tree']");
    }
    if (pageJoma == null) {
        def permTree = pageHelper.getOrCreateGroupPermissionsTree(jomaWrapper, groupName, log);
        if (permTree != null) {
            parentTree = pageHelper.getOrCreateParentTree(jomaWrapper, permTree, groupName, shareType, profile, role, log);
            if (parentTree != null) {
                // Get the portlet data based on input data
                portletData = pageHelper.getAllPortletData(jomaWrapper, layoutType, page, properties, collectionData, jsr168Preferences, jsr168PortletStatus, log);
                if (portletData != null) {
                    // Verify that all portlets can be created
                    def portletsCanBeCreated = true;
                    for (i in 1..portletData.size()) {
                        for (j in 1..portletData[i-1].size()) {
                            def portletCell = portletData[i-1][j-1];
                            if (portletCell.portletTemplate != null) {
                                def validPortlet = portletCell.portletTemplateJustCreated;
                                if (! validPortlet) {
                                    def canUserCreateAnotherExt = jomaWrapper.getMetadataObject("Extension", "@Name='CanUserCreateAnother'][OwningObject/Prototype[@Name='${portletCell.prototypeName}']");
                                    if (canUserCreateAnotherExt != null) {
                                        validPortlet = canUserCreateAnotherExt.value.equalsIgnoreCase("true");
                                    } else {
                                        // Assume that the portlet template was 
                                        // just created by a previous job and 
                                        // allow the portlet to be created
                                        validPortlet = true;
                                    }
                                }

                                if (! validPortlet) {
                                    log("verbose", "Portlet `${portletCell.name}' can not be shared because of portlet template settings");
                                    portletsCanBeCreated = false;
                                }
                            }
                        }
                    }
                    if (portletsCanBeCreated) {
                        log("verbose", "loadPage: $portletData");

                        // Create the page
                        pageJoma = jomaWrapper.newMetadata(pageName, "PSPortalPage");
                        pageJoma.trees.add(parentTree);
                        pageJoma.desc = pageDescription;
                        pageJoma.numberOfColumns = portletData[0].size();
                        pageJoma.type = "Column";

                        // Create the extension for the page rank
                        def pageRankExt = jomaWrapper.newMetadata("PageRank", "Extension");
                        pageRankExt.value = pageRank;
                        pageRankExt.owningObject = pageJoma;

                        // Create the extension for the layout type
                        def layoutTypeExt = jomaWrapper.newMetadata("LayoutType", "Extension");
                        layoutTypeExt.value = layoutType;
                        layoutTypeExt.owningObject = pageJoma;

                        // Create the extension for the share type
                        def shareTypeExt = jomaWrapper.newMetadata("SharedPage$shareType", "Extension");
                        shareTypeExt.owningObject = pageJoma;

                        // Create a list from the column widths
                        def columnWidths = columnWidthsStr.tokenize(",");

                        // Loop through each portlet (note column is the outer loop in this case)
                        for (j in 1..portletData[0].size()) {
                            // Create this column
                            column = jomaWrapper.newMetadata("COLUMNLAYOUTCOMPONENT", "PSColumnLayoutComponent");
                            pageJoma.layoutComponents.add(column);
                            column.numberOfPortlets = getNumberOfPortletsInColumn(portletData, j);
                            if (columnWidths.size() >= j) {
                                column.columnWidth = columnWidths[j-1];
                            }

                            // Loop through each row
                            for (i in 1..portletData.size()) {
                                def portletCell = portletData[i-1][j-1];

                                // Name can be null for column layouts that are not a perfect rectangle of portlets
                                if (portletCell.name != null) {
                                    // Create this cell different for "special" portlets
                                    def portlet = null;
                                    switch (portletCell.name) {
                                    case pageHelper.COLUMN_SPANNER_PORTLET_NAME:
                                        // This column's portlet is the previous column's portlet
                                        portlet = portletData[i-1][j-1-1].portlet;
                                        break;
                                    case pageHelper.SPANNER_PORTLET_NAME:
                                    case pageHelper.PLACE_HOLDER_PORTLET_NAME:
                                        // Create a spanner/placeHolder portlet for this cell
                                        portlet = jomaWrapper.newMetadata(portletCell.name, "PSPortlet");
                                        portlet.portletType = portletCell.name;
                                        break;
                                    default:
                                        // Create the portlet
                                        portlet = jomaWrapper.newMetadata(portletCell.name, "PSPortlet");
                                        if (portlet.desc != null) {
                                            portlet.desc = portletCell.description;
                                        }
                                        portlet.portletType = portletCell.portletTemplate.name - " template";
                                        portlet.usingPrototype = portletCell.portletTemplate;
                                        portlet.trees.add(permTree);

                                        // Create the property set for properties and/or preferences if needed
                                        def portletConfigRoot = null;
                                        if (portletCell.props != null || portletCell.preferences != null) {
                                            portletConfigRoot = jomaWrapper.newMetadata("PORTLET_CONFIG_ROOT", "PropertySet");
                                            portletConfigRoot.owningObject = portlet;
                                        }


                                        // If there are any properties to add to this portlet
                                        if (portletCell.props != null) {
                                            // Loop through each property
                                            for (mapEntry in portletCell.props) {
                                                // Create a property for this map entry
                                                def prop = jomaWrapper.newMetadata(mapEntry.key, "Property");
                                                prop.propertyName = mapEntry.key;
                                                prop.defaultValue = mapEntry.value;
                                                prop.SQLType = 12;
                                                prop.associatedPropertySet = portletConfigRoot;
                                            }
                                        }

                                        // If there are any collection items to add to this portlet
                                        if (portletCell.collectionObjects != null) {
                                            // Create the collection group
                                            def collectionGroup = jomaWrapper.newMetadata("Portal Collection", "Group");
                                            collectionGroup.desc = "Additional collection data for template portlet";
                                            collectionGroup.members.add(portlet);
                                            collectionGroup.trees.add(permTree);

                                            // Loop through each collection object
                                            for (collectionObject in portletCell.collectionObjects) {
                                                // Add this object to the collection
                                                collectionGroup.members.add(collectionObject);
                                            }
                                        }

                                        // If there are any preferences to add to this portlet
                                        if (portletCell.preferences != null) {
                                            // Loop through each preference
                                            for (mapEntry in portletCell.preferences) {
                                                // Create a property set for this map entry
                                                def preference = jomaWrapper.newMetadata(mapEntry.key, "PropertySet");
                                                preference.owningObject = portletConfigRoot;

                                                // Create an extension for the read only value
                                                def readOnlyExt = jomaWrapper.newMetadata("ReadOnly", "Extension");
                                                readOnlyExt.value = mapEntry.value.readOnly;
                                                readOnlyExt.owningObject = preference;

                                                // Create a property for each value in the value list
                                                for (val in mapEntry.value.valueList) {
                                                    def prop = jomaWrapper.newMetadata("PreferenceInstanceProperty", "Property");
                                                    prop.propertyName = prop.name;
                                                    prop.defaultValue = val;
                                                    prop.SQLType = 12;
                                                    prop.associatedPropertySet = preference;
                                                }
                                            }
                                        }

                                        // If there is any portlet status information
                                        if (portletCell.windowState != null || portletCell.portletMode != null) {
                                            // Create the portlet render state
                                            def portletRenderState = jomaWrapper.newMetadata("PortletRenderState", "PropertySet");
                                            portletRenderState.owningObject = portlet;

                                            // Create the window state extension
                                            if (portletCell.windowState != null) {
                                                def windowStateExt = jomaWrapper.newMetadata("WindowState", "Extension");
                                                windowStateExt.value = portletCell.windowState;
                                                windowStateExt.owningObject = portletRenderState;
                                            }

                                            // Create the portlet mode extension
                                            if (portletCell.portletMode != null) {
                                                def portletModeExt = jomaWrapper.newMetadata("PortletMode", "Extension");
                                                portletModeExt.value = portletCell.portletMode;
                                                portletModeExt.owningObject = portletRenderState;
                                            }
                                        }
                                        break;
                                    }

                                    // Add the portlet to this column
                                    portletData[i-1][j-1].portlet = portlet;
                                    column.portlets.add(portlet);
                                } else {
                                    log("verbose", "Skipping portlet[$i,$j] as there is no portlet defined");
                                }
                            }
                        }

                        // Make sure the sharing property is updated
                        pageHelper.updateSharingProperty(jomaWrapper, log);

                        // Create the page template in metadata
                        pageJoma.updateMetadataAll();
                        log("verbose", "Page created");
                    } else {
                        rc = pageHelper.RC_CANT_SHARE_PORTLET;
                        log("verbose", "One or more of the portlets can not be on a shared page");
                    }
                } else {
                    rc = pageHelper.RC_ERR_PAGE_DATA;
                    log("verbose", "Error in page data");
                }
            } else {
                rc = pageHelper.RC_ERR_CREATING_PARENT_TREE;
                log("verbose", "Error creating parent tree");
            }
        } else {
            rc = pageHelper.RC_ERR_CREATING_PERM_TREE;
            log("verbose", "Error creating $groupName permissions tree");
        }
    } else {
        rc = pageHelper.RC_OBJ_EXISTS;
        log("verbose", "Portal page with scope '$shareType' and name '$pageName' already exists");
    }

    return rc;
}

rc = 0;

try {
    // Connect to the metadata server
    jomaWrapper = evaluate(new File("$utilsDir/JomaWrapper.groovy").getText("UTF-8"));
    jomaWrapper.connect(hostStr, 
                        portStr, 
                        userStr, 
                        passwordStr,
                        repositoryStr);

    rc = loadPage(jomaWrapper, 
                  groupName, 
                  pageName, 
                  pageDescription, 
                  shareType, 
                  profile, 
                  role, 
                  pageRank, 
                  page, 
                  properties, 
                  collectionData, 
                  layoutType,
                  columnWidthsStr,
                  jsr168Preferences,
                  jsr168PortletStatus,
                  logger);

    // Disconnect from the metadata server
    jomaWrapper.disconnect();
} catch (Throwable t) {
    sw = new StringWriter();
    pw = new PrintWriter(sw);
    t.printStackTrace(pw);
    logger("info", sw.toString());
    rc = 1;
}

if (useAnt) {
    // Set the return code property
    project.setNewProperty(rcPropName, "" + rc);
} else {
    logger("info", "rc=$rc");
}

