useAnt = true;
if (useAnt) {
    // Get the script attributes
    hostStr = attributes.get("host");
    portStr = attributes.get("port");
    userStr = attributes.get("user");
    passwordStr = attributes.get("password");
    repositoryStr = attributes.get("repository");

    groupName = attributes.get("groupname");
    pageName = attributes.get("pagename");
    pageDescription = attributes.get("pagedescription");
    shareType = attributes.get("sharetype");
    profile = attributes.get("profile");
    role = attributes.get("role");
    pageRank = attributes.get("pagerank");
    pageTemplate = attributes.get("pagetemplate");
    properties = attributes.get("properties");
    collectionData = attributes.get("collectiondata");

    layoutType = attributes.get("layouttype");
    columnWidthsStr = attributes.get("columnwidths");
    jsr168Preferences = attributes.get("jsr168preferences");
    jsr168PortletStatus = attributes.get("jsr168portletstatus");

    rcPropName = attributes.get("resultsproperty");

    // Define the logger
    ant = new AntBuilder();
    logger = { level, msg ->
        ant.echo(level:level, message:msg);
    }

    // Get the utilities directory to locate JomaWrapper
    //utilsDir = project.getProperty("bsh.src.dir");
    utilsDir = project.getProperty("install.cfgwizard.utilities.dir");
} else {
    // Get the script attributes
    hostStr = args[0];
    portStr = args[1];
    userStr = args[2];
    passwordStr = args[3];
    repositoryStr = args[4];

    groupName = args[5];
    pageName = args[6];
    pageDescription = args[7];
    shareType = args[8];
    profile = args[9];
    role = args[10];
    pageRank = args[11];
    pageTemplate = args[12];
    properties = args[13];
    collectionData = args[14];

    layoutType = args[15];
    columnWidthsStr = args[16];
    jsr168preferences = args[17];
    jsr168portletStatus = args[18];

    // Define the logger
    logger = { level, msg ->
        println(msg);
    }

    // Get the utilities directory to locate JomaWrapper
    utilsDir = ".";
}

def loadPageTemplate(jomaWrapper, groupName, pageName, pageDescription, shareType, profile, role, pageRank, pageTemplate, properties, collectionData, layoutType, columnWidthsStr, jsr168Preferences, jsr168PortletStatus, log) {
    pageHelper = evaluate(new File("$utilsDir/PageHelper.groovy").getText("UTF-8"));
    def rc = pageHelper.RC_OK;

    // Verify that the page template does not exist
    def pageTemplateJoma = jomaWrapper.getMetadataObject("Tree", "@Name='$pageName'][ParentTree/Tree[@Name='$shareType']/ParentTree/Tree[@Name='$profile']/ParentTree/Tree[@Name='$role']/ParentTree/Tree[@Name='RolesTree']/ParentTree/Tree[@Name='$groupName Permissions Tree']");
    if (pageTemplateJoma == null) {
        // Get the parent tree
        def parentTree = pageHelper.getOrCreateParentTree(jomaWrapper, groupName, shareType, profile, role, log);
        if (parentTree != null) {
            // Get a property type to use (or create one if not found)
            def stringType = jomaWrapper.getMetadataObject("PropertyType", "@Name='String'");
            if (stringType == null) {
                stringType = jomaWrapper.newMetadata("String", "PropertyType");
                stringType.SQLType = 12;
            }

            // Get the portlet data based on input data
            portletData = pageHelper.getAllPortletData(jomaWrapper, layoutType, pageTemplate, properties, collectionData, jsr168Preferences, jsr168PortletStatus, log);
            if (portletData != null) {
                log("verbose", "loadPageTemplate: $portletData");

                // Create the page template
                pageTemplateJoma = jomaWrapper.newMetadata(pageName, "Tree");
                pageTemplateJoma.parentTree = parentTree;
                pageTemplateJoma.desc = pageDescription;
                pageTemplateJoma.treeType = "PortalPageTemplate";

                // Create the extension for the page rank
                def pageRankExt = jomaWrapper.newMetadata("PageRank", "Extension");
                pageRankExt.value = pageRank;
                pageRankExt.owningObject = pageTemplateJoma;

                // Create the extension for the layout type
                def layoutTypeExt = jomaWrapper.newMetadata("LayoutType", "Extension");
                layoutTypeExt.value = layoutType;
                layoutTypeExt.owningObject = pageTemplateJoma;

                // Create a list from the column widths
                def columnWidths = columnWidthsStr.tokenize(",");

                // Loop through each portlet (note column is the outer loop in this case)
                for (j in 1..portletData[0].size()) {
                    // Create this column
                    column = jomaWrapper.newMetadata("Column $j", "Tree");
                    pageTemplateJoma.subTrees.add(column);
                    column.treeType = "PortalPageColumnTemplate";

                    // Create the extension for the column width
                    if (columnWidths.size() >= j) {
                        def columnWidthExt = jomaWrapper.newMetadata("ColumnWidth", "Extension");
                        columnWidthExt.value = columnWidths[j-1];
                        columnWidthExt.owningObject = column;
                    }

                    // Loop through each row
                    for (i in 1..portletData.size()) {
                        def portletCell = portletData[i-1][j-1];

                        // Name can be null for column layouts that are not a perfect rectangle of portlets
                        if (portletCell.name != null) {
                            // Create the portlet
                            def portlet = jomaWrapper.newMetadata("Portlet Template Group", "Group");
                            portlet.desc = portletCell.name;
                            if (portletCell.description != null) {
                                portlet.desc += ",${portletCell.description}";
                            }
                            if (portletCell.portletTemplate != null) {
                                portlet.members.add(portletCell.portletTemplate);
                            }

                            // Create the property set for properties and/or preferences if needed
                            def portletConfigRoot = null;
                            if (portletCell.props != null || portletCell.preferences != null) {
                                portletConfigRoot = jomaWrapper.newMetadata("PORTLET_CONFIG_ROOT", "PropertySet");
                                portletConfigRoot.owningObject = portlet;
                            }


                            // If there are any properties to add to this portlet
                            if (portletCell.props != null) {
                                // Loop through each property
                                for (mapEntry in portletCell.props) {
                                    // Create a property for this map entry
                                    def prop = jomaWrapper.newMetadata(mapEntry.key, "Property");
                                    prop.propertyName = mapEntry.key;
                                    prop.defaultValue = mapEntry.value;
                                    prop.SQLType = 12;
                                    prop.associatedPropertySet = portletConfigRoot;
                                    prop.owningType = stringType;
                                }
                            }

                            // If there are any collection items to add to this portlet
                            if (portletCell.collectionObjects != null) {
                                // Create the collection group
                                def collectionGroup = jomaWrapper.newMetadata("Portal Collection", "Group");
                                collectionGroup.desc = "Additional collection data for template portlet";
                                portlet.members.add(collectionGroup);

                                // Loop through each collection object
                                for (collectionObject in portletCell.collectionObjects) {
                                    // Add this object to the collection
                                    collectionGroup.members.add(collectionObject);
                                }
                            }

                            // If there are any preferences to add to this portlet
                            if (portletCell.preferences != null) {
                                // Loop through each preference
                                for (mapEntry in portletCell.preferences) {
                                    // Create a property set for this map entry
                                    def preference = jomaWrapper.newMetadata(mapEntry.key, "PropertySet");
                                    preference.owningObject = portletConfigRoot;

                                    // Create an extension for the read only value
                                    def readOnlyExt = jomaWrapper.newMetadata("ReadOnly", "Extension");
                                    readOnlyExt.value = mapEntry.value.readOnly;
                                    readOnlyExt.owningObject = preference;

                                    // Create a property for each value in the value list
                                    for (val in mapEntry.value.valueList) {
                                        def prop = jomaWrapper.newMetadata("PreferenceInstanceProperty", "Property");
                                        prop.propertyName = prop.name;
                                        prop.defaultValue = val;
                                        prop.SQLType = 12;
                                        prop.associatedPropertySet = preference;
                                    }
                                }
                            }

                            // If there is any portlet status information
                            if (portletCell.windowState != null || portletCell.portletMode != null) {
                                // Create the portlet render state
                                def portletRenderState = jomaWrapper.newMetadata("PortletRenderState", "PropertySet");
                                portletRenderState.owningObject = portlet;

                                // Create the window state extension
                                if (portletCell.windowState != null) {
                                    def windowStateExt = jomaWrapper.newMetadata("WindowState", "Extension");
                                    windowStateExt.value = portletCell.windowState;
                                    windowStateExt.owningObject = portletRenderState;
                                }

                                // Create the portlet mode extension
                                if (portletCell.portletMode != null) {
                                    def portletModeExt = jomaWrapper.newMetadata("PortletMode", "Extension");
                                    portletModeExt.value = portletCell.portletMode;
                                    portletModeExt.owningObject = portletRenderState;
                                }
                            }

                            // Add the portlet to this column
                            portletData[i-1][j-1].portlet = portlet;
                            column.members.add(portlet);
                        } else {
                            log("verbose", "Skipping portlet[$i,$j] as there is no portlet defined.");
                        }
                    }
                }

                // Make sure the sharing property is updated
                pageHelper.updateSharingProperty(jomaWrapper, log);

                // Create the page template in metadata
                pageTemplateJoma.updateMetadataAll();
                log("verbose", "Page template created");
            } else {
                rc = pageHelper.RC_ERR_PAGE_DATA;
                log("verbose", "Error in pageTemplate data");
            }
        } else {
            rc = pageHelper.RC_ERR_CREATING_PERM_TREE;
            log("verbose", "Error creating $groupName Permissions Tree");
        }
    } else {
        rc = pageHelper.RC_OBJ_EXISTS;
        log("verbose", "Page template '$pageName' already exists");
    }

    return rc;
}

rc = 0;

try {
    // Connect to the metadata server
    jomaWrapper = evaluate(new File("$utilsDir/JomaWrapper.groovy").getText("UTF-8"));
    jomaWrapper.connect(hostStr, 
                        portStr, 
                        userStr, 
                        passwordStr,
                        repositoryStr);

    rc = loadPageTemplate(jomaWrapper, 
                          groupName, 
                          pageName, 
                          pageDescription, 
                          shareType, 
                          profile, 
                          role, 
                          pageRank, 
                          pageTemplate, 
                          properties, 
                          collectionData, 
                          layoutType,
                          columnWidthsStr,
                          jsr168Preferences,
                          jsr168PortletStatus,
                          logger);

    // Disconnect from the metadata server
    jomaWrapper.disconnect();
} catch (Throwable t) {
    sw = new StringWriter();
    pw = new PrintWriter(sw);
    t.printStackTrace(pw);
    logger("info", sw.toString());
    rc = 1;
}

if (useAnt) {
    // Set the return code property
    project.setNewProperty(rcPropName, "" + rc);
} else {
    logger("info", "rc=$rc");
}

