class PreferenceData {
    def readOnly = null;
    def valueList = null;

    String toString() {
        return "PreferenceData {readOnly:$readOnly, " +
                "valueList:$valueList}";
    }
}

class PortletCell {
    def name = null;
    def description = null;
    def prototypeName = null;
    def portletTemplate = null;
    def props = null;
    def collectionObjects = null;
    def preferences = null;
    def windowState = null;
    def portletMode = null;
    def portlet = null;
    def portletTemplateJustCreated = false;

    String toString() {
        return "PortletCell {name:$name, " +
                "description:$description, " +
                "prototypeName:$prototypeName, " +
                "portletTemplate:$portletTemplate, " +
                "props:$props, " +
                "collectionObjects:$collectionObjects, " +
                "preferences:$preferences, " +
                "windowState:$windowState, " +
                "portletMode:$portletMode}";
    }
}

class PageHelper {
    // Define common return codes
    def RC_OK                           = 0;
    def RC_EXCEPTION                    = 1;
    def RC_OBJ_EXISTS                   = 2;
    def RC_OBJ_NOT_EXISTS               = 3;
    def RC_ERR_CREATING_PERM_TREE       = 4;
    def RC_WARNING                      = 5;
    def RC_ERR_PAGE_DATA                = 6;
    def RC_CANT_SHARE_PORTLET           = 7;
    def RC_ERR_CREATING_PARENT_TREE     = 8;

    // Define keys to get values from maps
    def COLUMN_NUM_KEY                  = "columnNum";
    def PORTLET_POS_KEY                 = "portletPos";
    def PORTLET_NAME_KEY                = "portletName";
    def PORTLET_DESCRIPTION_KEY         = "portletDescription";
    def PROTOTYPE_NAME_KEY              = "prototypeName";
    def COL_SPAN_KEY                    = "colSpan";
    def ROW_SPAN_KEY                    = "rowSpan";
    def COL_POS_KEY                     = "colPos";
    def PROP_NAME_KEY                   = "propName";
    def PROP_VALUE_KEY                  = "propValue";
    def DATA_TYPE_KEY                   = "dataType";
    def SEARCH_STR_KEY                  = "searchStr";
    def READ_ONLY_KEY                   = "readOnly";
    def PROP_VALUES_KEY                 = "propValues";
    def WINDOW_STATE_KEY                = "windowState";
    def PORTLET_MODE_KEY                = "portletMode";

    // Define lists of keys to match the input lists
    def PAGE_TEMPLATE_KEY_LIST          = [COLUMN_NUM_KEY, PORTLET_POS_KEY, PORTLET_NAME_KEY, PORTLET_DESCRIPTION_KEY, PROTOTYPE_NAME_KEY, COL_SPAN_KEY, ROW_SPAN_KEY];
    def PROPERTIES_KEY_LIST             = [COL_POS_KEY, PROP_NAME_KEY, PROP_VALUE_KEY];
    def COLLECTION_DATA_KEY_LIST        = [COL_POS_KEY, DATA_TYPE_KEY, SEARCH_STR_KEY];
    def JSR168_PREFERENCES_KEY_LIST     = [COLUMN_NUM_KEY, PORTLET_POS_KEY, READ_ONLY_KEY, PROP_NAME_KEY, PROP_VALUES_KEY];
    def JSR168_PORTLET_STATUS_KEY_LIST  = [COLUMN_NUM_KEY, PORTLET_POS_KEY, WINDOW_STATE_KEY, PORTLET_MODE_KEY];

    // Define special portlet names
    def COLUMN_SPANNER_PORTLET_NAME     = "ColumnSpanner";
    def SPANNER_PORTLET_NAME            = "Spanner";
    def PLACE_HOLDER_PORTLET_NAME       = "PlaceHolder";

    def WINDOW_STATE_MINIMIZED          = "0";
    def WINDOW_STATE_NORMAL             = "1";
    def WINDOW_STATE_MAXIMIZED          = "2";

    def MODE_VIEW                       = "0";
    def MODE_EDIT                       = "1";
    def MODE_HELP                       = "2";


    def getPortletData(portletsList, layoutType, log) {
        int maxRow = 0;
        int maxCol = 0;
    
        // [COLUMN_NUM_KEY, PORTLET_POS_KEY, PORTLET_NAME_KEY, PORTLET_DESCRIPTION_KEY, PROTOTYPE_NAME_KEY]
        // 1,1,Bookmarks,Bookmarks portlet Description,Bookmarks template,1,1
        // 1,2,Alerts,Alerts portlet Description,Alerts template,1,1
        // 2,1,My collection Portlet,Description of collection portlet,Collection template,1,1
        // 2,2,Welcome,Welcome portlet Description,Welcome template,1,1
    
        // Loop through each portlet definition in the page template
        // to get the total number of rows and columns
        for (portletMap in portletsList) {
            int portletCol = portletMap[COLUMN_NUM_KEY].toInteger();
            if (portletCol > maxCol) {
                maxCol = portletCol;
            }
    
            int portletRow = portletMap[PORTLET_POS_KEY].toInteger();
            if (portletRow > maxRow) {
                maxRow = portletRow;
            }
        }
    
        // Allocate the space for all of the portlet data
        def portletData = new java.util.ArrayList();
        for (i in 1..maxRow) {
            portletData[i-1] = new java.util.ArrayList();
            for (j in 1..maxCol) {
                portletData[i-1][j-1] = new PortletCell();
            }
        }
        int totalPortlets = maxRow * maxCol;
    
        if (layoutType.equals("Column")) {
            // Loop through each portlet specified
            for (portletMap in portletsList) {
                // Create the portlet for this portlet map
                int row = portletMap[PORTLET_POS_KEY].toInteger();
                int col = portletMap[COLUMN_NUM_KEY].toInteger();
                def portlet = portletData[row-1][col-1];
                portlet.name = portletMap[PORTLET_NAME_KEY];
                portlet.description = portletMap[PORTLET_DESCRIPTION_KEY];
                portlet.prototypeName = portletMap[PROTOTYPE_NAME_KEY];
                def rowSpanStr = portletMap[ROW_SPAN_KEY];
                def colSpanStr = portletMap[COL_SPAN_KEY];
                int rowSpan = rowSpanStr.isInteger() ? rowSpanStr.toInteger() : 1;
                int colSpan = colSpanStr.isInteger() ? colSpanStr.toInteger() : 1;

                // Verify row and column span
                if (rowSpan > 1) {
                    log("verbose", "Column layout can not have a rowSpan > 1 for pageTemplate data $portletMap");
                    portletData = null;
                    break;
                } else if (colSpan > 1) {
                    log("verbose", "Column layout can not have a colSpan > 1 for pageTemplate data $portletMap");
                    portletData = null;
                    break;
                }
            }
        } else if (layoutType.equals("Grid")) {
            for (portletMap in portletsList) {
                // Create the portlet for this portlet map
                int row = portletMap[PORTLET_POS_KEY].toInteger();
                int col = portletMap[COLUMN_NUM_KEY].toInteger();
                def portlet = portletData[row-1][col-1];
                portlet.name = portletMap[PORTLET_NAME_KEY];
                portlet.description = portletMap[PORTLET_DESCRIPTION_KEY];
                portlet.prototypeName = portletMap[PROTOTYPE_NAME_KEY];
                def rowSpanStr = portletMap[ROW_SPAN_KEY];
                def colSpanStr = portletMap[COL_SPAN_KEY];
                int rowSpan = rowSpanStr.isInteger() ? rowSpanStr.toInteger() : 1;
                int colSpan = colSpanStr.isInteger() ? colSpanStr.toInteger() : 1;
    
                // Verify row and column span
                int rowExtent = row + rowSpan - 1;
                int colExtent = col + colSpan - 1;
                if (rowExtent > maxRow) {
                    log("verbose", "Rowspan greater than max rows for pageTemplate data $portletMap");
                    portletData = null;
                    break;
                } else if (colExtent > maxCol) {
                    log("verbose", "Colspan greater than max columns for pageTemplate data $portletMap");
                    portletData = null;
                    break;
                }
    
                // Setup the rows and columns for this portlet
                for (i in row..rowExtent) {
                    for (j in col..colExtent) {
                        def p = portletData[i-1][j-1];
                        if (i == row) {
                            if (j == col) {
                                // Top corner is the portlet itself
                                // Do nothing else
                            } else {
                                // Top row is filled with column spanners
                                p.name = COLUMN_SPANNER_PORTLET_NAME;
                            }
                        } else if (j == col) {
                            // Left column is filled with spanners
                            p.name = SPANNER_PORTLET_NAME;
                        } else {
                            // Middle rows and columns are filled with column spanners
                            p.name = COLUMN_SPANNER_PORTLET_NAME;
                        }
                    }
                }
            }
    
            // Fill in the gaps with place holders
            for (i in 1..maxRow) {
                for (j in 1..maxCol) {
                    if (portletData[i-1][j-1].name == null) {
                        portletData[i-1][j-1].name = PLACE_HOLDER_PORTLET_NAME;
                    }
                }
            }
        } else {
            log("verbose", "Invalid layoutType '$layoutType' spcified");
            portletData = null;
        }
    
        return portletData;
    }
    
    def getListFromString(source, keys, log) {
        def retval = new java.util.ArrayList();
    
        // Each row is delimited by ~
        def rows = source.tokenize("~");
        for (row in rows) {
            def rowMap = new java.util.HashMap();
    
            // Each value for the row is delimted by ,.  Setup the tokenizer to get 
            // the delimiters back so we can tell when empty values are specified 
            // (i.e. ww,xx,,zz).  Also, if there are more values than keys, the last
            // key becomes a list of strings rather than just a string.
            def strTok = new java.util.StringTokenizer(row, ",", true);
            int i = 0;
            rowMap[keys[i]] = "";
            while (strTok.hasMoreTokens()) {
                //log("verbose", "----- i:$i -----");
    
                // If the token is a delimiter
                def val = strTok.nextToken();
                if (val.equals(",")) {
                    // Make sure i doesn't go beyond keys.size()
                    if (i < keys.size()) {
                        // Go to the next key
                        i++;
    
                        // Initialize the next key to a blank string if there is a next key
                        if (i < keys.size()) {
                            rowMap[keys[i]] = "";
                        }
                    }
                // If there are no more keys left for this value
                } else if (i == keys.size()) {
                    //log("verbose", "Adding extra values to last entry rowMap[keys[i-1]]:${rowMap[keys[i-1]]}");
    
                    // Get the list from tha last key
                    def valueList = rowMap[keys[i-1]];
    
                    // If the list is not there yet
                    if (valueList instanceof String) {
                        // Create the list and add the value as the first item in the list
                        log("verbose", "Creating list for last entry");
                        valueList = new java.util.ArrayList();
                        valueList.add(rowMap[keys[i-1]]);
                        rowMap[keys[i-1]] = valueList;
                    }
    
                    // Add this value to the list
                    valueList.add(val);
                    //log("verbose", "Last entry value: ${rowMap[keys[i-1]]}");
                } else {    // This is a value with a key
                    rowMap[keys[i]] = val;
                }
            }
    
            // Add this rowmap to the list of maps to be returned
            retval += rowMap;
        }
    
        log("verbose", "getListFromString: keys:$keys, retval:$retval");
        return retval;
    }
    
    def convertColPos(l, log) {
        // Loop through each map stored in the list
        for (map in l) {
            // Convert col_pos to columnNum and portletPos
            def colPos = map[COL_POS_KEY];
            def values = colPos.tokenize("_");
            map[COLUMN_NUM_KEY] = values[0];
            map[PORTLET_POS_KEY] = values[1];
        }
    
        log("verbose", "convertColPos: $l");
    }
    
    def addPropertiesToPortlets(portletData, propertiesList, log) {
        // [COL_POS_KEY, PROP_NAME_KEY, PROP_VALUE_KEY]
        // 1_1,Name0,DefaultValue 0
        // 2_1,MyCollectionPortletPropertyName1,My Collection Portlet Property DefaultValue 1
        // 2_1,MyCollectionPortletPropertyName2,My Collection Portlet Property DefaultValue 2
        // 2_1,MyCollectionPortletPropertyName3,My Collection Portlet Property DefaultValue 3
    
        // Loop through each property to define
        for (propMap in propertiesList) {
            def col = propMap[COLUMN_NUM_KEY].toInteger();
            def row = propMap[PORTLET_POS_KEY].toInteger();
            def propName = propMap[PROP_NAME_KEY];
            def propValue = propMap[PROP_VALUE_KEY];
    
            // Create the properties map if needed
            if (portletData[row-1][col-1].props == null) {
                portletData[row-1][col-1].props = new java.util.HashMap();
            }
    
            // Add this property
            portletData[row-1][col-1].props[propName] = propValue;
        }
    
        log("verbose", "addPropertiesToPortlets: $portletData");
    }
    
    def addCollectionToPortlets(jomaWrapper, portletData, collectionList, log) {
        // [COL_POS_KEY, DATA_TYPE_KEY, SEARCH_STR_KEY]
        // 1_1,Document,@Name='SAS'
        // 2_1,Document,@Name='SAS'
        // 2_1,Document,@Name='SAS Integration Technologies'
    
        // Loop through each collection item to find
        for (collectionMap in collectionList) {
            def col = collectionMap[COLUMN_NUM_KEY].toInteger();
            def row = collectionMap[PORTLET_POS_KEY].toInteger();
            def dataType = collectionMap[DATA_TYPE_KEY];
            def searchStr = collectionMap[SEARCH_STR_KEY];
    
            // Create the collection list if needed
            if (portletData[row-1][col-1].collectionObjects == null) {
                portletData[row-1][col-1].collectionObjects = new java.util.ArrayList();
            }
    
            // Find the collection item in metadata
            def collectionItem = jomaWrapper.getMetadataObject(dataType, searchStr);
            if (collectionItem != null) {
                // Add this item to the collection list
                portletData[row-1][col-1].collectionObjects.add(collectionItem);
            } else {
                log("info", "WARNING:  collection item '${dataType}[${searchStr}]' not found.  Skipping!");
            }
        }
    
        log("verbose", "addCollectionToPortlets: $portletData");
    }
    
    def addPreferencesToPortlets(portletData, preferencesList, log) {
        // [COLUMN_NUM_KEY, PORTLET_POS_KEY, READ_ONLY_KEY, PROP_NAME_KEY, PROP_VALUES_KEY]
        // 1,1,false,prefName,Pref Value 1, Pref Value 2
    
        // Loop through each preference to define
        for (prefMap in preferencesList) {
            def col = prefMap[COLUMN_NUM_KEY].toInteger();
            def row = prefMap[PORTLET_POS_KEY].toInteger();
            def readOnly = prefMap[READ_ONLY_KEY];
            def propName = prefMap[PROP_NAME_KEY];
            def valueList = prefMap[PROP_VALUES_KEY];
            if (valueList instanceof String) {
                valueList = new java.util.ArrayList();
                valueList.add(prefMap[PROP_VALUES_KEY]);
            }
    
            // Create the preferences map if needed
            if (portletData[row-1][col-1].preferences == null) {
                portletData[row-1][col-1].preferences = new java.util.HashMap();
            }
    
            // Create the preference data
            def prefData = new PreferenceData(readOnly:readOnly, valueList:valueList);
    
            // Add this preference
            portletData[row-1][col-1].preferences[propName] = prefData;
        }
    
        log("verbose", "addPreferencesToPortlets: $portletData");
    }
    
    def addPortletStatusToPortlets(portletData, portletStatusList, log) {
        // [COLUMN_NUM_KEY, PORTLET_POS_KEY, WINDOW_STATE_KEY, PORTLET_MODE_KEY]
        // 1,1,Minimized,Edit
        // 1,2,Maximized,Help
        // 1,3,Normal,View
    
        // Loop through each portlet status to define
        for (statusMap in portletStatusList) {
            def col = statusMap[COLUMN_NUM_KEY].toInteger();
            def row = statusMap[PORTLET_POS_KEY].toInteger();
            def windowStateStr = statusMap[WINDOW_STATE_KEY].toLowerCase();
            def portletModeStr = statusMap[PORTLET_MODE_KEY].toLowerCase();
    
            // Convert the window state
            def windowState = WINDOW_STATE_NORMAL;
            switch (windowStateStr) {
            case "minimized":
                windowState = WINDOW_STATE_MINIMIZED;
                break;
            case "maximized":
                windowState = WINDOW_STATE_MAXIMIZED;
                break;
            }
    
            // Convert the portlet mode
            def portletMode = MODE_VIEW;
            switch (portletModeStr) {
            case "edit":
                portletMode = MODE_EDIT;
                break;
            case "help":
                portletMode = MODE_HELP;
                break;
            }
    
            // Assign the window state and portlet mode to this portlet
            portletData[row-1][col-1].windowState = windowState;
            portletData[row-1][col-1].portletMode = portletMode;
        }
    
        log("verbose", "addPortletStatusToPortlets: $portletData");
    }
    
    def updateSharingProperty(jomaWrapper, log) {
        def sharingProperty = jomaWrapper.getMetadataObject("Property", "@Name='Portal.LastSharingPerformed'");
        if (sharingProperty != null) {
            sharingProperty.defaultValue = "-1";
        } else {
            log("verbose", "Portal.LastSharingPerformed Property object not found");
        }
    }
    

    def getOrCreateGroupPermissionsTree(jomaWrapper, groupName, log) {
        return getOrCreatePermissionsTree(jomaWrapper, "IdentityGroup", groupName, log);
    }

    def getOrCreateUserPermissionsTree(jomaWrapper, userName, log) {
        return getOrCreatePermissionsTree(jomaWrapper, "Person", userName, log);
    }

    def getOrCreatePermissionsTree(jomaWrapper, identityType, identityName, log) {
        def permTree = null;
    
        // Get the identity based on the name
        def identity = jomaWrapper.getMetadataObject(identityType, "@Name='$identityName'");
        if (identity != null) {
            // Get the permissions tree from the identity
            def usedByPrototypes = identity.usedByPrototypes;
            if (usedByPrototypes != null && usedByPrototypes.size() > 0) {
                permTree = usedByPrototypes.get(0);
            }
    
            // If the premissions tree does not exist
            if (permTree == null) {
                // Find the portal application tree
                def paTree = jomaWrapper.getMetadataObject("Tree", "@Name='Portal Application Tree'");
                if (paTree != null) {
                    // Find the portal ACT
                    def portalACT = jomaWrapper.getMetadataObject("AccessControlTemplate", "@Name='Portal ACT'");
                    if (portalACT != null) {
                        // Find the grant read metadata permission
                        def grantReadMetadata = jomaWrapper.getMetadataObject("Permission", "@Name='ReadMetadata' and @Type='GRANT'");
                        if (grantReadMetadata != null) {
                            // Find the grant write metadata permission
                            def grantWriteMetadata = jomaWrapper.getMetadataObject("Permission", "@Name='WriteMetadata' and @Type='GRANT'");
                            if (grantWriteMetadata != null) {
                                log("verbose", "Creating Permissions Tree for $identityName");
    
                                // Create the permissions tree
                                permTree = jomaWrapper.newMetadata("$identityName Permissions Tree", "Tree");
                                permTree.treeType = " Permissions Tree";
                                permTree.parentTree = paTree;
                                permTree.usingPrototype = identity;
    
                                // Create the ACE for the permissions tree
                                def ace = jomaWrapper.newMetadata("GRANT $identityName access", "AccessControlEntry");
                                def identities = ace.identities;
                                identities.add(identity);
                                ace.identities = identities;
    
                                def permissions = ace.permissions;
                                permissions.add(grantReadMetadata);
                                if (identityType.equals("Person")) {
                                    permissions.add(grantWriteMetadata);
                                }
                                ace.permissions = permissions;
    
                                // Add the access controls to the perm tree
                                def accessControls = permTree.accessControls;
                                accessControls.add(portalACT);
                                accessControls.add(ace);
                                permTree.accessControls = accessControls;
    
                                log("verbose", "Created Permissions Tree for $identityName");
                            } else {
                                log("verbose", "Could not find GRANT WriteMetadata permissions");
                            }
                        } else {
                            log("verbose", "Could not find GRANT ReadMetadata permissions");
                        }
                    } else {
                        log("verbose", "Could not find Portal ACT");
                    }
                } else {
                    log("verbose", "Could not find Portal Application Tree");
                }
            }
        } else {
            log("verbose", "Could not find $identityType with name $identityName");
        }
    
        return permTree;
    }

    def getOrCreateParentTree(jomaWrapper, groupName, shareType, profile, role, log) {
        def parentTree = null;

        def permTree = getOrCreateGroupPermissionsTree(jomaWrapper, groupName, log);
        if (permTree != null) {
            parentTree = getOrCreateParentTree(jomaWrapper, permTree, groupName, shareType, profile, role, log);
        } else {
            log("verbose", "Error creating $groupName permissions tree");
        }

        return parentTree;
    }

    def getOrCreateParentTree(jomaWrapper, permTree, groupName, shareType, profile, role, log) {
        def parentTree = null;

        if (shareType.equalsIgnoreCase("Available")) {
            parentTree = permTree;
        } else {
            // Check if the parent tree exists already
            parentTree = jomaWrapper.getMetadataObject("Tree", "@Name='$shareType'][ParentTree/Tree[@Name='$profile']/ParentTree/Tree[@Name='$role']/ParentTree/Tree[@Name='RolesTree']/ParentTree/Tree[@Name='$groupName Permissions Tree']");
            if (parentTree == null) {
                // Create the roles tree
                def rolesTree = jomaWrapper.newMetadata("RolesTree", "Tree");
                rolesTree.parentTree = permTree;

                // Create the role tree
                def roleTree = jomaWrapper.newMetadata("$role", "Tree");
                roleTree.parentTree = rolesTree;

                // Create the profile tree
                def profileTree = jomaWrapper.newMetadata("$profile", "Tree");
                profileTree.parentTree = roleTree;

                // Create the default tree
                def defaultTree = jomaWrapper.newMetadata("Default", "Tree");
                defaultTree.parentTree = profileTree;

                // Create the sticky tree
                def stickyTree = jomaWrapper.newMetadata("Sticky", "Tree");
                stickyTree.parentTree = profileTree;

                // Return the parent tree based on share type
                if (shareType.equals("Default")) {
                    parentTree = defaultTree;
                } else {
                    parentTree = stickyTree;
                }
            }
        }

        return parentTree;
    }

    def getAllPortletData(jomaWrapper, layoutType, pageTemplate, properties, collectionData, jsr168Preferences, jsr168PortletStatus, log) {
        // Parse the lists out of the strings
        def portletsList = getListFromString(pageTemplate, PAGE_TEMPLATE_KEY_LIST, log);
        def propertiesList = getListFromString(properties, PROPERTIES_KEY_LIST, log);
        def collectionList = getListFromString(collectionData, COLLECTION_DATA_KEY_LIST, log);
        def preferencesList = getListFromString(jsr168Preferences, JSR168_PREFERENCES_KEY_LIST, log);
        def portletStatusList = getListFromString(jsr168PortletStatus, JSR168_PORTLET_STATUS_KEY_LIST, log);
        convertColPos(propertiesList, log);
        convertColPos(collectionList, log);

        // Get all the portlet templates
        def portletTemplates = jomaWrapper.getMetadataObjects("Prototype", "@MetadataType='PSPortlet'");
        def newPortletTemplates = new java.util.ArrayList();

        // Fill in the portlet data based on the input data
        def portletData = getPortletData(portletsList, layoutType, log);
        if (portletData != null) {
            // Add the other list data to portlet data
            addPropertiesToPortlets(portletData, propertiesList, log);
            addCollectionToPortlets(jomaWrapper, portletData, collectionList, log);
            addPreferencesToPortlets(portletData, preferencesList, log);
            addPortletStatusToPortlets(portletData, portletStatusList, log);

            // Get the portlet templates based on the prototype names
            for (i in 1..portletData.size()) {
                for (j in 1..portletData[i-1].size()) {
                    def portlet = portletData[i-1][j-1];

                    // Prototype name will be null for place holders, spanners, and column spanners
                    def prototypeName = portlet.prototypeName;
                    if (prototypeName != null) {
                        def found = false;
                        for (portletTemplate in portletTemplates) {
                            // If this portlet template is the one referenced in this portlet
                            if (portletTemplate.name.equals(prototypeName)) {
                                // Add this portlet template to the portlet map and stop the loop
                                portlet.portletTemplate = portletTemplate;
                                found = true;
                                break;
                            }
                        }

                        // If we did not find a portlet template, create one
                        if (! found) {
                            log("info", "Portlet template named '$prototypeName' not found.  Creating...");

                            // See if we have already created this portlet template
                            def foundPortletTemplate = null;
                            for (newPortletTemplate in newPortletTemplates) {
                                if (newPortletTemplate.name.equals(prototypeName)) {
                                    foundPortletTemplate = newPortletTemplate;
                                    break;
                                }
                            }

                            // Create the portlet template now if needed
                            if (foundPortletTemplate == null) {
                                // Create the portlet template
                                foundPortletTemplate = jomaWrapper.newMetadata(prototypeName, "Prototype");
                                foundPortletTemplate.metadataType = "PSPortlet";

                                // Add the new portlet template to the list
                                newPortletTemplates.add(foundPortletTemplate);
                            }

                            // Update the portlet cell with the new information
                            portlet.portletTemplate = foundPortletTemplate;
                            portlet.portletTemplateJustCreated = true;
                        }
                    }
                }
            }
        } else {
            log("verbose", "Error in page data");
        }

        return portletData;
    }

    def createPortalDocument(jomaWrapper, name, desc, uri, identityName, identityType, textRole, log) {
        def rc = RC_OK;
    
        def doc = jomaWrapper.getMetadataObject("Document", "@Name='$name' and @TextRole='$textRole'][Trees/Tree[@Name='$identityName Permissions Tree']");
        if (doc == null) {
            log("verbose", "Creating portal document.  name:$name, desc:$desc, uri:$uri, identityName:$identityName, identityType:$identityType, textRole:$textRole");

            def permTree = getOrCreatePermissionsTree(jomaWrapper, identityType, identityName, log);
            if (permTree != null) {
                // Create the document
                doc = jomaWrapper.newMetadata(name, "Document");
                doc.desc = desc;
                doc.URI = uri;
                doc.textRole = textRole;
                doc.textType="text/html";
                doc.trees.add(permTree);

                // Update the metadata server
                doc.updateMetadataAll();

                log("verbose", "Portal document created.");
            } else {
                rc = RC_ERR_CREATING_PERM_TREE;
                log("verbose", "Error creating $identityName Permissions Tree");
            }
        } else {
            rc = RC_OBJ_EXISTS;
            log("verbose", "A document with Name='$name' and TextRole='$textRole' already exists.");
        }
    
        return rc;
    }

    def deletePortalDocument(jomaWrapper, name, identityName, textRole, log) {
        def rc = RC_OK;

        def doc = jomaWrapper.getMetadataObject("Document", "@Name='$name' and @TextRole='$textRole'][Trees/Tree[@Name='$identityName Permissions Tree']");
        if (doc != null) {
            log("verbose", "Deleting portal document.  name:$name, identityName:$identityName, textRole:$textRole");

            // Delete the object
            doc.delete();
            doc.updateMetadataAll();

            log("verbose", "Portal document deleted.");
        } else {
            rc = RC_OBJ_NOT_EXISTS;
            log("verbose", "A document with Name='$name' and TextRole='$textRole' does not exist in $identityName Permissions Tree.");
        }

        return rc;
    }

    def deleteAllExtensions(owningObject, log) {
        // Get all the extensions and remove them
        for (def extIt = owningObject.extensions.iterator(); extIt.hasNext(); ) {
            def ext = extIt.next();

            log("verbose", "Deleting Extension '${ext.name}'");
            extIt.remove();
            ext.delete();
        }

    }

    def deleteAllPropSets(owningObject, log) {
        for (def propSetIt = owningObject.propertySets.iterator(); propSetIt.hasNext(); ) {
            def propSet = propSetIt.next();
    
            // Get all the extensions and remove them
            deleteAllExtensions(propSet, log);
    
            // Get all the properties and remove them
            for (def propIt = propSet.setProperties.iterator(); propIt.hasNext(); ) {
                def prop = propIt.next();
    
                log("verbose", "Deleting Property '${prop.name}'");
                propIt.remove();
                prop.delete();
            }
    
            // Delete all the property sets on this property set
            deleteAllPropSets(propSet, log);
    
            // Delete this property set
            log("verbose", "Deleting PropertySet '${propSet.name}'");
            propSetIt.remove();
            propSet.delete();
        }
    }

    def deletePortlet(jomaWrapper, portlet, deletePortlets, log) {
        def ok = false;

        // Get the template for this portlet
        def template = portlet.usingPrototype;
        if (template == null) {
            // Just delete spanners, column spanners, and place holders
            ok = true;
        } else if (deletePortlets) {
            def canUserCreateAnotherExt = jomaWrapper.getMetadataObject("Extension", "@Name='CanUserCreateAnother'][OwningObject/Prototype[@Id='${template.id}']");
            if (canUserCreateAnotherExt != null && canUserCreateAnotherExt.value.equalsIgnoreCase("true")) {
                // Delete all the property sets associated with this portlet
                deleteAllPropSets(portlet, log);

                // Delete collection groups
                for (def groupIt = portlet.groups.iterator(); groupIt.hasNext(); ) {
                    def group = groupIt.next();

                    log("verbose", "Deleting Group '${group.name}'");
                    groupIt.remove();
                    group.delete();
                }

                // Delete this portlet
                ok = true;
            } else {
                log("info", "WARNING:  Portlet ${portlet.name} not deleted due to portlet data constraint");
            }
        } else {
            log("verbose", "Portlet ${portlet.name} not deleted since deletePortles=$deletePortlets");
        }

        // Delete the portlet if we should
        if (ok) {
            log("verbose", "Deleting PSPortlet '${portlet.name}'");
            portlet.delete();
        }
    }

    def deletePortalPage(jomaWrapper, page, deletePortlets, log) {
        // Get all the extensions and remove them
        deleteAllExtensions(page, log);

        // Loop through each column
        for (def columnIt = page.layoutComponents.iterator(); columnIt.hasNext(); ) {
            def column = columnIt.next();

            // Loop through each portlet in this column
            for (def portletIt = column.portlets.iterator(); portletIt.hasNext(); ) {
                def portlet = portletIt.next();

                // Delete the portlet
                portletIt.remove();
                deletePortlet(jomaWrapper, portlet, deletePortlets, log);
            }

            // Delete the column
            log("verbose", "Deleting PSColumnLayoutComponent '${column.name}'");
            columnIt.remove();
            column.delete();
        }

        // Delete the page
        log("verbose", "Deleting PSPortalPage '${page.name}'");
        page.delete();
    }

    def deletePortletOnPageTemplate(jomaWrapper, portlet, log) {
        // Delete the property sets
        deleteAllPropSets(portlet, log);

        // Loop through the members to look for a collection group
        for (def membersIt = portlet.members.iterator(); membersIt.hasNext(); ) {
            def member = membersIt.next();

            if (member.name.equals("Portal Collection")) {
                log("verbose", "Deleting Group '${member.name}'");
                membersIt.remove();
                member.delete();
            }
        }

        log("verbose", "Deleting Group (Page Template Portlet) '${portlet.name}'");
        portlet.delete();
    }

    def deletePageTemplate(jomaWrapper, pageTemplate, deletePages, log) {
        // Get all the extensions and remove them
        deleteAllExtensions(pageTemplate, log);

        // Loop through each column
        for (def columnIt = pageTemplate.subTrees.iterator(); columnIt.hasNext(); ) {
            def column = columnIt.next();

            // Get all the extensions and remove them
            deleteAllExtensions(column, log);

            // Loop through each portlet in this column
            for (def portletIt = column.members.iterator(); portletIt.hasNext(); ) {
                def portlet = portletIt.next();

                // Delete the portlet
                portletIt.remove();
                deletePortletOnPageTemplate(jomaWrapper, portlet, log);
            }

            // Delete the column
            log("verbose", "Deleting Tree (Page Template column) '${column.name}'");
            columnIt.remove();
            column.delete();
        }

        if (deletePages) {
            for (def pagesIt = pageTemplate.usedByPrototypes.iterator(); pagesIt.hasNext(); ) {
                def page = pagesIt.next();

                // Delete the page
                pagesIt.remove();
                deletePortalPage(jomaWrapper, page, true, log);
            }
        }

        // Delete the page
        log("verbose", "Deleting Tree (Page Template) '${pageTemplate.name}'");
        pageTemplate.delete();
    }
}

new PageHelper();

