
import com.sas.meta.SASOMI.IOMI;
import com.sas.meta.SASOMI.ISecurity;
import com.sas.meta.SASOMI.ISecurityAdmin;
import com.sas.meta.SASOMI.ISecurityAdminHelper;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMIUtil;


// Get the script attributes
def host = attributes.get("host");
def port = attributes.get("port");
def user = attributes.get("user");
def password = attributes.get("password");
def metadataObjectIdentifier = attributes.get("metadataobjectidentifier");
def ACTObjectIdentifier = attributes.get("actobjectidentifier");
def identityType = attributes.get("identitytype");
def identityName = attributes.get("identityname");
def permissionDirective = attributes.get("permissiondirective");
def permissionName = attributes.get("permissionname");
def permissionCondition = attributes.get("permissioncondition");
def rcPropName = attributes.get("resultproperty");
//
// since attributes extends java.util.HashMap I can use containsKey to check the list of attributes
def actobjectidentifierexists = attributes.containsKey("actobjectidentifier");
def identitytypeexists = attributes.containsKey("identitytype");

rc = 0;

ant = new AntBuilder();

try {
    // Connect to the metadata server
        self.log("Connecting to metadata server:");
        self.log("    Host: " + host);
        self.log("    Port: " + port);
        self.log("    User: " + user);

        //get a connection facotry
        def _factory = new MdFactoryImpl();
        //get a connection
        _factory.makeOMRConnection( host, port, user, password );
        //get an omiConnection
      	omiConnection = _factory.getConnection().getCMRHandle();
      	ISecurityAdmin iSecAdmin = ISecurityAdminHelper.narrow(omiConnection);
      	if (iSecAdmin == null) {
      		self.log("ERROR: could not implement ISecurityAdmin API");
      	}  
        
        self.log("Connected to metadata server");
          
        if (identitytypeexists)
        {
        
          def permssionDirectiveWord = "Grant";

          if (permissionDirective.equals("D"))
          {
            permssionDirectiveWord = "Deny";
          }

        
          self.log("Attempting to add ACE To Object: " + metadataObjectIdentifier);
          self.log("to "  + permssionDirectiveWord + " "  + identityType + ": "  + identityName + " " + permissionName  + "    Condition: " + permissionCondition);

          //Examples:
          // These are examples of initializing multi-dimensional arrays in groovy.  Currently only one permission at a time can be added
          // via ant but this example can serve as a guide if we want to expand the code later.
          //String[][] Authorizations = [[ "IdentityGroup", "PUBLIC", "D", "WriteMetadata", "" ],["IdentityGroup", "PUBLIC", "D", "CheckInMetadata", "" ]] as String[][];
          //String[][] Authorizations = [[ "IdentityGroup", "PUBLIC", "D", "WriteMetadata", "" ]] as String[][];

          // something else to try:
          // def Authorizations = (String[][])[[ "IdentityGroup", "PUBLIC", "D", "WriteMetadata", "" ]["IdentityGroup", "PUBLIC", "D", "CheckInMetadata", "" ]]; 

          String[][] Authorizations = [[ identityType, identityName, permissionDirective, permissionName, permissionCondition ]] as String[][];
      	
          iSecAdmin.SetAuthorizationsOnObj("", metadataObjectIdentifier, 0, Authorizations);
          self.log("Successfully Set Authorizations");
        }
        
        else if (actobjectidentifierexists)
        {
          self.log("Attempting to apply ACT "+ ACTObjectIdentifier + " To Object: " + metadataObjectIdentifier);
          iSecAdmin.ApplyACTToObj("", metadataObjectIdentifier, 0, ACTObjectIdentifier);
          self.log("Successfully Set Authorizations");
        }
        
        else self.log("Unable to Set Authorizations");


        // Disconnect from the metadata server
        _factory.dispose();


    }


    
    catch (Throwable t) {

        sw = new StringWriter();
        pw = new PrintWriter(sw);
        t.printStackTrace(pw);

        self.log(sw.toString());
        rc = 1;
    }

// Set the return code property


project.setNewProperty(rcPropName, "" + rc);
