useAnt = true;
if (useAnt) {
    // Get the script attributes
    hostStr = attributes.get("host");
    portStr = attributes.get("port");
    userStr = attributes.get("user");
    passwordStr = attributes.get("password");
    repositoryStr = attributes.get("repository");
    softwareComponent = attributes.get("softwarecomponent");
    name = attributes.get("name");
    oldList = attributes.get("oldlist");
    newList = attributes.get("newlist");
    fqidPropName = attributes.get("fqidpropname");
    oldValuePropName = attributes.get("oldvaluepropname");
    newValuePropName = attributes.get("newvaluepropname");
    rcPropName = attributes.get("resultsproperty");

    // Define the logger
    ant = new AntBuilder();
    logger = { level, msg ->
        ant.echo(level:level, message:msg);
    }

    // Get the utilities directory to locate JomaWrapper
    //utilsDir = project.getProperty("bsh.src.dir");
    utilsDir = project.getProperty("install.cfgwizard.utilities.dir");
} else {
    // Get the script attributes
    hostStr = args[0];
    portStr = args[1];
    userStr = args[2];
    passwordStr = args[3];
    repositoryStr = args[4];
    softwareComponent = args[5];
    name = args[6];
    oldList = args[7];
    newList = args[8];

    // Define the logger
    logger = { level, msg ->
        println(msg);
    }

    // Get the utilities directory to locate JomaWrapper
    utilsDir = ".";
}


def updatePropertyValue(jomaWrapper, softwareComponent, name, oldListStr, newListStr, log) {
    def rc = 0;

    property = jomaWrapper.getMetadataObject("Property", "@Name='$name'][AssociatedPropertySet/PropertySet" + 
                                                         "[@Name='Application Configuration']/OwningObject/SoftwareComponent" + 
                                                         "[@Name='$softwareComponent']");
    if (property != null) {
        // Get the lists assuming CSV
        def oldList = oldListStr.tokenize(",");
        def newList = newListStr.tokenize(",");

        fqid = property.id;
        oldValue = property.defaultValue;

        // Verify lists are same size
        if (oldList.size() == newList.size()) {
            // Loop through each item of the list
            for (i = 0; i < oldList.size(); i++) {
                // Update the property with the new value (if in the old string)
                property.defaultValue = property.defaultValue.replaceAll(oldList[i].replaceAll("\\\\", "\\\\\\\\"), newList[i].replaceAll("\\\\", "\\\\\\\\"));
            }
            property.updateMetadataAll();

            newValue = property.defaultValue;
        } else {
            rc = 3;
            log("verbose", "old and new list are different sizes (old:$oldListStr, new:$newListStr)");
        }
    } else {
        rc = 2;
        log("verbose", "Could not find property $name in SoftwareComponent $softwareComponent");
    }

    return rc;
}

rc = 0;
fqid = "";
oldValue = "";
newValue = "";

try {
    // Connect to the metadata server
    jomaWrapper = evaluate(new File("$utilsDir/JomaWrapper.groovy").getText("UTF-8"));
    jomaWrapper.connect(hostStr, 
                        portStr, 
                        userStr, 
                        passwordStr,
                        repositoryStr);

    rc = updatePropertyValue(jomaWrapper, softwareComponent, name, oldList, newList, logger);
    if (rc == 0) {
        logger("debug", "fqidPropName=$fqidPropName, fqid=$fqid");
        logger("debug", "oldValuePropName=$oldValuePropName, oldValue=$oldValue");
        logger("debug", "newValuePropName=$newValuePropName, newValue=$newValue");

        if (fqidPropName != null && fqidPropName.length() > 0) {
            project.setNewProperty(fqidPropName, fqid);
        }
        if (oldValuePropName != null && oldValuePropName.length() > 0) {
            project.setNewProperty(oldValuePropName, oldValue);
        }
        if (newValuePropName != null && newValuePropName.length() > 0) {
            project.setNewProperty(newValuePropName, newValue);
        }
    }

    // Disconnect from the metadata server
    jomaWrapper.disconnect();
} catch (Throwable t) {
    sw = new StringWriter();
    pw = new PrintWriter(sw);
    t.printStackTrace(pw);
    logger("info", sw.toString());
    rc = 1;
}

if (useAnt) {
    // Set the return code property
    project.setNewProperty(rcPropName, "" + rc);
} else {
    logger("info", "rc=$rc");
    logger("info", "oldValue=$oldValue");
    logger("info", "newValue=$newValue");
}

