import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.CRC32;
import java.util.zip.Checksum;

// Set up return code
rc = 0;
String resultProperty = "changeHostNameRC";
Boolean debug = false;

// get attributes
String a_property  = attributes["property"];
String a_docpart   = attributes["docpart"];
String a_configobj = attributes["configobj"];
String a_file      = attributes["file"];
String a_pretoken  = attributes["pretoken"];
String a_posttoken = attributes["posttoken"];
String a_encoding  = attributes["encoding"];
String a_flags     = attributes["flags"]; //obsolete

// get elements
e_fileset = elements["fileset"];
e_propertyset = elements["propertyset"];

if (debug) {
   self.log("* a_property: " + a_property);
   self.log("* a_docpart: " + a_docpart);
   self.log("* a_configobj: " + a_configobj);
   self.log("* a_file: " + a_file);
   self.log("* a_pretoken: " + a_pretoken);
   self.log("* a_posttoken: " + a_posttoken);
   self.log("* a_encoding: " + a_encoding);
   self.log("* a_flags: " + a_flags);
   self.log("* e_fileset: " + e_fileset);
   self.log("* e_propertyset: " + e_propertyset);
}

// get messages
String m_invprop = project.getProperty("status.changeHostName.invalid.property.msg");
String m_hostvalues = project.getProperty("status.changeHostName.hostvalues.msg");
String m_invtask = project.getProperty("status.changeHostName.invalid.task.msg");
String m_oldhnf = project.getProperty("status.changeHostName.oldhost.notfound.msg");
String m_newhnf = project.getProperty("status.changeHostName.newhost.notfound.msg");
String m_noupdate = project.getProperty("status.changeHostName.noupdate.msg");
String m_loop = project.getProperty("status.changeHostName.loop.msg");

if (debug) {
   self.log("* m_invprop: " + m_invprop);
   self.log("* m_hostvalues: " + m_hostvalues);
   self.log("* m_invtask: " + m_invtask);
   self.log("* m_oldhnf: " + m_oldhnf);
   self.log("* m_newhnf: " + m_newhnf);
   self.log("* m_noupdate: " + m_noupdate);
   self.log("* m_loop: " + m_loop);
}

try {
   task = "";

   ostype = project.getProperty("os.localhost.type");
   if (ostype == null || ostype.length() == 0) {
      ostype == "unknown";
      self.log("The operating system type could not be determined. This may yield unexpected results.");
   }

   // change host property in configurable object
   if (a_property == null || a_property.length() == 0)
      a_property = "";
   else
      task = "property";

   // propertyset
   if (null != e_propertyset)
      task = task + ":propset";

   if ((task =~ "property") && (task =~ "propset"))
      throw new Exception(m_invprop);

   if (((task =~ "property") || (task =~ "propset")) && (a_configobj == null || a_configobj.length() == 0))
      a_configobj = project.getProperty("config.currprod.12byte");

   // change host in file
   if (a_file == null || a_file.length() == 0)
      a_file = "";
   else {
      task = task + ":file";
      file = new File(a_file);
   }

   if (((task =~ "file") || (task =~ "property")) && (a_pretoken == null))
      a_pretoken = "";

   if (((task =~ "file") || (task =~ "property")) && (a_posttoken == null))
      a_posttoken = "";

   // determine file encoding to use
   if ((task =~ "file") && (a_encoding == null || a_encoding.length() == 0))
      encoding = g_getEncoding(a_file);
   else
      encoding = a_encoding;

   // *** the flags option is now obsolete after removing the replaceregexp task
   if (null != a_flags)
      self.log("The flags attribute on changeHostName is obsolete. This attribute will be ignored: flags=" + a_flags);

   // report hosts in loose files
   if (null != e_fileset)
      task = "report";

   // use docpart if specified
   if (a_docpart == null)
      a_docpart = "";

   // get host name property values from sdm
   String p_oldhost = project.getProperty("deploymgr.changehost.old.host.name");
   String p_newhost = project.getProperty("deploymgr.changehost.new.host.name");
   if (null == p_oldhost || null == p_newhost || p_oldhost.length() == 0 || p_newhost.length() == 0)
      throw new Exception(m_hostvalues);

   // set docpart header flags
   Boolean objChanged = false;
   Boolean hostFoundInFile = false;
   Boolean fileChanged = false;

   // pattern match strings
   String prefixPattern = "";
   String suffixPattern = "";

   // tokenize host lists
   newHostList = new StringTokenizer(p_newhost, ",");
   oldHostList = new StringTokenizer(p_oldhost, ",");

   // review our task
   if (((task =~ "property") || (task =~ "propset")) && (task =~ "file"))
      self.log("Change host names found in properties and files.");
   else if (task =~ "file")
      self.log("Change host names found in files.");
   else if ((task =~ "property") || (task =~ "propset"))
      self.log("Change host names found in properties.");
   else if (task =~ "report")
      self.log("Report host names found in miscellaneous files.");
   else
      throw new Exception(m_invtask);

   // process the host pairs
   while ((oldHostList.hasMoreTokens()) && (newHostList.hasMoreTokens())) {
      String oldHostItem = oldHostList.nextToken().trim();
      String newHostItem = newHostList.nextToken().trim();

      // set pattern matching defaults
      if ((a_pretoken == null) || (a_pretoken.length() == 0))
         prefixPattern = "(^|[^-0-9a-zA-Z.])";
      else
         prefixPattern = "(" + java.util.regex.Pattern.quote(a_pretoken) + ")";

      if ((a_posttoken == null) || (a_posttoken.length() == 0))
         suffixPattern = "((?!\\.?[-0-9a-zA-Z]))"; // the host name must be followed by a terminating period (e.g., at the end of a sentence) or by a non-fqdn character
      else
         suffixPattern = "(" + java.util.regex.Pattern.quote(a_posttoken) + ")";

      Pattern matchPattern = Pattern.compile("(?i)" + prefixPattern + java.util.regex.Pattern.quote(oldHostItem) + suffixPattern);
      if (debug) {
         self.log("* matchPattern: " + matchPattern.toString());
      }

      // make sure we have host values
      if (null == oldHostItem || oldHostItem.length() == 0)
         throw new Exception(m_oldhnf);
      else if (null == newHostItem || newHostItem.length() == 0)
         throw new Exception(m_newhnf);

      if (debug) {
         self.log("* Old Host Name: " + oldHostItem);
         self.log("* New Host Name: " + newHostItem);
      }

      // replace host in a file
      if (task =~ "file") {
         String searchValue = a_pretoken + oldHostItem + a_posttoken;

         self.log("-Search for: " + searchValue + " in file: " + a_file + " Encoding: " + encoding);

         int matchLine = g_match(true, file, matchPattern, encoding);
         int lasMatchLine = -1;
         if (debug) {
            self.log("* matchLine: " + matchLine);
            self.log("* lasMatchLine: " + lasMatchLine);
         }

         if (matchLine > 0) {
            if (file.canWrite()) {
               long lastcksum = 0;
               long cksum = g_checkSum(file);
               boolean writefail = false;

               int matchOffset = g_match(false, file, matchPattern, encoding);
               int lastMatchOffset = -1;
               if (debug) {
                  self.log("* matchLine: " + matchLine);
                  self.log("* matchOffset: " + matchOffset);
                  self.log("* lastMatchOffset: " + lastMatchOffset);
                  self.log("* cksum: " + cksum);
                  self.log("* lastcksum: " + lastcksum);
               }

               // loop over each match found
               while ((matchLine > 0) && (lastcksum != cksum) && (matchOffset != lastMatchOffset) && (!writefail)) {
                  self.log("--Found match at line " + matchLine + " offset " + matchOffset);
                  lastcksum = cksum;

                  // make a backup file
                  def orig_content = file.getText(encoding);
                  File bakfile = new File(a_file + ".bak");
                  bakfile.write(orig_content, encoding);

                  FileOutputStream origfos = new FileOutputStream(file, false); // true to append; false to overwrite

                  try {
                     // write new content to the original file
                     byte[] newBytes = orig_content.replaceFirst(matchPattern.toString(), "\$1" + newHostItem + "\$2").getBytes(encoding);
                     origfos.write(newBytes);
                     origfos.close();
                  }
                  catch(Throwable t) {
                     origfos.close();
                     file.delete();
                     bakfile.renameTo(file);
                     writefail = true;
                  }
                  finally {
                     bakfile.delete();
                  }

                  cksum = g_checkSum(file);

                  // write docpart
                  if (!writefail) {
                     g_insertDocpart("files", a_docpart, oldHostItem, newHostItem, a_file, matchLine);
                     fileChanged = true;
                  }

                  // find more matches
                  lasMatchLine = matchLine;
                  matchLine = g_match(true, file, matchPattern, encoding);

                  if (matchLine == lasMatchLine)
                     lastMatchOffset = matchOffset;
                  else
                     lastMatchOffset = -1;
                  matchOffset = g_match(false, file, matchPattern, encoding);

                  if (debug) {
                     self.log("* matchLine: " + matchLine);
                     self.log("* lasMatchLine: " + lasMatchLine);
                     self.log("* matchOffset: " + matchOffset);
                     self.log("* lastMatchOffset: " + lastMatchOffset);
                  }
               }

               // throw a warning if the write failed
               if (writefail) {
                  self.log("Warning: " + m_noupdate);
                  g_issueWarning("files", "changeHost_files_warning_item.txt", m_noupdate, oldHostItem, newHostItem, a_file, m_noupdate);
               }

               // throw a warning if the file didn't acutally change
               else if (lastcksum == cksum) {
                  self.log("Warning: " + m_noupdate);
                  g_issueWarning("files", "changeHost_files_warning_item.txt", m_noupdate, oldHostItem, newHostItem, a_file, m_noupdate);
               }

               // throw a loop warning if the we found too many matches on a single line
               else if ((matchOffset == lastMatchOffset) && (matchOffset >= 0)) {
                  self.log("Warning: " + m_loop);
                  g_issueWarning("files", "changeHost_files_warning_item.txt", m_loop, oldHostItem, newHostItem, a_file, m_loop);
               }
            }
            // throw a warning if the file is not writable
            else {
               self.log("Warning: " + m_noupdate);
               g_issueWarning("files", "changeHost_files_warning_item.txt", m_noupdate, oldHostItem, newHostItem, a_file, m_noupdate);
            }
         }
         else {
            if (debug)
               self.log("* --No match found");
         }
      } // file

      // replace host in a configurable object property
      if (task =~ "property") {
         value = project.getProperty(a_property);

         String searchValue = a_pretoken + oldHostItem + a_posttoken;
         self.log("-Search for: " + searchValue + " in property: " + a_property + " with current value: " + value);

         if (value != null) {
            Matcher matcher = matchPattern.matcher(value);
            newValue = value;

            while (matcher.find()) {
               self.log("--Found match.");

               newValue = newValue.replaceFirst(matchPattern.toString(), "\$1" + newHostItem + "\$2");

               // set the new property value for a later call to setObjectProperties
               project.setProperty(a_property, newValue);
               g_insertDocpart("configobj", a_docpart, value, newValue, a_property, "");
               if (debug)
                  self.log("---Set property " + a_property + " from " + value + " to " + newValue);

               objChanged = true;
            }
         }

      } // property

      // change host name in a propertyset
      if (task =~ "propset") {
         // iterate over all propertysets
         for (Iterator i = e_propertyset.iterator(); i.hasNext();) {
            pset = i.next();
            //return properties object for this propertyset
            psetProps = pset.getProperties();
            Enumeration psetEnum = psetProps.keys();

            while (psetEnum.hasMoreElements()) {
               String key = psetEnum.nextElement();
               String value = psetProps.getProperty(key);

               self.log("-Search for: " + oldHostItem + " in property: " + key + " with current value: " + value);

               if (value != null) {
                  Matcher matcher = matchPattern.matcher(value);
                  newValue = value;

                  while (matcher.find()) {
                     self.log("--Found match.");

                     newValue = newValue.replaceFirst(matchPattern.toString(), "\$1" + newHostItem + "\$2");

                     // set the new property value for a later call to setObjectProperties
                     project.setProperty(key, newValue);
                     g_insertDocpart("configobj", a_docpart, value, newValue, key, "");
                     if (debug)
                        self.log("---Set property " + key + " from " + value + " to " + newValue);

                     objChanged = true;
                  }
               }

            }
         }
      } // propset

      // report host names found in other files
      if (task =~ "report") {
         self.log("-Search for: " + oldHostItem);

         for (Iterator i = e_fileset.iterator(); i.hasNext();) {
            fs = i.next();

            basedir = fs.getDir(project);
            incFiles = fs.getDirectoryScanner(project).getIncludedFiles();
            for (int j = 0; j < incFiles.length; j++) {
               String fsItem = incFiles[j];

               try {
                  basefile = new File(basedir, fsItem);
                  encoding = g_getEncoding(fsItem);
                  isrfile = new InputStreamReader(new FileInputStream(basefile), encoding);
                  self.log("--Checking file: " + basefile + " Encoding: " + encoding);

                  // match on pattern
                  int line = 0;
                  int offset = 0;
                  boolean setFound = false;
                  String match = "";

                  isrfile.eachLine {
                     line += 1;
                     Matcher matcher = matchPattern.matcher(it);

                     // loop over each line to find all matches
                     while(matcher.find(offset)) {
                        offset = matcher.start() + 1;
                        self.log("---Found match at line " + line + " offset " + offset + ": " + it);

                        // write the file where a match was found in first
                        if (! setFound) {
                           docpart = "changeHost_reported_list.txt";
                           g_insertDocpart("reported", docpart, oldHostItem, newHostItem, basefile.toString(), it);
                        }
                        // write the matched text (less any "<" which messes up the html)
                        def newIt = (it =~ /</).replaceAll("&lt;")
                        setFound = true;
                        g_insertDocpart("reported", a_docpart, oldHostItem, newHostItem, line.toString(), newIt.trim());
                        hostFoundInFile = true;
                     }
                     offset = 0;
                  }
                  isrfile.close();
               }
               catch (Throwable t) {
                  self.log("---Skipping file due to exception: " + t.toString());
               }
            }
         }
      } // report
   }

   if (task =~ "file")
      g_writeDocpartHeader("files");

   if ((task =~ "property") || (task =~ "propset")) {
      if (objChanged) {
         self.log("-Updating the configurable object.");
         // call SetObjectProperties task
         update = project.createTask("updateConfigObj");
         update.setDynamicAttribute("configobj", a_configobj);
         update.perform();
      }

      g_writeDocpartHeader("configobj");
   }

   if (task =~ "report")
      g_writeDocpartHeader("reported");
}
catch (Throwable t) {
   t.printStackTrace();

   if (rc == 0)
      rc = 1;
   project.setProperty(resultProperty, "" + rc);
   self.log("Return code: " + rc);

   org.apache.tools.ant.Task exittask = project.createTask("fail");
   exittask.setMessage("Exiting due to exception: " + t.toString());
   exittask.execute();
   return;
}


// search for a pattern in a file
int g_match(flag, filename, pattern, encoding) {
   Boolean _match = false;
   int _line = 0;
   int _offset = 0;

   try {
      _isrfile = new InputStreamReader(new FileInputStream(filename), encoding);

      // match on pattern
      _isrfile.eachLine {
         Matcher _matcher = pattern.matcher(it);

         if (! _match)
            _line+=1;

         if (_matcher.find() && (!_match)) {
            _match = true;
            _offset = _matcher.start() + 1;
         }
      }
      _isrfile.close();

      if (_match)
         if(flag)
            return _line;
         else
            return _offset;
      else
         return -1;
   }
   catch (Throwable t) {
      return -1;
   }
}


// determine file encoding
// note that this code emulates the encoding settings used in copyOrigFiles.
// any changes made to this should be reflected in copyOrigFiles as well.
def g_getEncoding(filename) {
   try {
      // default to file.encoding
      _encoding = project.getProperty("file.encoding");

      if ((filename.endsWith(".css")) || (filename.endsWith(".policy")))
         _encoding = "UTF8";

      if (ostype.equals("zos")) {
         if ((filename.endsWith(".txt")) ||
             (filename.endsWith(".sas")) ||
             (filename.endsWith(".cfg")))
         _encoding = project.getProperty("zos.sasinputs.encoding");
      }
      else {
         if ((filename.endsWith(".xmi")) ||
             (filename.endsWith(".xml")) ||
             (filename.endsWith(".xml.sample")) ||
             (filename.endsWith(".crh")) ||
             (filename.endsWith(".cth")) ||
             (filename.endsWith(".dsx")) ||
             (filename.endsWith(".etx")) ||
             (filename.endsWith(".gdx")) ||
             (filename.endsWith(".ivd")) ||
             (filename.endsWith(".phx")) ||
             (filename.endsWith(".wsdl")))
         _encoding = "UTF8";
      }

      return _encoding;
   }
   catch (Throwable t) {
      _encoding = project.getProperty("file.encoding");
      return _encoding;
   }
}


// insert docpart
void g_insertDocpart(name, docpart, oldhost, newhost, param, param2) {
   try {
      if (docpart.length() == 0)
         docpart = "changeHost_" + name + "_item.txt";

      insert = project.createTask("addDocpart");
      insert.setDynamicAttribute("name", name);
      insert.setDynamicAttribute("docpart", docpart);
      insert.setDynamicAttribute("oldhost", oldhost);
      insert.setDynamicAttribute("newhost", newhost);
      insert.setDynamicAttribute("param", param.toString());
      insert.setDynamicAttribute("param2", param2.toString());
      insert.perform();
   }
   catch (Throwable t) {
      self.log("Error writing docpart: " + t.toString());
   }
}


// write docpart header
void g_writeDocpartHeader(name) {
   try {
      header = project.createTask("addDocpartHeader");
      header.setDynamicAttribute("name", name);
      header.perform();
   }
   catch (Throwable t) {
      self.log("Error writing docpart header: " + t.toString());
   }
}


// get a file checksum
def g_checkSum(filename) {
   try {
      Checksum checksum = new CRC32();

      String text = file.getText();
      byte[] bytes = text.getBytes();

      checksum.update(bytes);

      long lngChecksum = checksum.getValue();
      return lngChecksum;
   }
   catch (Throwable t) {
      return 0;
   }
}


// issue a task warning
void g_issueWarning(task, docpart, message, oldhost, newhost, param1, param2) {
   self.log("Set warning status: " + message);
   statusProps = new Properties();
   statusProps.setProperty("warningsEncountered", "true");
   statusProps.setProperty("error.msg", message);
   statusFileName = project.getProperty("whatFailed.property.file");
   statusFile = new File(statusFileName);
   statusFOS = new FileOutputStream(statusFile);
   statusProps.store(statusFOS, null);
   statusFOS.close();
   g_insertDocpart(task, docpart, oldhost, newhost, param1, param2);
}


// Set the return code property
project.setProperty(resultProperty, "" + rc);
self.log("rc=" + rc);
