class JomaWrapperInternal {
    def ant = new AntBuilder();
    def host = "";
    def port = "";
    def user = "";
    def password = "";
    def reposFQID = null;
    def reposId = null;
    def objStore = null;
    
    def mdFactory = null;
    def mdOMIUtil = null;

    void connect(host, port, user, password) {
        this.host = host;
        this.port = port;
        this.user = user;
        this.password = password;

        echo("verbose", "Connecting to metadata server:");
        echo("verbose", "    Host: " + host);
        echo("verbose", "    Port: " + port);
        echo("verbose", "    User: " + user);

        mdFactory = new com.sas.metadata.remote.MdFactoryImpl(false);

        // Connect to the metadata server
        com.sas.metadata.remote.MdOMRConnection connection = mdFactory.getConnection();
        connection.makeOMRConnection(host, port, user, password);
        
        // Create the joma object store
        objStore = mdFactory.createObjectStore();

        echo("verbose", "Connected to metadata server");
        
        mdOMIUtil = mdFactory.getOMIUtil();

        // Get the repository information
        reposFQID = mdOMIUtil.getFoundationReposID();
       	reposId = reposFQID.substring(9);
        
    }

    void disconnect() {
        // Disconnect from the metadata server
        objStore.dispose();
        mdFactory.closeOMRConnection();
    }

    def getMetadataObjects(type, search) {
        def xmlSelect = "<XMLSelect search=\"" + type + "[" + search + "]\" />";

        echo("verbose", "Searching for metadata");
        echo("verbose", "    Type: " + type);
        echo("verbose", "    Search: " + xmlSelect);

        def objects = mdOMIUtil.getMetadataObjectsSubset(objStore, 
                                                            reposFQID,
                                                            type,
                                                            com.sas.metadata.remote.MdOMIUtil.OMI_XMLSELECT |
                                                            com.sas.metadata.remote.MdOMIUtil.OMI_ALL_SIMPLE |
                                                            com.sas.metadata.remote.MdOMIUtil.OMI_TEMPLATE |
                                                            com.sas.metadata.remote.MdOMIUtil.OMI_GET_METADATA,
                                                            xmlSelect);

        echo("verbose", "Found metadata count: " + objects.size());
        for (def it = objects.iterator(); it.hasNext(); ) {
            def obj = it.next();
            echo("verbose", "    Name: " + obj.getName());
        }

        return objects;
    }

    def getMetadataObject(type, searchStr) {
        def retval = null;
    
        def objects = getMetadataObjects(type, searchStr);
        if (objects != null && objects.size() > 0) {
            retval = objects.get(0);
        }
    
        return retval;
    }

    // Calls the ant echo task with the specified level and message
    def echo(level, msg) {
        ant.echo(level:level, message:msg);
    }
}

/**
 * PWCBHandler sets the password for a given user when invoked by apache's
 * Rampart. This handler class sets the password on the apache Callback when
 * it's invoked and simply sets that value to whatever password was given to
 * construct the PWCBHandler assuming the usernames match.
 * 
 * @author zamars
*/
class PWCBHandler
		implements javax.security.auth.callback.CallbackHandler {
	
	private String	_username, _password;
	public PWCBHandler(String username, String password) {
		_username = username;
		_password = password;
	}
	/**
	 * Set the credentials on the Callback.
	 */
	public void handle(javax.security.auth.callback.Callback[] callbacks) throws IOException, javax.security.auth.callback.UnsupportedCallbackException {
		for (int i = 0; i < callbacks.length; i++) {
			org.apache.ws.security.WSPasswordCallback pwcb = (org.apache.ws.security.WSPasswordCallback)callbacks[i];
			if (pwcb.getIdentifer().equals(_username)) {
				pwcb.setPassword(_password);
				break;
			}
		}
	}
}

/**
 * GroovyWebServicesClient is a wrapper for an apache axis2 web service client that
 * encapsulates certain properties and behaviors that are common across SAS BI
 * Web Services. This class handles setting security options and ws-addressing
 * settings.
 * 
 * @author zamars
 */
class GroovyWebServicesClient
		extends org.apache.axis2.client.ServiceClient {

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2
	 */
	public static final String	WEB_SERVICES_NS_92			= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2";

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/ListWebServices
	 */
	public static final String	LIST_WEB_SERVICES_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/ListWebServices";

	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/MakeWebService
	 */
	public static final String	MAKE_WEB_SERVICE_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/MakeWebService";
	
	/**
	 * http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/RemoveWebService
	 */
	public static final String	REMOVE_WEB_SERVICE_ACTION_92	= "http://support.sas.com/xml/namespace/biwebservices/webservicemaker-9.2/RemoveWebService";

	/**
	 * Generates the outflow configuration parameter for the service client. The
	 * outflow configuration controls how security for the message is handled as
	 * it is leaving the apache axis2 context. Here, we say add a username token
	 * in plain text and add a timestamp.
	 * 
	 * @param uName
	 *            The username that has permission to make this web service
	 *            call.
	 * @return The OutflowConfiguration Parameter.
	 */
	private org.apache.axis2.description.Parameter getOutflowConfiguration(String uName) {
		org.apache.rampart.handler.config.OutflowConfiguration outflow = new org.apache.rampart.handler.config.OutflowConfiguration();
		outflow.setActionItems("UsernameToken Timestamp");
		outflow.setUser(uName);
		outflow.setPasswordType("PasswordText");
		return outflow.getProperty();
	}

	private org.apache.axis2.client.Options	_options;

	/**
	 * Sets options on the client to commonly-used values.
	 * 
	 * @param endpoint
	 *            The endpoint of the service we're connecting to.
	 * @param action
	 *            The web service action to perform.
	 * @param user
	 *            The user who has permission to perform this web service call.
	 * @param password
	 *            The password associated with the user.
	 * @param timeoutInMillis
	 *            The socket and client connection time to wait before failing
	 *            in milliseconds.
	 * @throws AxisFault
	 *             If there is a problem engaging WS-Addressing or WS-Security
	 *             modules.
	 */
	private void setOptions(String endpoint, String action, String user, String password, int timeoutInMillis) throws org.apache.axis2.AxisFault {
		Integer timeout = new Integer(timeoutInMillis);
		_options = (this.getOptions() == null ? new org.apache.axis2.client.Options()
				: this.getOptions());
		_options.setTo(new org.apache.axis2.addressing.EndpointReference(endpoint));
		_options.setAction(action);
		_options.setTransportInProtocol(org.apache.axis2.Constants.TRANSPORT_HTTP);
		_options.setSoapVersionURI(org.apache.axis2.Constants.URI_SOAP11_ENV);
		// turn off chunking to see if it fixes a .NET problem
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.CHUNKED, "false");
		_options.setProperty(org.apache.axis2.Constants.Configuration.ENABLE_MTOM, org.apache.axis2.Constants.VALUE_FALSE);
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.SO_TIMEOUT, timeout);
		_options.setProperty(org.apache.axis2.transport.http.HTTPConstants.CONNECTION_TIMEOUT, timeout);
		_options.setProperty(org.apache.axis2.description.WSDL2Constants.ATTRIBUTE_MUST_UNDERSTAND, "0");
		this.setOptions(_options);
		
		if (user != null && password != null) {
			// fix for .NET axis incompatibility (different year ns)
			_options.setProperty(org.apache.axis2.addressing.AddressingConstants.WS_ADDRESSING_VERSION, org.apache.axis2.addressing.AddressingConstants.Submission.WSA_NAMESPACE);
			_options.setProperty(org.apache.ws.security.handler.WSHandlerConstants.PW_CALLBACK_REF, new PWCBHandler(user, password));
			_options.setProperty(org.apache.ws.security.handler.WSHandlerConstants.USER, user);
			_options.setProperty(org.apache.rampart.handler.WSSHandlerConstants.OUTFLOW_SECURITY, getOutflowConfiguration(user));
			this.engageModule(new javax.xml.namespace.QName(org.apache.axis2.Constants.MODULE_ADDRESSING));
			this.engageModule(new javax.xml.namespace.QName("rampart"));
		}
		
	}

	/**
	 * Construct a GroovyWebServicesClient.
	 * 
	 * @param endpoint
	 *            The endpoint of the web service to connect to.
	 * @param action
	 *            The web service action to perform.
	 * @param user
	 *            The user who has permission to perform this web service call.
	 * @param password
	 *            The password associated with the user.
	 * @throws AxisFault
	 *             If there are communication problems or if there is a SOAP
	 *             fault.
	 */
	public GroovyWebServicesClient(URL endpoint, String action, String user, String password) throws org.apache.axis2.AxisFault {
		super();
		setOptions(endpoint.toExternalForm(), action, user, password, 900000);
	}

}

class Credentials {
	public String username;
	public String password;
}

class GroovyWebServiceMaker {
	public static final String METADATA_NAME = "WebServiceMaker";
	public static final String REGISTERED_SAS_APPLICATION_IDENTIFIER = "Registered SAS Application";
	public static final String BIWEBSERVICES_DOTNET_IDENTIFIER = "BI Web Services for .NET 9.2";
	public static final String BIWEBSERVICES_JAVA_IDENTIFIER = "BI Web Services for Java 9.2";
	
	protected com.sas.metadata.remote.DeployedComponent deployedComponent;
	protected URL endpoint;
	
	public GroovyWebServiceMaker(com.sas.metadata.remote.DeployedComponent deployedComponent) {
		this.deployedComponent = deployedComponent;
	}
	
	public GroovyWebServiceMaker(URL endpoint) {
		this.endpoint = endpoint;
	}
	
	public URL getEndpoint() throws java.rmi.RemoteException, com.sas.metadata.remote.MdException {
		if (endpoint == null && deployedComponent != null) {
			// see if we have a connection on this maker here, if so, use that
			com.sas.metadata.remote.TCPIPConnection tcpipConnection = null;
			boolean useJavaInference = false;
			int numConnections = deployedComponent.getSourceConnections().size();
			if (deployedComponent.getSourceConnections() != null && numConnections > 0) {
				for (Iterator it = deployedComponent.getSourceConnections().iterator(); it.hasNext();) {
					Object connection = it.next();
					if (connection instanceof com.sas.metadata.remote.TCPIPConnection) {
						tcpipConnection = (com.sas.metadata.remote.TCPIPConnection)connection;
						break;
					}
				}
			} else {
				// no source connection here, so we'll need to infer that from the Registered SAS Application DeployedComponent
				com.sas.metadata.remote.SoftwareComponent parentSC = deployedComponent.getDescriptiveComponent();
				if (parentSC != null && parentSC.getName().equals(BIWEBSERVICES_JAVA_IDENTIFIER)) { //ApplicationKeys.BI_WEB_SERVICES, "SAS BI Web Services for Java 9.2"
					for (Iterator it = parentSC.getDeployedComponents().iterator(); it.hasNext();) {
						com.sas.metadata.remote.DeployedComponent dc = (com.sas.metadata.remote.DeployedComponent)it.next();
						if (dc.getName().equals(REGISTERED_SAS_APPLICATION_IDENTIFIER)) {
							if (dc.getSourceConnections() != null) {
								for (Iterator scIt = dc.getSourceConnections().iterator(); it.hasNext();) {
									Object connection = scIt.next();
									if (connection instanceof com.sas.metadata.remote.TCPIPConnection) {
										tcpipConnection = (com.sas.metadata.remote.TCPIPConnection)connection;
										useJavaInference = true;
									}
									break;
								}
							}
							if (tcpipConnection != null)
								break;
						}
					}
				}
			}
			if (tcpipConnection != null) {
				try {
					String path = tcpipConnection.getService();
					if (useJavaInference) {
						if (!path.endsWith("/"))
							path += "/";
						path += "services/WebServiceMaker";
					}
					endpoint = new URL(tcpipConnection.getCommunicationProtocol(), tcpipConnection.getHostName(), tcpipConnection.getPort(), path);
				} catch (MalformedURLException e) {}
			}
		}
		return endpoint;
	}
	
	/**
	 * Delete a service using the supplied metadata credentials.
	 * @param serviceName The name of the service to delete.
	 * @param username The username that will be authenticated in metadata.
	 * @param password The password associated with the username.
	 * @throws MdException 
	 * @throws RemoteException 
	 */
	public void deleteService(String serviceName, String username, String password) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault {
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.REMOVE_WEB_SERVICE_ACTION_92, username, password);
		org.apache.axiom.om.OMElement payload = GetRemoveWebServiceElement(serviceName);
		client.sendReceive(payload);
	}
	
	/**
	 * Delete a service using the userid and password of the current user context. This method uses a OTP.
	 * @param serviceName The name of the service to delete.
	 * @throws ServiceException
	 * @throws RemoteException
	 * @throws MdException 
	 */
	public void deleteService(String serviceName, com.sas.services.user.UserContextInterface userContext) throws com.sas.services.ServiceException, java.rmi.RemoteException, com.sas.metadata.remote.MdException {
		Credentials creds = GetOTP(userContext);
		deleteService(serviceName, creds.username, creds.password);
	}
	
	public URL makeWebService(String webServiceName, String[] stps, String namespace, String[] keywords, String username, String password, boolean createMetadata) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault, MalformedURLException {
		if (stps == null || stps.length < 1)
			throw new IllegalArgumentException("You must include stored processes to create a web service");
		
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.MAKE_WEB_SERVICE_ACTION_92, username, password);
		org.apache.axiom.om.OMElement payload = getMakeWebServiceElement(webServiceName, stps, namespace, keywords, createMetadata);
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		if(result != null)
		{
			org.apache.axiom.om.OMElement elem = (org.apache.axiom.om.OMElement)result.getFirstElement();
							    				
			String wsUrl = elem.getText();
			
			if((wsUrl != null) && (wsUrl.length() > 0))
			{
				return new URL(wsUrl);
			}
			else
			{
				// no axis fault, but no URL returned
				// probably from Java in Directory mode which will not deploy the new service but instead dump it
				// to a directory. so return null here and let the caller decide what to do
				return null;
			}
		} else {
			throw new org.apache.axis2.AxisFault("Web Service Maker did not return a result");
		}
	}
	
	public URL makeWebService(String webServiceName, String[] stps, String namespace, String[] keywords, com.sas.services.user.UserContextInterface userContext, boolean createMetadata) throws java.rmi.RemoteException, com.sas.metadata.remote.MdException, org.apache.axis2.AxisFault, MalformedURLException {
		if (stps == null || stps.length < 1)
			throw new IllegalArgumentException("You must include stored processes to create a web service");
		
		Credentials creds = GetOTP(userContext);
		return makeWebService(webServiceName, stps, namespace, keywords, creds.username, creds.password, createMetadata);
	}
	
	public List listWebServices(com.sas.services.user.UserContextInterface userContext) throws org.apache.axis2.AxisFault, java.rmi.RemoteException, com.sas.metadata.remote.MdException, com.sas.services.ServiceException {
		List retList = new ArrayList();
		Credentials creds = GetOTP(userContext);
		
		GroovyWebServicesClient client = new GroovyWebServicesClient(getEndpoint(), GroovyWebServicesClient.LIST_WEB_SERVICES_ACTION_92, creds.username, creds.password);
		org.apache.axiom.om.OMElement payload = getListWebServicesElement();
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		if(result != null)
		{
			org.apache.axiom.om.OMElement listResult = (org.apache.axiom.om.OMElement) result.getFirstElement();
			for(Iterator children = listResult.getChildren(); children.hasNext();)
			{
				org.apache.axiom.om.OMElement elem = (org.apache.axiom.om.OMElement)children.next();
				if(elem.getLocalName().equals("string"))
				{
					String serviceName = elem.getText();
					retList.add(serviceName);
				}
			}
		}
		return retList;
	}
	
	public static org.apache.axiom.om.OMElement GetRemoveWebServiceElement(String serviceName) {
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("RemoveWebService", omNs);
        org.apache.axiom.om.OMElement serviceNameEl = fac.createOMElement("serviceName", omNs);
        org.apache.axiom.om.OMText serviceNameText = fac.createOMText(serviceNameEl, serviceName);
        serviceNameEl.addChild(serviceNameText);
        method.addChild(serviceNameEl);
        return method;
	}
	
	protected static Credentials GetOTP(com.sas.services.user.UserContextInterface userContext) throws com.sas.services.ServiceException, java.rmi.RemoteException {
		Credentials creds = new Credentials();
		com.sas.services.information.ServerInterface authServer = userContext.getAuthServer();
		com.sas.services.user.UserIdentityInterface id = authServer.generatePassword(null);
		creds.username = id.getPrincipal().toString();
		creds.password = id.getCredential().toString();
		return creds;
	}
	
	/**
	 * Creates a <MakeWebService> element for the WebServiceMaker that is used when Finish is pressed in the wizard.
	 * @return
	 */
	public static org.apache.axiom.om.OMElement getMakeWebServiceElement(String webServiceName, String[] stps, String namespace, String[] keywords, boolean createMetadata)
	{
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("MakeWebService", omNs);
        org.apache.axiom.om.OMElement stringElement = null;
        org.apache.axiom.om.OMText stringElementText = null;
        
        // Create storedProcessPaths elements
        org.apache.axiom.om.OMElement storedProcessPaths = fac.createOMElement("storedProcessPaths", omNs);
        for(int i=0; i<stps.length; i++)
        {
        	stringElement = fac.createOMElement("string", omNs);
        	stringElementText = fac.createOMText(stringElement, stps[i]);
        	stringElementText.setOptimize(false);
            stringElement.addChild(stringElementText);
        	storedProcessPaths.addChild(stringElement);
        }
        method.addChild(storedProcessPaths);
        
        // Create serviceName element
        org.apache.axiom.om.OMElement serviceName = fac.createOMElement("serviceName", omNs);
        org.apache.axiom.om.OMText serviceNameText = fac.createOMText(serviceName, webServiceName);
        serviceNameText.setOptimize(false);
        serviceName.addChild(serviceNameText);
        method.addChild(serviceName);
        
        // Create serviceNamespace element
        org.apache.axiom.om.OMElement serviceNamespace = fac.createOMElement("serviceNamespace", omNs);
        if (namespace != null) {
        	org.apache.axiom.om.OMText serviceNamespaceText = fac.createOMText(serviceNamespace, namespace);
        	serviceNamespaceText.setOptimize(false);
        	serviceNamespace.addChild(serviceNamespaceText);
        }
        method.addChild(serviceNamespace);
        
        // Create keyword elements

        org.apache.axiom.om.OMElement keywordsEl = fac.createOMElement("keywords", omNs);
        if (keywords != null) {
        	for (int i = 0; i < keywords.length; i++)
        	{
        		stringElement = fac.createOMElement("string", omNs);
        		stringElementText = fac.createOMText(stringElement, keywords[i]);
        		stringElementText.setOptimize(false);
        		stringElement.addChild(stringElementText);
        		keywordsEl.addChild(stringElement);
        	}
        }
        method.addChild(keywordsEl);
        
        
        // Create publishMetadata element
        org.apache.axiom.om.OMElement publishMetadataEl = fac.createOMElement("publishMetadata", omNs);
        org.apache.axiom.om.OMText publishMetadataText = fac.createOMText(publishMetadataEl, String.valueOf(createMetadata));
        publishMetadataEl.addChild(publishMetadataText);
        method.addChild(publishMetadataEl);
        
        return method;
	}
	
	/**
     * Creates the <ListWebServices> element used to check for name collisions.
     * @return The <ListWebServices> element.
     */
	public static org.apache.axiom.om.OMElement getListWebServicesElement()
	{
		org.apache.axiom.om.OMFactory fac = org.apache.axiom.om.OMAbstractFactory.getOMFactory();
        org.apache.axiom.om.OMNamespace omNs = fac.createOMNamespace(GroovyWebServicesClient.WEB_SERVICES_NS_92, "");
        org.apache.axiom.om.OMElement method = fac.createOMElement("ListWebServices", omNs);
        return method;
	}
}

class WSMWrapper
{
	GroovyWebServiceMaker webServiceMaker;
	def ant = new AntBuilder();
	
	WSMWrapper(hostStr, portStr, userStr, passwordStr)
	{
		//no url passed in, must retrieve from metadata
		//source("C:/Program Files/SAS/SASDeploymentManager/9.2/products/cfgwizard__92210__prt__xx__sp0__1/Utilities/JomaWrapper.groovy");
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "In WSM Constructor via metadata. About to create JOMA connection.");
		def jomaWrapper = new JomaWrapperInternal();
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Created JOMA connection. Now connecting.");
		jomaWrapper.connect(hostStr, portStr, userStr, passwordStr);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Connected via JOMA. About to getMetadataObjects.");
		String typeStr = "TCPIPConnection";
		String searchStr = "Source/DeployedComponent[@Name=&apos;Registered SAS Application&apos;]/DescriptiveComponent/SoftwareComponent[@Name contains &apos;BI Web Services for Java&apos;]";
		def objects = jomaWrapper.getMetadataObjects(typeStr, searchStr);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Got metadata objects.");
		if (objects != null && objects.size() > 0)
		{
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Found TCPIPConnection via BIWS for Java.");
			def connection = objects.get(0);
			def hostName = connection.getHostName();
			def hostPort = connection.getPort();
			def hostService = connection.getService();
			def hostProtocol = connection.getCommunicationProtocol();
			String wsmUrl = hostProtocol + "://" + hostName + ":" + hostPort + hostService + "/services/WebServiceMaker";
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "WSM url: " + wsmUrl + ". About to initialize.");
			this.initialize(wsmUrl);
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Finished initializing WSM.");
		} else
		{
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "WSM not found via Java. Trying generic WSM search. About to getMetadataObjects.");
			searchStr = "Source/DeployedComponent[@Name=&apos;WebServiceMaker&apos;]";
			objects = jomaWrapper.getMetadataObjects(typeStr, searchStr);
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Got metadata objects.");
			if (objects != null && objects.size() > 0)
			{
				ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Found connection object via generic WSM search.");
				def connection = objects.get(0);
				def hostName = connection.getHostName();
				def hostPort = connection.getPort();
				def hostService = connection.getService();
				def hostProtocol = connection.getCommunicationProtocol();
				String wsmUrl = hostProtocol + "://" + hostName + ":" + hostPort + hostService;
				ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "WSM url: " + wsmUrl + ". About to initialize.");
				this.initialize(wsmUrl);
				ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Finished initializing WSM.");
			}
			else
				throw new Exception("Found no metadata objects that match the search for WSM. Cannot construct WSM URL");
		}
	}
	
	WSMWrapper(String wsmUrlStr)
	{
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "In constructor via URL. About to initialize.");
		this.initialize(wsmUrlStr);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Finished initializing WSM.");
	}
	
	void initialize(String wsmUrlStr)
	{
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, " Creating URL object.");
		java.net.URL wsmUrl = new java.net.URL(wsmUrlStr);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Created URL, now creating underlying common GroovyWebServiceMaker object.");
		webServiceMaker = new GroovyWebServiceMaker(wsmUrl);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Created underlying GroovyWebServiceMaker object.");
	}
	
	String makeWebService(String serviceNameStr, String[] storedProcesses, String namespaceStr, String[] keywords, String usernameStr, String passwordStr, String publishMetadataStr, String overwriteStr)
	{
		// Default namespace to http://tempuri.org/ if not set
		if (namespaceStr == null)
		{
			namespaceStr = "http://tempuri.org/" + serviceNameStr;
		}
		
		// Default publishing metadata to true if not set
		boolean publishMetadata	= false;
		if (publishMetadataStr != null)
		{
			publishMetadata = Boolean.parseBoolean(publishMetadataStr);
		}
		
		// Default overwriting existing service to false if not set
		boolean overwrite = false;
		if (overwriteStr != null)
		{
			overwrite = Boolean.parseBoolean(overwriteStr);
		}
		
		boolean found = this.listWebServices(usernameStr, passwordStr).any{it.compareToIgnoreCase(serviceNameStr) == 0};
		
		if (found)
		{
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Web service '$serviceNameStr' already exists");
			if (overwrite)
			{
				ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Overwriting '$serviceNameStr' web service");
				this.removeWebService(serviceNameStr, usernameStr, passwordStr);
			}
			else
			{
				return null;
			}
		}
		
		// create web service
		java.net.URL newServiceEndpoint = webServiceMaker.makeWebService(serviceNameStr, storedProcesses, namespaceStr, keywords, usernameStr, passwordStr, publishMetadata);
		if (newServiceEndpoint != null)
		{
			return newServiceEndpoint.toString();
		} else {
			return null;
		}
	}
	
	void removeWebService(String serviceName, String userName, String password)
	{
		webServiceMaker.deleteService(serviceName, userName, password);
	}
	
	List listWebServices(String userName, String password)
	{
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "In wsmWrapper.listWebServices. About to create GroovyWebServicesClient.");
		List retList = new ArrayList();
		GroovyWebServicesClient client = new GroovyWebServicesClient(webServiceMaker.getEndpoint(), GroovyWebServicesClient.LIST_WEB_SERVICES_ACTION_92, userName, password);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Created GroovyWebServicesClient. Getting payload.");
		org.apache.axiom.om.OMElement payload = com.sas.web.services.common.WebServiceMaker.getListWebServicesElement();
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Got payload. Now send/receive.");
		org.apache.axiom.om.OMElement result = client.sendReceive(payload);
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Got result.");
		if(result != null)
		{
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Result was not null. Getting element and text.");
			org.apache.axiom.om.OMElement listResult = (org.apache.axiom.om.OMElement) result.getFirstElement();
			retList = listResult.getChildren().findAll{ it.getLocalName() == "string"}*.getText();
			ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Got arraylist of existing services.");
		}
		ant.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Returning for wsmWrapper.listWebServices.");
		return retList;
	}
}

def handleThrowable(Throwable t)  {
	def ant = new AntBuilder();
	if (t instanceof org.apache.axis2.AxisFault)
	{
		String soapMessage = t.getLocalizedMessage();
		if (t.getDetail() != null)
		{
			soapMessage = com.sas.web.services.common.WebServiceUtilities.GetErrorDetailFromAxisFault(t);
		}
		ant.echo(level:org.apache.tools.ant.types.LogLevel.WARN, soapMessage);
	}
	def sw = new StringWriter();
	def pw = new PrintWriter(sw);
	t.printStackTrace(pw);
	ant.echo(level:org.apache.tools.ant.types.LogLevel.WARN, sw.toString());
}

class ExitingExceptionHandler implements java.lang.Thread.UncaughtExceptionHandler
{
    public void uncaughtException(Thread t, Throwable e)
    {
        handleThrowable(e);
        System.exit(0);
    }
}

Thread.currentThread().setUncaughtExceptionHandler(new ExitingExceptionHandler());

// get common attributes
String wsmUrl	 		= attributes.get("wsmurl");
String serviceName 		= attributes.get("servicename");
String username			= attributes.get("username");
String password			= attributes.get("password");
String outputProperty	= attributes.get("outputproperty");
def globalAnt = new AntBuilder();
// take reasonable defaults for username and password if not supplied
if (username == null || password == null || "".equals(username) || "".equals(password))
{
	username = project.getProperty("metadata.connection.userid");
	password = project.getProperty("metadata.connection.passwd");
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Set username and password to defaults");
}

// global try-catch
try {

// create wsm wrapper
def wsm = null;
if (wsmUrl == null || "".equals(wsmUrl))
{
	def metadataHost = project.getProperty("iomsrv.metadatasrv.host");
	def metadataPort = project.getProperty("iomsrv.metadatasrv.port");
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Set metadata connection info. Now constructing WSM.");
	wsm = new WSMWrapper(metadataHost, metadataPort, username, password);
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Constructed WSM via metadata lookup.");
} else
{
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "URL specified, constructing WSM.");
	wsm = new WSMWrapper(wsmUrl);
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Constructed WSM via URL.");
}

// determine which ant task we're doing
String currentTaskName = self.getTaskName();
globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Task name: " + currentTaskName);
if ("makeWebServiceGroovy".compareToIgnoreCase(currentTaskName) == 0) // compare strings case insensitively
{
	def storedProcesses = (String[])elements.get("storedprocesses").get(0).getItems()*.toString();
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, storedProcesses.length + " stored proceses");
	def keywords = elements.get("keywords");
	if (keywords != null)
	{
		keywords = (String[])keywords.get(0).getItems()*.toString();
		globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, keywords.length + " keywords");
	}

	def namespace = attributes.get("namespace");
	if (namespace != null && "".equals(namespace))
		namespace = null;
	
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "namespace: " + namespace);
	
	def publishMetadata = attributes.get("publishmetadata");
	if (publishMetadata != null && "".equals(publishMetadata))
		publishMetadata = null;
	
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "publish metadata: " + publishMetadata);
	
	def overwrite = attributes.get("overwrite");
	if (overwrite != null && "".equals(overwrite))
		overwrite = null;
		
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "overwrite: " + overwrite);
	
	try
	{
		def newServiceEndpoint = wsm.makeWebService(serviceName, storedProcesses, namespace, keywords, username, password, publishMetadata, overwrite);
		if (newServiceEndpoint != null)
		{
			project.setNewProperty(outputProperty, newServiceEndpoint);
		}
	} catch (Throwable e)
	{
		handleThrowable(e);
	}
}else if ("removeWebServiceGroovy".compareToIgnoreCase(currentTaskName) == 0)
{
	try
	{
		wsm.removeWebService(serviceName, username, password);
		project.setNewProperty(outputProperty, "true");
	} catch (Throwable e)
	{
		handleThrowable(e);
		project.setNewProperty(outputProperty, "false");
	}
} else if ("webServiceExistsGroovy".compareToIgnoreCase(currentTaskName) == 0)
{
	globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "In webServiceExists. About to list web services.");
	try
	{
		boolean exists = wsm.listWebServices(username, password).any{it.compareToIgnoreCase(serviceName) == 0};
		globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Listed web services. exists is " + Boolean.toString(exists));
		project.setNewProperty(outputProperty, Boolean.toString(exists));
		globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Set property, now exiting.");
	} catch (Throwable e)
	{
		project.setNewProperty(outputProperty, Boolean.toString(false));
		globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Caught throwable during webServiceExists, now handling.");
		handleThrowable(e);
		globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Handled throwable.");
		
	}
}
globalAnt.echo(level:org.apache.tools.ant.types.LogLevel.INFO, "Leaving groovy script.");

} catch (Throwable t) {
	handleThrowable(t);
}
