// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Connectionfactory extends Resource {

    String resourceType = "connectionfactory"
    String name;
    String jndiname;
	String useNamingAlias;
	String maxConnections;
    String servername;
    String messagingDir;
    
    Connectionfactory() {}
    Connectionfactory(String c,String j, String s, String n, String m){
        name = c
        jndiname = j
        servername = s
		useNamingAlias = n
		maxConnections = m
    }
    
    /*
		connectionfactory.cf1.name=SAS Queue Factory
		connectionfactory.cf1.jndiname=sas/jms/QueueConnectionFactory
		connectionfactory.cf1.servername=SASServer1
    */
    int configure(String instance, Properties scriptProperties){
        try {
    
	        Logger.log('DEBUG', "connectionfactory - Configuring ${name}")
			name = scriptProperties.get("connectionfactory."+instance+".name")
			jndiname = scriptProperties.get("connectionfactory."+instance+".jndiname")  
	        servername = scriptProperties.get("connectionfactory."+instance+".servername") 
	        //messagingDir = scriptProperties.get("connectionfactory."+instance+".messagingDir") 
	        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
			def connectionfactoryFile
			def connectionfactoryParser
			
			messagingDir = (new JBossVersion(jbossInstallDir)).getMessagingDir()
			if (messagingDir == 'hornetq'){
				connectionfactoryFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/hornetq-jms.xml")
				connectionfactoryParser = new Parser()
				connectionfactoryParser.parse(connectionfactoryFile)
				
				//First find the entries...this contains individual entry elements with jndi names
				def entriesNode = connectionfactoryParser.findNode('//connection-factory[@name="NettyConnectionFactory"]/entries')
				if (entriesNode){
					def entryNode = connectionfactoryParser.findNode("//connection-factory[@name='NettyConnectionFactory']/entries/entry[@name='$jndiname']")
					if (entryNode == null) {
					   Node newEntry = connectionfactoryParser.addNode("entry","",entriesNode)
					                  connectionfactoryParser.addAttribute("name",jndiname,newEntry)
					   Logger.log('DEBUG', "connectionfactory - creating new entry for $jndiname")
					   connectionfactoryParser.output(connectionfactoryFile)
					}
				}else {
				    Logger.log('DEBUG', "connectionfactory - can't find entries element, unable to create new binding for $jndiname")
				}
			} else if (messagingDir == 'jms'){
			
				//Check on maxConnections.  Default in JBossMQ is 20, users may have asked for more...
				maxConnections = scriptProperties.get("connectionfactory."+instance+".maxConnections")					
				if (! maxConnections.equalsIgnoreCase("not-set")){
					def jmsdsFile= new File("$jbossInstallDir/server/$servername/deploy/$messagingDir/jms-ds.xml")
					def jmsdsParser = new Parser()
										
					jmsdsParser.parse(jmsdsFile.getCanonicalPath())	
					Node maxConnectionsNode = jmsdsParser.findNode("//connection-factories/tx-connection-factory/max-pool-size")					
					
					jmsdsParser.updateTextNode(maxConnections,maxConnectionsNode)
					Logger.log('DEBUG', "connectionfactory - DUMP - " + jmsdsFile.getCanonicalPath())
					Logger.log('DEBUG', maxConnectionsNode)
					
					jmsdsParser.output(jmsdsFile.getCanonicalPath() )

				}
				
				connectionfactoryFile = new File("$jbossInstallDir/server/$servername/deploy/$messagingDir/sas-jms-service.xml")
				if (! connectionfactoryFile.exists()){
					createJmsServiceFile(connectionfactoryFile)
				}
				
				connectionfactoryParser = new Parser()
				connectionfactoryParser.parse(connectionfactoryFile.getCanonicalPath())
				useNamingAlias = scriptProperties.get("connectionfactory."+instance+".useNamingAlias")
				
				if (useNamingAlias.equalsIgnoreCase("true")){
					//Instead of a local/remote factory, just generate a local, mapping our name to "ConnectionFactory"
					/*
					   <mbean code="org.jboss.naming.NamingAlias" 
                              name="jboss.mq:service=NamingAlias,fromName=${name}">
                          <attribute name="ToName">ConnectionFactory</attribute>
                          <attribute name="FromName">${jndiname}</attribute>
                          <depends>jboss:service=Naming</depends>
                       </mbean>
					*/
				  //First find the bindings...this contains individual binding elements with jndi names
				  def node = connectionfactoryParser.findNode('//server')
				  if (node != null) {
					  Logger.log('DEBUG', "connectionfactory - found server")
					  Node existingNode = connectionfactoryParser.findNode("//mbean[@name='jboss.mq:service=NamingAlias,fromName=$name']")
					  if (existingNode != null) {
						  Logger.log('DEBUG', "connectionfactory - found Node for $name! removing...");
						  def textNode = existingNode.getPreviousSibling();
						  if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
							   Logger.log('DEBUG', "connectionfactory - removing previous whitespace text node...");
							   connectionfactoryParser.removeNode(textNode);
						   }
						   connectionfactoryParser.removeNode(existingNode);
					  }
		  
					  //def mbean = connectionfactoryParser.addNode("mbean","",node.getFirstChild(), "SIBLING")
					  def mbean = connectionfactoryParser.addNode("mbean","",node)
					  connectionfactoryParser.addTextNode("\n\n   ",mbean,"SIBLING")
					  connectionfactoryParser.addAttribute("code","org.jboss.naming.NamingAlias", mbean)
					  connectionfactoryParser.addAttribute("name","jboss.mq:service=NamingAlias,fromName=$name", mbean)
  
					  def attribute = connectionfactoryParser.addNode("attribute","ConnectionFactory", mbean)
						  connectionfactoryParser.addAttribute("name","ToName",attribute)
						  
					  attribute = connectionfactoryParser.addNode("attribute",jndiname, mbean)
						  connectionfactoryParser.addAttribute("name","FromName",attribute)
  
					  def depends = connectionfactoryParser.addNode("depends","jboss:service=Naming", mbean)
						  
					  Logger.log('DEBUG', "connectionfactory - DUMP - connectionfactoryFile.getCanonicalPath()")
					  Logger.log('DEBUG', node)
					  
					  connectionfactoryParser.output(connectionfactoryFile.getCanonicalPath())
					  
  
				  }else {
						 Logger.log('DEBUG', "connectionfactory - can't find server element, unable to create new connectionfactory for $jndiname")
				  }

				} else {
				    //This is the typical case...generate a LinkRefPairService, mapping to local/remote factory.
					//First find the bindings...this contains individual binding elements with jndi names
			          /*
					     <mbean code="org.jboss.naming.LinkRefPairService"	name="jboss.jms:alias=@connectionFactoryName@">
							<attribute name="JndiName">@jndiName@</attribute>
							<attribute name="RemoteJndiName">ConnectionFactory</attribute>
							<attribute name="LocalJndiName">java:/JmsXA</attribute>
							<depends>jboss:service=Naming</depends>
						 </mbean>
					  */
					//First find the bindings...this contains individual binding elements with jndi names
					def node = connectionfactoryParser.findNode('//server')
					if (node != null) {
						Logger.log('DEBUG', "connectionfactory - found server")
						Node existingNode = connectionfactoryParser.findNode("//mbean[@name='jboss.jms:alias=$name']")
						if (existingNode != null) {
							Logger.log('DEBUG', "connectionfactory - found Node for $name! removing...");
							def textNode = existingNode.getPreviousSibling();
							if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
								 Logger.log('DEBUG', "connectionfactory - removing previous whitespace text node...");
								 connectionfactoryParser.removeNode(textNode);
							 }
							 connectionfactoryParser.removeNode(existingNode);
						}
			
						//def mbean = connectionfactoryParser.addNode("mbean","",node.getFirstChild(), "SIBLING")
						def mbean = connectionfactoryParser.addNode("mbean","",node)
						connectionfactoryParser.addTextNode("\n\n   ",mbean,"SIBLING")
						connectionfactoryParser.addAttribute("code","org.jboss.naming.LinkRefPairService", mbean)
						connectionfactoryParser.addAttribute("name","jboss.jms:alias=$name", mbean)
	
						def attribute = connectionfactoryParser.addNode("attribute",jndiname, mbean)
						    connectionfactoryParser.addAttribute("name","JndiName",attribute)
							
	                    attribute = connectionfactoryParser.addNode("attribute","ConnectionFactory", mbean)
						    connectionfactoryParser.addAttribute("name","RemoteJndiName",attribute)
	
						attribute = connectionfactoryParser.addNode("attribute","java:/JmsXA", mbean)
							connectionfactoryParser.addAttribute("name","LocalJndiName",attribute)
			            
						Logger.log('DEBUG', "connectionfactory - DUMP - connectionfactoryFile.getCanonicalPath()")
			            Logger.log('DEBUG', node)
						
						connectionfactoryParser.output(connectionfactoryFile.getCanonicalPath())
						
	
					}else {
			               Logger.log('DEBUG', "connectionfactory - can't find server element, unable to create new connectionfactory for $jndiname")
			        }
				}
			
			} else {
			
				connectionfactoryFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/connection-factories-service.xml")
				connectionfactoryParser = new Parser()     
				connectionfactoryParser.parse(connectionfactoryFile )
				
				//First find the bindings...this contains individual binding elements with jndi names
				def node = connectionfactoryParser.findNode('//mbean[@name="jboss.messaging.connectionfactory:service=ConnectionFactory"]/attribute[@name="JNDIBindings"]/bindings')
				if (node != null) {
				    Logger.log('DEBUG', "connectionfactory - Found bindings")
				def bindingList = node.getChildNodes()
				def jndiNameExists = false
				
				for (int i=0;i<bindingList.getLength();i++) {
				   def binding = bindingList.item(i)
				   if (binding.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
				      if (binding.getNodeName() == "binding") {
				         if (binding.getTextContent() == jndiname) {
				            jndiNameExists = true
				   }  }  }  }
				if (!jndiNameExists) {
				   def newBinding = connectionfactoryParser.addNode("binding",jndiname,node.getFirstChild(),"SIBLING")
				   Logger.log('DEBUG', "connectionfactory - creating new binding for $jndiname")          
				   connectionfactoryParser.output(connectionfactoryFile)
				}
				}
				else {
				 Logger.log('DEBUG', "connectionfactory - can't find bindings element, unable to create new binding for $jndiname")
				 }
			}
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }


}