// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node


class Externalcontext extends Resource {
    
    String resourceType = "externalcontext"
    String local;
    String remote;
    String url;
    String initialcontextfactory;
    String servername;
    String name;
    
    Externalcontext() {}
    
    Externalcontext(String l,String r,String u,String i,String s ){
        local = l
        remote = r
        url = u
        initialcontextfactory = i
        servername = s
        
        Logger.log('DEBUG',"Externalcontext - processing local="+local)

    }
    
    /*
     *   <mapForeignJNDINameScript
               file="${jboss.tmp.service}"
               file_out="${jboss.tmp.service}"
               xpathnode="//server"
               localjndiname="${localJndiName}"
               remotejndiname="${remoteJndiName}"
               foreignproviderurl="${foreignProviderURL}"
               initialcontextfactory="${initialContextFactory}"/>

		externalcontext.externalcontext1.local=sas/jms/SharedServicesAlertQueue
		externalcontext.externalcontext1.remote=sas/jms/AlertQueue
		externalcontext.externalcontext1.url=jnp://${webappsrv.host}:${biservmid.port} = jnp://myhost.sas.com:8080
		externalcontext.externalcontext1.initialcontextfactory=org.jnp.interfaces.NamingContextFactory
		externalcontext.externalcontext1.servername=SASServer1
		
       
        foreignProviderURL="jnp://${bisrvmidHost}:${biservmidPort}"
        initialContextFactory="org.jnp.interfaces.NamingContextFactory"
        localName="sas/jms/AlertQueue"
        
		<mbean code="org.jboss.naming.ExternalContext" 
		       name="jboss.jndi:service=ExternalContext,jndiName=${localName}">
		    <attribute name="JndiName">${localName}</attribute>
		    <attribute name="Properties">
		        java.naming.factory.initial=${initialContextFactory}
		        java.naming.provider.url=${foreignProviderURL}
		    </attribute>
		    <attribute name="RemoteAccess">true</attribute>
		    <depends>jboss:service=Naming</depends>
		</mbean>
        
    */
    
    int configure(String instance, Properties scriptProperties){
        try {
    
        Logger.log('DEBUG', "externalcontext - Configuring ${instance}")
        local = scriptProperties.get("externalcontext."+instance+".local")
        remote = scriptProperties.get("externalcontext."+instance+".remote")  
        url = scriptProperties.get("externalcontext."+instance+".url") 
        initialcontextfactory = scriptProperties.get("externalcontext."+instance+".initialcontextfactory")  
        servername = scriptProperties.get("externalcontext."+instance+".servername")  
        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
        
        name = "local:$local remote:$remote"
                    
        def ecFile = new String("$jbossInstallDir/server/$servername/conf/jboss-service.xml")
        def ecParser = new Parser(ecFile)
        def serverNode = ecParser.findNode("//server")
		def firstNode
       
        def ecNode
        def attrNode
        
        //First find existing ExternalContext...delete if found...
        ecNode = ecParser.findNode("//mbean[@name='jboss.jndi:service=ExternalContext,jndiName="+local+"']");

        if (ecNode != null) {
            Logger.log('DEBUG',"externalcontext - Found ExternalContext :${local}")
                         def textNode = ecNode.getPreviousSibling();
             if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
                  Logger.log('DEBUG', "externalcontext - removing previous whitespace text node...");
                  ecParser.removeNode(textNode);
              }
             ecParser.removeNode(ecNode)
         }

		firstNode = serverNode.getFirstChild();
		
		/* By adding the ExternalContext mbean node as a SIBLING to the first node under //server
		 * we make this entry the "new" first node.  Moving the ExternalContext to the top
		 * requires we add a "depends" node on JNDI.  If we don't add the depends, this 
		 * ExternalContect will be instantiated before JNDI.
		 */
		
		 //	 <mbean code="org.jboss.naming.ExternalContext" 
		 //            name="jboss.jndi:service=ExternalContext,jndiName=${localName}">
         ecNode = ecParser.addNode("mbean","",firstNode, "SIBLING")
            ecParser.addAttribute("code","org.jboss.naming.ExternalContext",ecNode)
            ecParser.addAttribute("name","jboss.jndi:service=ExternalContext,jndiName=${local}",ecNode)
         
		//         <attribute name="JndiName">${localName}</attribute> 	
		attrNode = ecParser.addNode("attribute",local,ecNode)
            ecParser.addAttribute("name","JndiName",attrNode)
		 
		 //         <attribute name="Properties">
		 //               java.naming.factory.initial=${initialContextFactory}
		 //               java.naming.provider.url=${foreignProviderURL}
	     //         </attribute>
         def space1="\n            "
         def space2="\n        "
         attrNode = ecParser.addNode("attribute","${space1}java.naming.factory.initial=${initialcontextfactory}${space1}java.naming.provider.url=${url}${space2}",ecNode)
            ecParser.addAttribute("name","Properties",attrNode)
			
		 //         <attribute name="RemoteAccess">true</attribute>	
         attrNode = ecParser.addNode("attribute","true",ecNode)
            ecParser.addAttribute("name","RemoteAccess",attrNode)
			
		 //		    <depends>jboss:service=Naming</depends>	
		 ecParser.addNode("depends","jboss:service=Naming",ecNode)
			
         ecParser.output(ecFile)
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }
	
	int changeHost(String instance, Properties scriptProperties){
		try {	
			Logger.log('DEBUG', "externalcontext - ChangeHost ${instance}")
			local = scriptProperties.get("externalcontext."+instance+".local")
			remote = scriptProperties.get("externalcontext."+instance+".remote")
			url = scriptProperties.get("externalcontext."+instance+".url")
			initialcontextfactory = scriptProperties.get("externalcontext."+instance+".initialcontextfactory")
			servername = scriptProperties.get("externalcontext."+instance+".servername")
			def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
			
			name = "local:$local remote:$remote"
						
			def ecFile = new String("$jbossInstallDir/server/$servername/conf/jboss-service.xml")
			def ecParser = new Parser(ecFile)
			def serverNode = ecParser.findNode("//server")
	
			def ecNode
			def attrNode
			
			//First find existing ExternalContext...delete if found...
			ecNode = ecParser.findNode("//mbean[@name='jboss.jndi:service=ExternalContext,jndiName="+local+"']/attribute[@name='Properties']");
	
			if (ecNode) {
				Logger.log('DEBUG',"externalcontext - ChangeHost of ExternalContext $local using $url")				
				def space1="\n            "
				def space2="\n        "
				attrNode = ecParser.addNode("attribute","${space1}java.naming.factory.initial=${initialcontextfactory}${space1}java.naming.provider.url=${url}${space2}",ecNode,"Sibling")
				ecParser.addAttribute("name","Properties",attrNode)
				ecParser.removeNode(ecNode)
	   		    ecParser.output(ecFile)
			 }else{
			    Logger.log('DEBUG',"externalcontext - can't locate ExternalContext $local")
			    return 8;
			 }
			
		 }
		 catch (Throwable e) {
			 Logger.log('DEBUG', "Failure : " + e.toString());
			 e.printStackTrace();
			 return 8;
		 }
		 return 0;
	}
}
