// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved


package com.sas.config.jboss

import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import com.sas.config.jboss.*



class JBossPorts {
    String jndiPort
    String rmiPort
    String hajndiPort
    String hajndiRmiPort
    String snmpagentPort
    String snmpagentTrapdPort
    String hypersonicPort
    String corbaorbPort
    String ejb3Port
    String invokerRmiobjectPort
    String invokerServerBindPort
    String remotingConnectorPort
    String invokerPooledhaServerbindport
    String messagingPort
    String webserverPortAjp
    String webserverPortHttp
    String webservicePort
    String invocationlayerServerbindport
    String webserverPortHttps
    String rmiobjectport

    String server
    String name
    String portIncrement
    String version
    String bindingsFile
    Properties scriptProperties
    
    JBossPorts (inServer, inName, inPortIncrement, inVersion, inScriptProp){
        server = inServer
        name = inName
        portIncrement = inPortIncrement
        version = inVersion
        scriptProperties = inScriptProp
        
        def installDir = scriptProperties.get("config.jboss.install.dir")
//      def bindingsFile = installDir+"/server/default/conf/bindingservice.beans/META-INF/bindings-jboss-beans.xml"
        bindingsFile = installDir+"/server/"+name+"/conf/bindingservice.beans/META-INF/bindings-jboss-beans.xml"

        if (version == '4') {
           
           //TODO:  Enable binding manager in jboss-service.xml
           
           //Update sample binding file with proper ports for this server.
           def sampleBindings = installDir+"/docs/examples/binding-manager/sample-bindings.xml"
           bindingsFile = installDir+"/server/"+name+"/conf/port-bindings.xml"
           Ant.copyFile(sampleBindings, bindingsFile, 'true')
        }
           
        Logger.log('DEBUG', "JBossPorts - calling getProp(server."+server+".port.jndi,"+portIncrement+")")

        jndiPort = getPort("server."+server+".port.jndi", portIncrement)
        rmiPort = getPort("server."+server+".port.rmi", portIncrement)
        hajndiPort = getPort("server."+server+".port.hajndi", portIncrement)
        hajndiRmiPort = getPort("server."+server+".port.hajndiRmi", portIncrement)
        snmpagentPort = getPort("server."+server+".port.snmpagent", portIncrement)
        snmpagentTrapdPort = getPort("server."+server+".port.snmpagentTrapd", portIncrement)
        hypersonicPort = getPort("server."+server+".port.hypersonic", portIncrement)
        corbaorbPort = getPort("server."+server+".port.corbaorb", portIncrement)
        ejb3Port = getPort("server."+server+".port.ejb3", portIncrement)
        invokerRmiobjectPort = getPort("server."+server+".port.invokerRmiobject", portIncrement)
        invokerServerBindPort = getPort("server."+server+".port.invokerServerBind", portIncrement)
        remotingConnectorPort = getPort("server."+server+".port.remotingConnector", portIncrement)
        invokerPooledhaServerbindport = getPort("server."+server+".port.invokerPooledhaServerbind", portIncrement)
        messagingPort = getPort("server."+server+".port.messaging", portIncrement)
        webserverPortAjp = getPort("server."+server+".port.webserverAjp", portIncrement)
        webserverPortHttp = getPort("server."+server+".port.webserverHttp", portIncrement)
        webservicePort = getPort("server."+server+".port.webservice", portIncrement)
        invocationlayerServerbindport = getPort("server."+server+".port.invocationlayerServerbind", portIncrement)
        webserverPortHttps = getPort("server."+server+".port.webserverHttps", portIncrement)
        rmiobjectport = getPort("server."+server+".port.rmiobject", portIncrement)

        Logger.log('DEBUG', "Configuring " + server)
        Logger.log('DEBUG', "Ports - jndiPort = " + jndiPort)
        Logger.log('DEBUG', "        rmiPort = " + rmiPort)
        Logger.log('DEBUG', "        hajndiPort = " + hajndiPort)
        Logger.log('DEBUG', "        hajndiRmiPort = " + hajndiRmiPort)
        Logger.log('DEBUG', "        snmpagentPort = " + snmpagentPort)
        Logger.log('DEBUG', "        snmpagentTrapdPort = " + snmpagentTrapdPort)
        Logger.log('DEBUG', "        hypersonicPort = " + hypersonicPort)
        Logger.log('DEBUG', "        corbaorbPort = " + corbaorbPort)
        Logger.log('DEBUG', "        ejb3Port = " + ejb3Port)
        Logger.log('DEBUG', "        invokerRmiobjectPort = " + invokerRmiobjectPort)
        Logger.log('DEBUG', "        invokerServerBindPort = " + invokerServerBindPort)
        Logger.log('DEBUG', "        remotingConnectorPort = " + remotingConnectorPort)
        Logger.log('DEBUG', "        invokerPooledhaServerbindport = " + invokerPooledhaServerbindport)
        Logger.log('DEBUG', "        messagingPort = " + messagingPort)
        Logger.log('DEBUG', "        webserverPortAjp = " + webserverPortAjp)
        Logger.log('DEBUG', "        webserverPortHttp = " + webserverPortHttp)
        Logger.log('DEBUG', "        webservicePort = " + webservicePort)
        Logger.log('DEBUG', "        invocationlayerServerbindport = " + invocationlayerServerbindport)
        Logger.log('DEBUG', "        webserverPortHttps = " + webserverPortHttps)
        Logger.log('DEBUG', "        rmiobjectport = " + rmiobjectport)
        Logger.log('DEBUG', "Port Offset = "+portIncrement)


        def parser = new Parser()
        parser.parse(bindingsFile)
        def doc = parser.getDocument()
        
        def elementRoot = doc.documentElement
        
        def xpath = XPathFactory.newInstance().newXPath();

        
        if (version=="4") {
            /*
               First, update "name" attribute in our port def file:
               
               <service-bindings>
               <!-- ********************************************************** -->
               <!-- *                        ports-default                   * -->
               <!-- ********************************************************** -->
               <server name="SASServer1">
            */
            
            def xpathParm = "//service-bindings/server"
            parser.addAttribute("name", name, xpathParm)
            
            /* Next, add the ServiceBindingManager defining our new port def file with server:
            
               <mbean code="org.jboss.services.binding.ServiceBindingManager"
                 name="jboss.system:service=ServiceBindingManager">
                 <attribute name="ServerName">ports-01</attribute>
                 <attribute name="StoreURL">${jboss.home.url}/docs/examples/binding-manager/sample-bindings.xml</attribute>
                 <attribute name="StoreFactoryClassName">
                   org.jboss.services.binding.XMLServicesStoreFactory
                 </attribute>
               </mbean>     
            */
            
            def serviceParser = new Parser()
            def serviceXmlFile = installDir+"/server/"+name+"/conf/jboss-service.xml"
            //def serviceXpath="//server/mbean[@name='jboss.rmi:type=RMIClassLoader']"
            def serviceXpath="//server/mbean"  /* Just find first mbean and stuff it there, don't worry about RMIClassLoader */
            def serviceBindingXpath="//server/mbean[@name='jboss.system:service=ServiceBindingManager']"

            serviceParser.parse(serviceXmlFile)
     
            def serviceBindingNode = null
            serviceBindingNode = serviceParser.findNode(serviceBindingXpath)
            if (serviceBindingNode) {
               //we already have a Service Binding, only update the StoreURL text field
               def storeURL = serviceBindingXpath + "/attribute[@name='StoreURL']"
               def serverName = serviceBindingXpath + "/attribute[@name='ServerName']"
               serviceParser.updateTextNode(bindingsFile, storeURL)
               serviceParser.updateTextNode(name, serverName)
            } else {
                // Since we can only insert before a node, go find the RMIClassLoader,
                // then get it's next sibling, and insert before that, thus missing
                // any comments as well as inserting in the middle of the service file
                // directly above the commented out ServiceBindingManager example.
                //
                def serviceNode=serviceParser.findNode(serviceXpath)
                def serviceBindingSibling = serviceNode.getNextSibling()
                
                def mbeanNode = serviceParser.addNode("mbean","",serviceBindingSibling, "SIBLING")
                    serviceParser.addAttribute("code","org.jboss.services.binding.ServiceBindingManager", mbeanNode)
                    serviceParser.addAttribute("name","jboss.system:service=ServiceBindingManager", mbeanNode)
                    def attrNode = serviceParser.addNode("attribute",name,mbeanNode)
                        serviceParser.addAttribute("name","ServerName",attrNode)
                    attrNode = serviceParser.addNode("attribute", bindingsFile, mbeanNode)
                        serviceParser.addAttribute("name","StoreURL",attrNode)
                    attrNode = serviceParser.addNode("attribute","org.jboss.services.binding.XMLServicesStoreFactory",mbeanNode)
                        serviceParser.addAttribute("name","StoreFactoryClassName", attrNode)
    
                serviceParser.addComment(" ==================================================================== ",mbeanNode)
                serviceParser.addComment(" Service Binding                                                      ",mbeanNode)
                serviceParser.addComment(" ==================================================================== ",mbeanNode)
            }
                      
            serviceParser.output(serviceXmlFile)
            
        }
        else {
            // Find and update ServiceBindingManagementObject binding set parameter
            def xpathParm = "//bean[@name='ServiceBindingManagementObject']/constructor/parameter";
    
            def portParameter = xpath.evaluate(xpathParm, elementRoot, XPathConstants.NODESET).item(0);
            if (portParameter != null) {
                def oldTextNode = portParameter.getFirstChild();
                def newText=""
                def currentText=""
                if (oldTextNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
                {
                   currentText=oldTextNode.getNodeValue()
                }
               def text="\${jboss.service.binding.set:" + name + "}"
               def newTextNode=doc.createTextNode(text)
               Logger.log('DEBUG', "Port Config: currentText="+currentText)
               Logger.log('DEBUG', "Port Config: newText="+newTextNode.getNodeValue())
                portParameter.removeChild(oldTextNode)
                portParameter.appendChild(newTextNode)
            }

            xpathParm="//bean[@name='ServiceBindingManagementObject']/constructor/parameter/set"
            def serviceBindingSet = xpath.evaluate(xpathParm, elementRoot, XPathConstants.NODESET).item(0);

            def xpathParm2="//bean[@name='ServiceBindingManagementObject']/constructor/parameter/set/inject[@bean='${name}Bindings']"
            def oldInject_Element = xpath.evaluate(xpathParm2, elementRoot, XPathConstants.NODESET).item(0)
            if (oldInject_Element != null){
                def textNode = oldInject_Element.getPreviousSibling();
                if (textNode != null && textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
                    println("   Removing  whitespace text node:"+textNode.getData()+":");
                    serviceBindingSet.removeChild(textNode);                
                }else {
                    Logger.log('DEBUG', "Port Config: no previous white space found")
                }
                textNode = oldInject_Element.getNextSibling();
                if (textNode != null && textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
                    println("   Removing  whitespace text node:"+textNode.getData()+":");
                    serviceBindingSet.removeChild(textNode);                
                }else {
                    Logger.log('DEBUG', "Port Config: no post white space found")
                }
                Logger.log('DEBUG', "Port Config: removing inject element - "+name+"Bindings")
                serviceBindingSet.removeChild(oldInject_Element);
            } else {
                Logger.log('DEBUG', "Port Config: attempted inject removal , didn't find "+name+"Bindings")
            }

                
            // <inject bean="PortsDefaultBindings"/>
            def inject_Element=doc.createElement("inject");
            inject_Element.setAttribute("bean",name+"Bindings");
            
            serviceBindingSet.insertBefore(inject_Element,serviceBindingSet.getFirstChild());
    
            // Remove any old SASServerNBindings bean
            xpathParm="//bean[@name='${name}Bindings']"
            def oldBindings_Element = xpath.evaluate(xpathParm, elementRoot, XPathConstants.NODESET).item(0)
            if (oldBindings_Element != null){
                def textNode = oldBindings_Element.getPreviousSibling();
                if (textNode != null && textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
                    Logger.log('DEBUG', "Port Config: Removing  whitespace text node from Bean:"+textNode.getData()+":");
                    elementRoot.removeChild(textNode);                
                }else {
                    Logger.log('DEBUG', "Port Config: no previous white space found before Bean")
                }
                Logger.log('DEBUG', "Port Config: Removing Bindings bean - "+name+"Bindings")
                elementRoot.removeChild(oldBindings_Element);
            }else {
                Logger.log('DEBUG', "Port Config: attempted bean removal , didn't find "+name+"Bindings")
            }

            xpathParm="//bean[@name='ServiceBindingManagementObject']"
            def serviceBindingManager_bean = xpath.evaluate(xpathParm, elementRoot, XPathConstants.NODESET).item(0);
    
            //Create serviceBindingSet Bindings bean for Our server
            def bean_Element = doc.createElement("bean");
            bean_Element.setAttribute("name",name+"Bindings");
            bean_Element.setAttribute("class","org.jboss.services.binding.impl.ServiceBindingSet");
            def constructor_Element = doc.createElement("constructor");
            def parm1_Element = doc.createElement("parameter");
            def parm1_Text = doc.createTextNode(name);
            def parm2_Element = doc.createElement("parameter");
            def parm2_Text = doc.createTextNode("\${jboss.bind.address}");
            def parm3_Element = doc.createElement("parameter");
            def parm3_Text = doc.createTextNode(portIncrement);
            def parm4_Element = doc.createElement("parameter");
            def parm4_nullElement = doc.createElement("null");
    
            def whitespaceTextNode=doc.createTextNode("\n\n   ");
            
            //Add our serviceBindingSet to the list of bindings for this ServiceBindingManager
            elementRoot.insertBefore(bean_Element,serviceBindingManager_bean.getNextSibling());
            elementRoot.insertBefore(whitespaceTextNode, bean_Element);
            
            bean_Element.appendChild(constructor_Element);
            constructor_Element.appendChild(parm1_Element);
            parm1_Element.appendChild(parm1_Text);
            constructor_Element.appendChild(parm2_Element);
            parm2_Element.appendChild(parm2_Text);
            constructor_Element.appendChild(parm3_Element);
            parm3_Element.appendChild(parm3_Text);
            constructor_Element.appendChild(parm4_Element);
            parm4_Element.appendChild(parm4_nullElement);        
        }
         
        parser.output(bindingsFile)
    }
    
    String getPort(prop, portIncrement){
        Logger.log('DEBUG', "getPort - prop = $prop")
        Integer port=0
        def tmpPort = scriptProperties.get(prop)    
        def tmpProp = prop.substring("server..port.".length()+server.length())
        Logger.log('DEBUG', "getPort - tmpprop = $tmpProp")
        
        if (version=='4'){
            //For JBoss4, need to increment ports based on increment value.
            // Don't need to decrement in the binding file since those are all absolute
            if (tmpPort == null){
                tmpPort = DefaultPorts[tmpProp]
                tmpPort = (Integer.valueOf(tmpPort) + Integer.valueOf(portIncrement)).toString()     
            }

            updatePort(prop, tmpPort, bindingsFile)
        
        }
        else
        {
            if (tmpPort == null){
                tmpPort = DefaultPorts[tmpProp]       
            }else {
                port = Integer.valueOf(tmpPort) - Integer.valueOf(portIncrement)
                tmpPort = port.toString()
            }
            updatePort(prop, tmpPort, bindingsFile)
            tmpPort = (Integer.valueOf(tmpPort) + Integer.valueOf(portIncrement)).toString()
        }
        
        return tmpPort
    }

    String updatePort(prop, port, file){
        
        def propPort = scriptProperties.get(prop)    
        
        if (propPort != null || version=='4')
        {
//          we have a port that is set in the properties file, replace that value in the port defaults file
            def newProp = prop.substring("server..port.".length()+server.length())
            String defaultPort = DefaultPorts[newProp]
            if (defaultPort != port)
            {
                Logger.log('DEBUG', "updatePort - "+prop+" found in properties, updating "+defaultPort+" to "+port+" in "+file)
                Map<String,String> replaceMap = new HashMap<String,String>()
                replaceMap.put(">"+defaultPort, ">"+port)
                if (version == '4') {       
                    replaceMap.put(":"+defaultPort, ":"+port)
                    replaceMap.put('port="'+defaultPort, 'port="'+port)
                    replaceMap.put("jnp.discoveryPort="+defaultPort, "jnp.discoveryPort="+port)
                }
                Utils.replace (new File(file), replaceMap) 
            }                      
        }            
        return port
    }
}

public class DefaultPorts {
	static final String jndi="1099"
	static final String rmi="1098"
	static final String hajndi="1100"
	static final String hajndiRmi="1101"
	static final String snmpagent="1161"
	static final String snmpagentTrapd="1162"
	static final String hypersonic="1701"
	static final String corbaorb="3528"
	static final String ejb3="3873"
	static final String invokerRmiobject="4444"
	static final String invokerServerBind="4445"
	static final String remotingConnector="4446"
	static final String invokerPooledhaServerbind="4448"
	static final String messaging="4457"
	static final String webserverAjp="8009"
	static final String webserverHttp="8080"
	static final String webservice="8083"
	static final String invocationlayerServerbind="8093"
	static final String webserverHttps="8443"
	static final String rmiobject="19001"

    DefaultPorts(){}

}