// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import java.io.File
//import org.apache.commons.io.FileUtils

class JDBCProvider extends Resource {
	
    String resourceType = "jdbcprovider"
    String name;
    String servername;
    String classpath;
    java.util.ArrayList classpathList;
	
    JDBCProvider() {}
    JDBCProvider(String n, String s, String c){
        name = n
        servername = s
        classpath = c   
		
		classpathList=[]
		
		for (sourceJar in classpath.tokenize(';')){
			classpathList.add(sourceJar)
		 }

    }

	int configure(String instance, Properties scriptProperties){
        try {
    
		Logger.log('DEBUG', "jdbcprovider - Configuring ${instance}")

		name = scriptProperties.get("jdbcprovider."+instance+".name")
		servername = scriptProperties.get("jdbcprovider."+instance+".servername")
		classpath = scriptProperties.get("jdbcprovider."+instance+".classpath")
		//def jbossDatasourceOptions = scriptProperties.get("datasource."+instance+".jbossDatasourceOptions")
		def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

		Logger.log('DEBUG',"jdbcprovider - jdbcprovider=${name}")
		Logger.log('DEBUG',"jdbcprovider - servername=${servername}")
		Logger.log('DEBUG',"jdbcprovider - classpath=${classpath}")
       
		if (classpath!=null){
		   File targetServerFile = new File(jbossInstallDir,"server")
		   File targetServernameFile = new File(targetServerFile,servername)
		   File targetLibFile = new File(targetServernameFile,"lib")
		   def targetDir = targetLibFile.getCanonicalPath()
		   
		   Logger.log('DEBUG',"jdbcprovider - touch $targetDir")
		   Ant.touchDir(targetDir)
		   for (sourceJar in classpath.tokenize(';')){
		      Logger.log('DEBUG', "jdbcprovider - copy $sourceJar $targetDir")
        
		      Ant.copyFileToDir(sourceJar,targetDir, "true")
        
		   }
		}

     }
     catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
     }
     return 0;
	}
	
	int unconfigure(String instance, Properties scriptProperties){
		try {
	
		Logger.log('DEBUG', "jdbcprovider - Un-Configuring ${instance}")

		name = scriptProperties.get("jdbcprovider."+instance+".name")
		servername = scriptProperties.get("jdbcprovider."+instance+".servername")
		classpath = scriptProperties.get("jdbcprovider."+instance+".classpath")
		
		//def jbossDatasourceOptions = scriptProperties.get("datasource."+instance+".jbossDatasourceOptions")
		def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
        def configType = scriptProperties.'config.type'
		
	   
		if (classpath == null){
			Logger.log('DEBUG',"jdbcprovider.unconfigure - nothing to remove, exiting...")
		}else {
		    Logger.log('DEBUG',"jdbcprovider.unconfigure - jdbcprovider=${name}")
		    Logger.log('DEBUG',"jdbcprovider.unconfigure - servername=${servername}")
		    Logger.log('DEBUG',"jdbcprovider.unconfigure - classpath=${classpath}")
		
		   def targetDir = new String("${jbossInstallDir}/server/${servername}/lib")
		   Logger.log('DEBUG',"jdbcprovider - touch $targetDir")
		   Ant.touchDir(targetDir)
		   
		   // get a list of all datasource classpaths and current jdbcDriver classpath(except for this one).
		   // If the current jar we want to remove is in the existing list, don't remove it.
		   
		   def datasourceInstances
		   def datasourceInstanceList
		   def jdbcproviderInstances
		   def jdbcproviderInstanceList
		   def classpathUsed=""

		   datasourceInstances = scriptProperties.'datasource'
		   if (datasourceInstances)
		   	  datasourceInstanceList = datasourceInstances.split(/ /)
				 
		   jdbcproviderInstances = 	 scriptProperties.'jdbcprovider'
		   if (jdbcproviderInstances)
		      jdbcproviderInstanceList = jdbcproviderInstances.split(/ /)

		   
		   datasourceInstanceList.each {
			   classpathUsed += scriptProperties.get("datasource."+it+".classpath")	+ ";"
		   }
		   		   
		   jdbcproviderInstanceList.each {
			   if (it != instance) {
				   classpathUsed += scriptProperties.get("jdbcprovider."+it+".classpath") + ";"
			   }
		   }
			       
		   for (sourcePath in classpath.tokenize(';')){
			  def sourceJar=(new File(sourcePath)).getName();
			  if (classpathUsed.contains(sourceJar)){
				  Logger.log('DEBUG', "jdbcprovider - can't delete $targetDir/$sourceJar, still in use by other resources")
			  }else {				 
				  if (configType == "auto"){
					  Logger.log('DEBUG', "jdbcprovider - delete $targetDir/$sourceJar")		
					  Ant.deleteFile("$targetDir/$sourceJar")
				  }else{
				  	Logger.log('DEBUG', "jdbcprovider - need to delete $targetDir/$sourceJar")
				  }
			  }
		
		   }
		}

	 }
	 catch (Throwable e) {
			 Logger.log('DEBUG', "Failure : " + e.toString());
			 e.printStackTrace();
			 return 8;
	 }
	 return 0;
	}

}

