// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Queue extends Resource {
	
    String resourceType = "queue"
    String name;
    String jndiname;
    String servername;
    String messagingDir;
    
    Queue() {}
    Queue(String c,String j, String s){
        name = c
        jndiname = j
        servername = s
    }
    
   // String name
   // Properties scriptProperties
    
   // queue(String qname, Properties properties){
   //     name = qname;
   //     scriptProperties = properties
   //     
   //     configure()
   // }
    /*
		queues.q1.name=AlertQueue
		queues.q1.jndiname=sas/jms/AlertQueue
		queues.q1.servername=SASServer1
    */
    int configure(String instance, Properties scriptProperties){
        try {

	        Logger.log('DEBUG', "queue - Configuring ${instance}")
	           
	        name = scriptProperties.get("queue."+instance+".name")
	        jndiname = scriptProperties.get("queue."+instance+".jndiname")  
	        servername = scriptProperties.get("queue."+instance+".servername") 
	 
	        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'            
			def messagingDir = (new JBossVersion(jbossInstallDir)).getMessagingDir()
			def queuesFile
			def queuesParser
			if (messagingDir == 'hornetq'){
 
				queuesFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/hornetq-jms.xml")
				queuesParser = new Parser()
				queuesParser.parse(queuesFile)
				def configurationNode = queuesParser.findNode('//configuration')
				//First find the entires...this contains individual entry elements with jndi names
				def queueNode = queuesParser.findNode('//queue[@name="$name"]')
				if (queueNode){
					Logger.log('DEBUG', "queue - found Node for $name! removing...");
					queuesParser.removeNode(queueNode);
				}
				queueNode = queuesParser.addNode("queue","",configurationNode)
				queuesParser.addAttribute("name","$name", queueNode)
				def entryNode = queuesParser.addNode("entry","",queueNode)
				queuesParser.addAttribute("name","$jndiname", entryNode)
	
				queuesParser.output(queuesFile)

			} else if (messagingDir == 'jms'){
				queuesFile = new File("$jbossInstallDir/server/$servername/deploy/$messagingDir/sas-jms-service.xml")

				if (! queuesFile.exists()){
					createJmsServiceFile(queuesFile)
				}
				queuesParser = new Parser()
				queuesParser.parse(queuesFile.getCanonicalPath())
				//First find the bindings...this contains individual binding elements with jndi names
				/*
				   <mbean code="org.jboss.mq.server.jmx.Queue"
				      name="jboss.mq.destination:service=Queue,name=@queueName@">
				      <depends optional-attribute-name="DestinationManager">jboss.mq:service=DestinationManager</depends>
				      <attribute name="JNDIName">@jndiName@</attribute>
				   </mbean>

				 */
				def node = queuesParser.findNode('//server')
				if (node != null) {
					Logger.log('DEBUG', "queue - found server")
					Node existingNode = queuesParser.findNode("//mbean[@name='jboss.mq.destination:service=Queue,name=$name']")
					if (existingNode != null) {
						Logger.log('DEBUG', "queue - found Node for $name! removing...");
						def textNode = existingNode.getPreviousSibling();
						if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
							 Logger.log('DEBUG', "queue - removing previous whitespace text node...");
							 queuesParser.removeNode(textNode);
						 }
						 queuesParser.removeNode(existingNode);
					}
		
					def mbean = queuesParser.addNode("mbean","",node)
					queuesParser.addTextNode("\n\n   ",mbean,"SIBLING")
					queuesParser.addAttribute("code","org.jboss.mq.server.jmx.Queue", mbean)
					queuesParser.addAttribute("name","jboss.mq.destination:service=Queue,name=$name", mbean)

					def attribute = queuesParser.addNode("attribute",jndiname, mbean)
					    queuesParser.addAttribute("name","JNDIName",attribute)
						
                    attribute = queuesParser.addNode("depends","jboss.mq:service=DestinationManager", mbean)
					    queuesParser.addAttribute("optional-attribute-name","DestinationManager",attribute)
					
					queuesParser.output(queuesFile.getCanonicalPath())
					

				}else {
		               Logger.log('DEBUG', "queue - can't find server element, unable to create new queue for $jndiname")
		        }
			} else {
		        queuesFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/destinations-service.xml")
		        queuesParser = new Parser()     
		        queuesParser.parse(queuesFile)
		
		           //First find the bindings...this contains individual binding elements with jndi names
		        def node = queuesParser.findNode('//server')
		        if (node != null) {
		            Logger.log('DEBUG', "queue - found server")
		            Node existingNode = queuesParser.findNode("//mbean[@name='jboss.messaging.destination:service=Queue,name=$name']")
		            if (existingNode != null) {
		                Logger.log('DEBUG', "queue - found Node for $name! removing...");
		                def textNode = existingNode.getPreviousSibling();
		                if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
		                     Logger.log('DEBUG', "queue - removing previous whitespace text node...");
		                     queuesParser.removeNode(textNode);
		                 }
		                 queuesParser.removeNode(existingNode);
		            }
		
		            def mbean = queuesParser.addNode("mbean","",node.getFirstChild(), "SIBLING")
		            queuesParser.addTextNode("\n\n   ",mbean,"SIBLING")
		            queuesParser.addAttribute("code","org.jboss.jms.server.destination.QueueService", mbean)
		            queuesParser.addAttribute("name","jboss.messaging.destination:service=Queue,name=$name", mbean)
		            queuesParser.addAttribute("xmbean-dd","xmdesc/Queue-xmbean.xml", mbean)
		            def depends = queuesParser.addNode("depends","jboss.messaging:service=ServerPeer", mbean)
		            queuesParser.addAttribute("optional-attribute-name","ServerPeer", depends)
		            queuesParser.addNode("depends","jboss.messaging:service=PostOffice", mbean)
		            def attribute = queuesParser.addNode("attribute",jndiname, mbean)
		            queuesParser.addAttribute("name","JNDIName", attribute)
		
		            queuesParser.output(queuesFile)
		        } else {
		               Logger.log('DEBUG', "queue - can't find server element, unable to create new queue for $jndiname")
		        }
			 }
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
        }
	
}