// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Topic extends Resource {
    
    String resourceType = "topic"
    String name;
    String jndiname;
    String servername;
    String messagingDir;
    
    Topic() {}
    Topic(String c,String j, String s){
        name = c
        jndiname = j
        servername = s
    }
    
    /*
		topics.t1.name=AlertTopic
		topics.t1.jndiname=sas/jms/AlertTopic
		topics.t1.servername=SASServer1
    */
    int configure(String instance, Properties scriptProperties){
        try {

	        Logger.log('DEBUG', "topic - Configuring ${instance}")
	           
	        name = scriptProperties.get("topic."+instance+".name")
	        jndiname = scriptProperties.get("topic."+instance+".jndiname")  
	        servername = scriptProperties.get("topic."+instance+".servername") 
	
	        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'
			def messagingDir = (new JBossVersion(jbossInstallDir)).getMessagingDir()
			def topicsParser
			def topicFile
			if (messagingDir == 'hornetq'){
	 
			    topicFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/hornetq-jms.xml")
				topicsParser = new Parser()
				topicsParser.parse(topicFile)
				def configurationNode = topicsParser.findNode('//configuration')
				//First find the entires...this contains individual entry elements with jndi names
				def topicNode = topicsParser.findNode('//topic[@name="$name"]')
				if (topicNode){
					Logger.log('DEBUG', "topic - found Node for $name! removing...");
					topicsParser.removeNode(queueNode);
				}
				topicNode = topicsParser.addNode("topic","",configurationNode)
				topicsParser.addAttribute("name","$name", topicNode)
				def entryNode = topicsParser.addNode("entry","",topicNode)
				topicsParser.addAttribute("name","$jndiname", entryNode)
	
				topicsParser.output(topicFile)
	
			} else if (messagingDir == 'jms'){
				topicFile = new File("$jbossInstallDir/server/$servername/deploy/$messagingDir/sas-jms-service.xml")

				if (! topicFile.exists()){
					createJmsServiceFile(topicFile)
				}
				topicsParser = new Parser()
				topicsParser.parse(topicFile.getCanonicalPath())
				//First find the bindings...this contains individual binding elements with jndi names
				/*
				   <mbean code="org.jboss.mq.server.jmx.Topic"
				      name="jboss.mq.destination:service=Topic,name=@topicName@">
				      <depends optional-attribute-name="DestinationManager">jboss.mq:service=DestinationManager</depends>
				      <attribute name="JNDIName">@jndiName@</attribute>
				   </mbean>

				 */
				def node = topicsParser.findNode('//server')
				if (node != null) {
					Logger.log('DEBUG', "topic - found server")
					Node existingNode = topicsParser.findNode("//mbean[@name='jboss.mq.destination:service=Topic,name=$name']")
					if (existingNode != null) {
						Logger.log('DEBUG', "topic - found Node for $name! removing...");
						def textNode = existingNode.getPreviousSibling();
						if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
							 Logger.log('DEBUG', "topic - removing previous whitespace text node...");
							 topicsParser.removeNode(textNode);
						 }
						 topicsParser.removeNode(existingNode);
					}
		
					def mbean = topicsParser.addNode("mbean","",node)
					topicsParser.addTextNode("\n\n   ",mbean,"SIBLING")
					topicsParser.addAttribute("code","org.jboss.mq.server.jmx.Topic", mbean)
					topicsParser.addAttribute("name","jboss.mq.destination:service=Topic,name=$name", mbean)

					def attribute = topicsParser.addNode("attribute",jndiname, mbean)
					    topicsParser.addAttribute("name","JNDIName",attribute)
						
                    attribute = topicsParser.addNode("depends","jboss.mq:service=DestinationManager", mbean)
					    topicsParser.addAttribute("optional-attribute-name","DestinationManager",attribute)
					
					topicsParser.output(topicFile.getCanonicalPath())
				}else {
				Logger.log('DEBUG', "topic - can't find server element, unable to create new topic for $jndiname")
		        }
			}
			else {
		        topicFile = new String("$jbossInstallDir/server/$servername/deploy/$messagingDir/destinations-service.xml")
		        topicsParser = new Parser()     
		        topicsParser.parse(topicFile)
		
		        //First find the bindings...this contains individual binding elements with jndi names
		        def node = topicsParser.findNode('//server')
		        if (node != null) {
		            Logger.log('DEBUG', "topic - found server")
		            Node existingNode = topicsParser.findNode("//mbean[@name='jboss.messaging.destination:service=Topic,name=$name']")
		            if (existingNode != null) {
		                Logger.log('DEBUG', "topic - found Node for $name! removing...");
		                def textNode = existingNode.getPreviousSibling();
		                if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
		                    Logger.log('DEBUG', "queue - removing previous whitespace text node...");
		                    topicsParser.removeNode(textNode);
		                }
		                topicsParser.removeNode(existingNode);
		              }
		
		              def mbean = topicsParser.addNode("mbean","",node.getFirstChild(), "SIBLING")
		              topicsParser.addTextNode("\n\n   ",mbean,"SIBLING")
		              topicsParser.addAttribute("code","org.jboss.jms.server.destination.TopicService", mbean)
		              topicsParser.addAttribute("name","jboss.messaging.destination:service=Topic,name=$name", mbean)
		              topicsParser.addAttribute("xmbean-dd","xmdesc/Topic-xmbean.xml", mbean)
		              def depends = topicsParser.addNode("depends","jboss.messaging:service=ServerPeer", mbean)
		              topicsParser.addAttribute("optional-attribute-name","ServerPeer", depends)
		              topicsParser.addNode("depends","jboss.messaging:service=PostOffice", mbean)
		              def attribute = topicsParser.addNode("attribute",jndiname, mbean)
		              topicsParser.addAttribute("name","JNDIName", attribute)
		              topicsParser.output(topicFile)
		        } else {
		              Logger.log('DEBUG', "topic - can't find server element, unable to create new topic for $jndiname")
		        }  
			}
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }   
	
}