// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.sdw.ant

import com.sas.config.ContextBuilder
import com.sas.config.Docpart
import com.sas.config.DirectoryResourceLoader
import com.sas.config.TemplateEngine
import com.sas.config.VelocityTemplateEngine
import com.sas.config.sdw.SDW
import com.sas.config.sdw.SDWDocpart
import com.sas.config.sdw.SDWHelper
import com.sas.config.weblogic.WebLogicDeletedResourceLoaderFilter
import com.sas.config.weblogic.WebLogicContextBuilder
import com.sas.config.websphere.docparts.WebSphereContextBuilder
import com.sas.config.jboss.docparts.JBossContextBuilder

public class DocpartTask extends SDWTask {
	private String install_currprodconfig_dir
	private String resourceDirectory
	private String temp_dir
	private String config_lev_instructions_file
	private String webappsrv_container_type
	private String templateFile
	private String token
	
	public void setInstallCurrProdConfigDir(String icpcd) { install_currprodconfig_dir = icpcd }
	public void setResourceDirectory(String rd) { resourceDirectory = rd }
	public void setTempDir(String td) { temp_dir = td }
	public void setContainerType(String ct) { webappsrv_container_type = ct }
	public void setTemplateFile(String tf) { templateFile = tf }
	public void setToken(String t) { token = t }
	public void setConfig_lev_instructions_file(String cf) {config_lev_instructions_file = cf}
	
	protected boolean validateInternal() {
		println ">>> " + getClass().getName() + ".validateInternal()"
		
		SDW sdw = getSDWInstance()

		if(install_currprodconfig_dir == null) {
			install_currprodconfig_dir = sdw.getProperty("install.currprod.config.dir")
		}
		
		if(resourceDirectory == null) {
			resourceDirectory = sdw.getProperty("webappsrv.custom.scripts.props")
		}
		
		if(temp_dir == null) {
			temp_dir = sdw.getProperty("temp.dir")
		}
		
		if(config_lev_instructions_file == null) {
			config_lev_instructions_file = sdw.getProperty("config.lev.instructions.file")
		}
		
		if(webappsrv_container_type == null) {
			webappsrv_container_type = sdw.getProperty("webappsrv.container.type")
		}
		
		if(templateFile == null) {
			templateFile = "$install_currprodconfig_dir/en/${webappsrv_container_type}Instructions.vm"
		}
		
		if(token == null) {
			//Use underscores since there is a bug in SDW's token cleanup.
			token = "\${instructions_webappsrv_details_table_head}"
		}
		
		println "install_currprodconfig_dir = $install_currprodconfig_dir"
		println "resourceDirectory=$resourceDirectory"
		println "temp_dir=$temp_dir"
		println "config_lev_instructions_file=$config_lev_instructions_file"
		println "config_lev_instructions_file=$config_lev_instructions_file"
		println "templatFile=$templateFile"
		println "token=$token"
		
		if(install_currprodconfig_dir == null) return false
		if(resourceDirectory == null) return false
		if(temp_dir == null) return false
		if(config_lev_instructions_file == null) return false
		if(config_lev_instructions_file == null) return false
		if(templateFile == null) return false
		if(token == null) { return false }
		
		webappsrv_container_type = webappsrv_container_type.toLowerCase()		
		println "<<< " + getClass().getName() + ".validateInternal() - return true"
		return true;
	}
	
	protected void executeInternal() {
		println ">>> " + getClass().getName() + ".executeInternal()"
		SDW sdw = getSDWInstance()
		SDWHelper sdwHelper = new SDWHelper(sdw)
						
		SDWDocpart sdwDocpart = new SDWDocpart(sdw)
		sdwDocpart.setContextBuilder(getContextBuilderInstance())

		File docpartFile = new File("$temp_dir/${webappsrv_container_type}Instructions.html")
		sdwDocpart.create(new File(templateFile), docpartFile)

		sdwDocpart.insert(new File(config_lev_instructions_file), token, docpartFile)
		//docpartFile.delete()

		println "<<< " + getClass().getName() + ".executeInternal()"		
	}
	
	protected ContextBuilder getContextBuilderInstance() {
	    println ">>> " + getClass().getName() + "getContextBuilderInstance()"
				
		switch(webappsrv_container_type) {
			case "weblogic":
				DirectoryResourceLoader directoryResourceLoader = new DirectoryResourceLoader(new File("$resourceDirectory"))
				directoryResourceLoader.setFilter(new WebLogicDeletedResourceLoaderFilter())		
			
				WebLogicContextBuilder contextBuilder = new WebLogicContextBuilder()
				contextBuilder.loadResources(directoryResourceLoader)
				return contextBuilder
			case "websphere":
				DirectoryResourceLoader directoryResourceLoader = new DirectoryResourceLoader(new File("$resourceDirectory"))
				// Enabling the filter will filter out all WebSphere property files.
				//  directoryResourceLoader.setFilter(new WebSphereDeletedResourceLoaderFilter())        
					
				WebSphereContextBuilder contextBuilder = new WebSphereContextBuilder("configure")
				contextBuilder.loadResources(directoryResourceLoader)
				return contextBuilder
				
			case "jboss":
				//DirectoryResourceLoader directoryResourceLoader = new DirectoryResourceLoader(new File("$resourceDirectory"))
				// Enabling the filter will filter out all jboss property files.
				//  directoryResourceLoader.setFilter(new WebSphereDeletedResourceLoaderFilter())        
		        

 
				ContextBuilder contextBuilder = new JBossContextBuilder("configure")
				contextBuilder.setConfigPropertiesFile("$resourceDirectory/jbossScripting.properties")
				return contextBuilder;
				
			default:
				throw new UnsupportedOperationException("$webappsrv_container_type not supported!")
				break
		}
		println "<<< " + getClass().getName() + "getContextBuilderInstance()"
		
	}
}
