// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic;

import java.io.File;
import com.sas.config.Resource

class Datasource extends WebLogicResource {
	public static final String CONNECTION_URL = "databaseServerURL"
	public static final String PROPERTIES = "options"
	public static final String CLASSPATH = "classPath"	
	public static final String SECURE = "requireUsernamePassword"
		
	public Datasource() {}
	public Datasource(File propertiesFile) {
		super(propertiesFile)
	}	
	
	public String getHostname() {
		String url = getProperty(CONNECTION_URL)
		String hostname = null
		def matcher = (url =~ /.*:\/\/([^:]*):.*$/)
		if(matcher.find()) { hostname = matcher.group(1) }
		matcher = (url =~ /.*:@([^:]*):.*$/)
		if(matcher.find()) { hostname = matcher.group(1) }	
		return hostname
	}
	
	public String getPort() {
		String url = getProperty(CONNECTION_URL)
		String port = null
		def matcher = (url =~ /.*:\/\/[^:]*:([0-9]*).*$/)
		if(matcher.find()) { port = matcher.group(1) }
		matcher = (url =~ /.*:@[^:]*:([0-9]*).*$/)
		if(matcher.find()) { port = matcher.group(1) }		
		return port
	}
	
	public String getProperties() {
		String options = getProperty(PROPERTIES)
		StringBuilder properties = new StringBuilder()
		
		if(options != null && options != "not-set") {
			options.split(",").each() { option ->
				if(properties.length() > 0) {
					properties.append("\n")
				}
				properties.append(option.trim())
			}
		}
		
		if(properties.length() > 0) {
			return properties.toString()
		}
		
		return null
	}
	
	public List<String> getClasspath() {
		List<String> classpath = new ArrayList<String>()
		String path = getProperty(CLASSPATH)
		if(path != null) {
			path.split(";").each() { jar ->
				classpath.add(jar)
			}
		}		
		return classpath
	}
}
