// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic;

import java.io.File;

import com.sas.config.Resource

import com.sas.util.SasPasswordString

class MailSession extends WebLogicResource {
	public static final String USER = "mailSessionUser"
	public static final String PASSWORD = "mailSessionPassword"
	public static final String REQUIRES_AUTHENTICATION = "mailsrvRequiresAuthentication"
	
	public MailSession() {}
	public MailSession(File propertiesFile) {
		super(propertiesFile)
	}
	
	public void setUser(String user) { 
		setProperty(USER, user) 
	}
	
	public void setPassword(String password) { 
		setProperty(PASSWORD, password)
	}
	
	public String buildMailSessionProperties(boolean hideCrendentials) {
		return buildMailSessionProperties(hideCrendentials, ";")
	}
	
	public String buildMailSessionPropertiesForDoc(boolean hideCrendentials) {
		return buildMailSessionProperties(hideCrendentials, "\n")
	}	
	
	public String buildMailSessionProperties(boolean hideCredentials, String delimiter) {
		StringBuilder builder = new StringBuilder()
		builder.append("mail.debug=false")
		builder.append("${delimiter}mail.smtp.host=" + (String) getProperty("mailSessionSMTPHost"))
		builder.append("${delimiter}mail.transport.protocol=smtp")
		
		String user = (String) getProperty(USER)
		String password = (String) getProperty(PASSWORD)
		boolean requiresAuthentication = Boolean.parseBoolean((String) getProperty(REQUIRES_AUTHENTICATION))
		
		if((user != null && user.length() > 0) || requiresAuthentication) {
			builder.append("${delimiter}mail.user=" + ((hideCredentials) ? "********" : user))
		}
		
		if((password != null && password.length() > 0)  || requiresAuthentication) {
			builder.append("${delimiter}mail.password=" + ((hideCredentials) ? "********" : SasPasswordString.decode(password)))		
		}
		
		return builder.toString()
	}
}
