// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import  com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: Credentials
// Purpose: Encapsulate the attributes and methods used to manipulate the WAS credentials.properties file
// Methods:
//		clear  - set values for all userId/password key names to null 
// ****************************************************************************************************************

class Credentials extends Resource {
	// Class variables available to all methods:
	def resourcePropsFileName
	
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties
	// Global properties are fetched and initialized by the base Resource class
	public Credentials(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
	}
	
	//**************************************************************************************
	// Method to encode all "_passwd" entries in the websphere.scripting.credentials file
	public int encodeAll(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the credentials.properties file for msgs.
		logger.info("Credentials-encode: Ensure all passwords in the \"${resourcePropsFileName}\" file are SAS-encoded...")
		method_RC = resourceProps.encodeAll()		// Encode the values for all "_passwd" properties in the file
		logger.info("Credentials-encode: Update of \"${resourcePropsFileName} Return Code: " + method_RC)
		
		if (method_RC == 1)  {
			method_MSG =  ": ===> Credentials.encodeAll() COMPLETED SUCCESSFULLY; \"${resourcePropsFileName}\"; passwords have been encoded."
			logger.info("${method_MSG}")
		} else {
			method_MSG = "**SEVERE** : Credentials.encodeAll() FAILED to encode passwords and save \"${resourcePropsFileName}\" properties file."
			method_RC = -99
		}
		return method_RC
	}
	
	// ****************************************************************************************************
	// Method to clear the WAS credentials.properties file by nullifying all the values for all properties.
	public int clear(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the credentials.properties file for msgs.
		logger.info("Credentials-clear: Nullify all property values for the \"${resourcePropsFileName}\" file...")
		method_RC = resourceProps.clearAll()		// Nullify the values for all properties in the file
		logger.info("Credentials-clear: Update of \"${resourcePropsFileName} Return Code: " + method_RC)

		if (method_RC == 1) {	// Return code of one indicates credentials.properties file was cleared and saved successfully
			method_MSG =  ": ===> Credentials.clear() COMPLETED SUCCESSFULLY; \"${resourcePropsFileName}\" has been cleared and updated."
			logger.info("${method_MSG}")
		} else {
			method_MSG = "**SEVERE** : Credentials.clear() FAILED to clear and update \"${resourcePropsFileName}\" properties file."
			method_RC = -99
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END clear method

}	// END Credentials class