// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import  com.sas.config.util.ResourceProperties
import org.apache.log4j.Logger
import org.apache.log4j.PropertyConfigurator

public class GlobalProperties {
	final static Logger logger = Logger.getLogger("com.sas.config");	// Instantiate singleton of log4j object for logging
	// Class variables loaded from the Global Properties file:
	// The following class variables should be accessible using a getter:	
	ResourceProperties globalProps			// Object for global properties	
	String  globalPropsFileName				// Filename for global properties file
	String  scriptingDirectory				// Scripting directory path
	Boolean DEBUG							// DEBUG flag 
	Boolean DEBUG_WASDriver					// DEBUG flag for the WAS driver program
	String  File_separator					// File separation character for use in Velocity scripts
	String  appsrvnodename					// WAS managed node name
	String  appsrvnodermiport				// WAS nodeagent RMI port number
	String  appsrvnodesoapport				// WAS nodeagent SOAP port number
	String  cellname						// WAS cell name
	String  config_lev_web_common_dir		// SDW-provided <SAS-Config>/Lev1/Web directory
	String  config_lev_web_staging_dir		// SDW-provided <SAS-Config>/Lev1/Web/Staging directory
	String  defaultsibusname				// WAS default SIBus name
	String  dmgr							// WAS dmgr server name
	String  dmgrhttpport					// WAS dmgr HTTP port
	String  dmgrhttpsport					// WAS dmgr HTTPS port
	String  dmgrnodename					// WAS dmgr node name
	String  dmgrport						// WAS SOAP or RMI port used for communications between node and dmgr
	String  dmgrprofilename					// WAS dmgr profile name
	String  dmgrprotocol					// WAS dmgr communications protocol (SOAP or RMI) used between node and dmgr
	String  dmgrrmiport						// WAS dmgr RMI port number
	String  dmgrrmiprops					// WAS dmgr RMI client properties filename ("sas.client.props")
	String  dmgrrmitimeout					// WAS dmgr RMI timeout value
	String  dmgrsoapport					// WAS dmgr SOAP port number
	String  dmgrsoapprops					// WAS dmgr SOAP client properties filename ("soap.client.props")
	String  dmgrsoaptimeout					// WAS dmgr SOAP timeout value
	String  dmgripcprops					// WAS dmgr IPC client properties filename ("ipc.client.props")
	String  dmgripctimeout					// WAS dmgr IPC timeout value
	Boolean mustCreateDmgrProfile			// WAS create dmgr profile if "true", otherwise, unnecessary
	String  nodeagent						// WAS name of nodeagent server
	String  nodermiprops					// WAS node RMI properties file
	String  nodermitimeout					// WAS node RMI timeout value
	String  nodesoapprops					// WAS node SOAP properties file
	String  nodesoaptimeout					// WAS node SOAP timeout value
	String  nodeipcprops					// WAS node IPC properties file
	String  nodeipctimeout					// WAS node IPC timeout value
	String  os_localhost_default_locale 	// Locale ("en" or "ja", "zn", etc.)
	String  os_localhost_machine_type		// Machine architecture ("lax", "win", "wx6", "r64", "sax", "s64", and "zos")	
	String  os_localhost_type				// Host type ("unx", "win" or "zos")
	String  profilename						// WAS node profile name
	String  script_executable				// Script executable pgm "cmd" or "/bin/sh" or "/bin/ksh"
	String  script_ext						// Script extension ".bat" or ".sh"	
	String  script_flag						// Script flag "/c" or "-c"
	Boolean webapp_auto_deploy				// SDW auto-deploy (true) or manual deploy (false)
	String  webappsrv_admin_host			// WAS Admin dmgr host
	Boolean webappsrv_admin_security_is_enabled	// WAS Admin Security is enabled (true) or disabled (false)
	Boolean webappsrv_auto_config			// SDW auto-config (true) or manual config (false)
	Boolean webappsrv_scripting_cache_credentials	// Whether it is ok to save userIds and passwords in a file
	Boolean webappsrv_policy_use_restrictive	// WAS Java2 Security is enforced (true)
	String  websphere_appsrv_logs			// WAS node logs
	String  websphere_dmgr_logs				// WAS dmgr logs
	String  websphere_full_version			// WAS full version with fixpack (e.g. "7.0.0.11")	
	String  websphere_install_dir			// WAS install directory
	String  websphere_profile_dir			// WAS profiles directory
	String  websphere_scripting_classpath	// Scripting classpath for Groovy compiles and runtime
	String  websphere_scripting_dir			// Scripting directory name
	String  websphere_scripting_jython		// Scripting Jython scripts location
	String  websphere_scripting_lib			// Scripting Java lib containing all necessary jars
	String  websphere_scripting_logs		// Scripting logs directory
	String  websphere_scripting_props		// Scripting properties directory containing the global and resource properties files
	String  websphere_scripting_scripts		// Scripting scripts to drive the WAS config
	String  websphere_scripting_src			// Scripting Groovy source directory
	String  websphere_scripting_tasks		// Scripting tasks directory containing the websphere.configuration.tasks file
	String  websphere_scripting_tasks_file	// Scripting default task file full path name
	String	websphere_scripting_credentials	// Scripting filename for WAS credentials file
	String  websphere_temp_dir				// SDW temporary directory for WebSphere 
	String  websphere_wsadmin_classpath		// Classpath for use by wsadmin command
	String  websphere_wsadmin_jython		// Location of SAS Jython code for WAS configs
	Boolean webspherend_is_installed		// WAS is installed (true) or not installed (false)
	def		wsadminOptionsList				// ArrayList of common wsadmin options
	String  wsadminProps					// WAS wsadmin connection command-line parameters

	//**************************************
	// CTOR for GlobalProperties class
	//**************************************
	public GlobalProperties(String fn) {	
		globalProps = new ResourceProperties(fn)	// Load the global properties from the provided Global Properties File Name
		globalPropsFileName = fn					// Save global properties file name
		scriptingDirectory = globalProps.getProperty("scriptingDirectory")	// Scripting directory path
		websphere_scripting_props = globalProps.getProperty("websphere.scripting.props")	// Scripting directory path for properties files		
		logger.info("GlobalProperties: Load all global property values from \"${globalPropsFileName}\"...")
		DEBUG = globalProps.getBooleanProperty("DEBUG")						// DEBUG flag
		if (DEBUG == null)
			DEBUG = false 
		logger.info("GlobalProperties: Debugging requested? ${DEBUG}")		// Write info message about debugging status
		DEBUG_WASDriver = globalProps.getBooleanProperty("DEBUG_WASDriver")	// DEBUG flag for WAS driver program
		appsrvnodename = globalProps.getProperty("appsrvnodename")			// WAS managed node name
		appsrvnodermiport = globalProps.getProperty("appsrvnodermiport") 	// WAS nodeagent RMI port number
		appsrvnodesoapport = globalProps.getProperty("appsrvnodesoapport")	// WAS nodeagent SOAP port number
		cellname = globalProps.getProperty("cellname")						// WAS cell name
		defaultsibusname = globalProps.getProperty("defaultsibusname")		// WAS default SIBus name
		wsadminProps = globalProps.getProperty("wsadminProps")				// WAS wsadmin connection parameters
		mustCreateDmgrProfile = globalProps.getBooleanProperty("mustCreateDmgrProfile")	// WAS create dmgr profile if "true", otherwise unnecessary 
		dmgr = globalProps.getProperty("dmgr")								// WAS dmgr server name
		dmgrnodename = globalProps.getProperty("dmgrnodename")				// WAS dmgr node name
		dmgrprofilename = globalProps.getProperty("dmgrprofilename")		// WAS dmgr profile name		
		dmgrhttpport = globalProps.getProperty("dmgrhttpport")				// WAS dmgr HTTP port
		dmgrhttpsport = globalProps.getProperty("dmgrhttpsport")			// WAS dmgr HTTPS port
		dmgrport = globalProps.getProperty("dmgrport")						// WAS SOAP or RMI port used for communications between node and dmgr						
		dmgrprotocol = globalProps.getProperty("dmgrprotocol")				// WAS dmgr communications protocol (SOAP or RMI) used between node and dmgr
		dmgrrmiport = globalProps.getProperty("dmgrrmiport")				// WAS dmgr RMI port number
		dmgrrmiprops = globalProps.getProperty("dmgrrmiprops")				// WAS dmgr RMI client properties filename ("sas.client.props")
		dmgrrmitimeout = globalProps.getProperty("dmgrrmitimeout")			// WAS dmgr RMI timeout value
		dmgrsoapport = globalProps.getProperty("dmgrsoapport")				// WAS dmgr SOAP port number
		dmgrsoapprops = globalProps.getProperty("dmgrsoapprops")			// WAS dmgr SOAP client properties filename ("soap.client.props")
		dmgrsoaptimeout = globalProps.getProperty("dmgrsoaptimeout")		// WAS dmgr SOAP timeout value
		dmgripcprops = globalProps.getProperty("dmgripcprops")				// WAS dmgr IPC client properties filename ("ipc.client.props")
		dmgripctimeout = globalProps.getProperty("dmgripctimeout")			// WAS dmgr IPC timeout value
		nodeagent = globalProps.getProperty("nodeagent")					// WAS name of nodeagent server
		nodermiprops = globalProps.getProperty("nodermiprops")				// WAS node RMI properties file
		nodermitimeout = globalProps.getProperty("nodermitimeout")			// WAS node RMI timeout value
		nodesoapprops = globalProps.getProperty("nodesoapprops")			// WAS node SOAP properties file
		nodesoaptimeout = globalProps.getProperty("nodesoaptimeout")		// WAS node SOAP timeout value
		nodeipcprops = globalProps.getProperty("nodeipcprops")				// WAS node IPC properties file
		nodeipctimeout = globalProps.getProperty("nodeipctimeout")			// WAS node IPC timeout value
		os_localhost_default_locale = globalProps.getProperty("os.localhost.default.locale")	// Locale ("en" or "ja", "zn", etc.)	
		os_localhost_type = globalProps.getProperty("os.localhost.type")	// Host type ("unx", "win" or "zos")
		os_localhost_machine_type = globalProps.getProperty("os.localhost.machine.type")	// Machine architecture ("lax", "win", "wx6", "r64", "sax", "s64", and "zos")  
		profilename = globalProps.getProperty("profilename")				// WAS node profile name
		script_executable = globalProps.getProperty("script.executable")	// Script executable pgm "cmd" or "/bin/sh" or "/bin/ksh"
		script_ext = globalProps.getProperty("script.ext")					// Script extension ".bat" or ".sh"	
		script_flag = globalProps.getProperty("script.flag")				// Script flag "/c" or "-c"
		webappsrv_admin_security_is_enabled = globalProps.getBooleanProperty("webappsrv.admin.security.is_enabled")	// WAS Admin Security is enabled (true) or disabled (false)
		webappsrv_scripting_cache_credentials = globalProps.getBooleanProperty("webappsrv.scripting.cache_credentials")	// Whether it is ok to save userIds and passwords in a file
		webappsrv_policy_use_restrictive = globalProps.getBooleanProperty("webappsrv.policy.use_restrictive")	// WAS Java2 Security is enforced (true)	
		webappsrv_auto_config = globalProps.getBooleanProperty("webappsrv.auto_configure")	// SDW auto-config (true) or manual config (false)
		webspherend_is_installed = globalProps.getBooleanProperty("webspherend.is_installed")	// WAS is installed (true) or not installed (false)
		webapp_auto_deploy = globalProps.getBooleanProperty("webapp.auto_deploy")			// SDW auto-deploy (true) or manual deploy (false)			
		webappsrv_admin_host = globalProps.getProperty("webappsrv.admin.host")		// WAS Admin dmgr host
		websphere_appsrv_logs = globalProps.getProperty("websphere.appsrv.logs")	// WAS node logs location
		websphere_dmgr_logs = globalProps.getProperty("websphere.dmgr.logs")		// WAS dmgr logs location
		websphere_full_version = globalProps.getProperty("websphere.full.version")	// WAS full version with fixpack (e.g. "7.0.0.11")	
		websphere_install_dir = globalProps.getProperty("websphere.install.dir")	// WAS install directory
		websphere_profile_dir = globalProps.getProperty("websphere.profile.dir")	// WAS profiles directory
		websphere_scripting_dir = globalProps.getProperty("websphere.scripting.dir")	// Scripting directory name
		websphere_scripting_lib = globalProps.getProperty("websphere.scripting.lib")	// Scripting Java lib containing all necessary jars
		websphere_scripting_logs = globalProps.getProperty("websphere.scripting.logs")	// Scripting logs directory
		websphere_scripting_props = globalProps.getProperty("websphere.scripting.props")	// Scripting properties directory containing the global and resource properties files
		websphere_scripting_src = globalProps.getProperty("websphere.scripting.src")	// Scripting Groovy source directory
		websphere_scripting_scripts = globalProps.getProperty("websphere.scripting.scripts") 	// Scripts used to drive WAS config
		websphere_scripting_jython = globalProps.getProperty("websphere.scripting.jython")	// Scripting Jython scripts location
		websphere_scripting_tasks = globalProps.getProperty("websphere.scripting.tasks")	// Scripting tasks directory containing the websphere.configuration.tasks file
		websphere_scripting_tasks_file = globalProps.getProperty("websphere.scripting.tasks.file")	// Scripting default tasks file full path name
		websphere_scripting_classpath = globalProps.getProperty("websphere.scripting.classpath")	// Scripting Groovy compile and runtime classpath
		websphere_scripting_credentials = globalProps.getProperty("websphere.scripting.credentials")	// Scripting filename for WAS credentials file	
		websphere_temp_dir = globalProps.getProperty("websphere.temp.dir")	// SDW temporary directory for WebSphere 
		websphere_wsadmin_classpath	= globalProps.getProperty("websphere.wsadmin.classpath")	// Classpath for use by wsadmin command
		websphere_wsadmin_jython = globalProps.getProperty("websphere.wsadmin.jython")	// Location of SAS Jython code for WAS configs
		config_lev_web_common_dir = globalProps.getProperty("config.lev.web.common.dir")	// SDW-provided <SAS-Config>/Lev1/Web directory
		config_lev_web_staging_dir = globalProps.getProperty("config.lev.web.staging.dir")	// SDW-provided <SAS-Config>/Lev1/Web/Staging directory
		File_separator = globalProps.getProperty("File_separator")	// File separator character for use in Velocity scripts
		wsadminOptionsList = ["-wsadmin_classpath", "${websphere_wsadmin_classpath}", "-conntype", "${dmgrprotocol}", "-port", "${dmgrport}", "-host", "${webappsrv_admin_host}", "-lang", "jython"]					// ArrayList of common wsadmin options
	}	// END CTOR

	// Dump the values of all global properties
	public void printAll() {
		logger.debug("GlobalProperties: ============= Begin All Property Values ============")
		logger.debug("GlobalProperties: globalPropsFileName=${globalPropsFileName}")		
		logger.debug("GlobalProperties: scriptingDirectory=${scriptingDirectory}")
		logger.debug("GlobalProperties: websphere_scripting_props=${websphere_scripting_props}") 
		logger.debug("GlobalProperties: DEBUG=${DEBUG}")						
		logger.debug("GlobalProperties: DEBUG_WASDriver=${DEBUG_WASDriver}")		
		logger.debug("GlobalProperties: appsrvnodename=${appsrvnodename}")
		logger.debug("GlobalProperties: appsrvnodermiport=${appsrvnodermiport}")
		logger.debug("GlobalProperties: appsrvnodesoapport${appsrvnodesoapport}")
		logger.debug("GlobalProperties: cellname=${cellname}")			
		logger.debug("GlobalProperties: defaultsibusname=${defaultsibusname}")
		logger.debug("GlobalProperties: wsadminProps=${wsadminProps}")	
		logger.debug("GlobalProperties: mustCreateDmgrProfile=${mustCreateDmgrProfile}")
		logger.debug("GlobalProperties: dmgr=${dmgr}")						
		logger.debug("GlobalProperties: dmgrnodename=${dmgrnodename}")			
		logger.debug("GlobalProperties: dmgrprofilename=${dmgrprofilename}")		
		logger.debug("GlobalProperties: dmgrhttpport=${dmgrhttpport}")		
		logger.debug("GlobalProperties: dmgrhttpsport=${dmgrhttpsport}")
		logger.debug("GlobalProperties: dmgrport=${dmgrport}")							
		logger.debug("GlobalProperties: dmgrprotocol=${dmgrprotocol}")	
		logger.debug("GlobalProperties: dmgrrmiport=${dmgrrmiport}")		
		logger.debug("GlobalProperties: dmgrrmiprops=${dmgrrmiprops}")			
		logger.debug("GlobalProperties: dmgrrmitimeout=${dmgrrmitimeout}")		
		logger.debug("GlobalProperties: dmgrsoapport=${dmgrsoapport}")
		logger.debug("GlobalProperties: dmgrsoapprops=${dmgrsoapprops}")
		logger.debug("GlobalProperties: dmgrsoaptimeout=${dmgrsoaptimeout}")
		logger.debug("GlobalProperties: dmgripcprops=${dmgripcprops}")
		logger.debug("GlobalProperties: dmgripctimeout=${dmgripctimeout}")
		logger.debug("GlobalProperties: nodeagent=${nodeagent}")
		logger.debug("GlobalProperties: nodermiprops=${nodermiprops}")
		logger.debug("GlobalProperties: nodermitimeout=${nodermitimeout}")
		logger.debug("GlobalProperties: nodesoapprops=${nodesoapprops}")		
		logger.debug("GlobalProperties: nodesoaptimeout=${nodesoaptimeout}")
		logger.debug("GlobalProperties: nodeipcprops=${nodeipcprops}")
		logger.debug("GlobalProperties: nodeipctimeout=${nodeipctimeout}")
		logger.debug("GlobalProperties: os_localhost_default_locale=${os_localhost_default_locale}")	
		logger.debug("GlobalProperties: os_localhost_type=${os_localhost_type}")
		logger.debug("GlobalProperties: os_localhost_machine_type=${os_localhost_machine_type}")
		logger.debug("GlobalProperties: profilename=${profilename}")				
		logger.debug("GlobalProperties: script_executable=${script_executable}")
		logger.debug("GlobalProperties: script_ext=${script_ext}")	
		logger.debug("GlobalProperties: script_flag=${script_flag}")		
		logger.debug("GlobalProperties: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
		logger.debug("GlobalProperties: webappsrv_policy_use_restrictive=${webappsrv_policy_use_restrictive}")	
		logger.debug("GlobalProperties: webappsrv_auto_config=${webappsrv_auto_config}")
		logger.debug("GlobalProperties: webspherend_is_installed=${webspherend_is_installed}")
		logger.debug("GlobalProperties: webapp_auto_deploy=${webapp_auto_deploy}")
		logger.debug("GlobalProperties: webappsrv_admin_host=${webappsrv_admin_host}")
		logger.debug("GlobalProperties: webappsrv_scripting_cache_credentials=${webappsrv_scripting_cache_credentials}")
		logger.debug("GlobalProperties: websphere_appsrv_logs=${websphere_appsrv_logs}") 
		logger.debug("GlobalProperties: websphere_dmgr_logs=${websphere_dmgr_logs}")
		logger.debug("GlobalProperties: websphere_full_version=${websphere_full_version}")
		logger.debug("GlobalProperties: websphere_install_dir=${websphere_install_dir}")
		logger.debug("GlobalProperties: websphere_profile_dir=${websphere_profile_dir}")
		logger.debug("GlobalProperties: websphere_scripting_dir=${websphere_scripting_dir}")
		logger.debug("GlobalProperties: websphere_scripting_scripts=${websphere_scripting_scripts}")
		logger.debug("GlobalProperties: websphere_scripting_lib=${websphere_scripting_lib}")
		logger.debug("GlobalProperties: websphere_scripting_logs=${websphere_scripting_logs}")
		logger.debug("GlobalProperties: websphere_scripting_props=${websphere_scripting_props}")
		logger.debug("GlobalProperties: websphere_scripting_scripts=${websphere_scripting_scripts}")
		logger.debug("GlobalProperties: websphere_scripting_src=${websphere_scripting_src}")
		logger.debug("GlobalProperties: websphere_scripting_tasks=${websphere_scripting_tasks}")
		logger.debug("GlobalProperties: websphere_scripting_tasks_file=${websphere_scripting_tasks_file}")
		logger.debug("GlobalProperties: websphere_scripting_jython=${websphere_scripting_jython}")
		logger.debug("GlobalProperties: websphere_scripting_classpath=${websphere_scripting_classpath}")
		logger.debug("GlobalProperties: websphere_scripting_credentials=${websphere_scripting_credentials}")
		logger.debug("GlobalProperties: websphere_temp_dir=${websphere_temp_dir}")
		logger.debug("GlobalProperties: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
		logger.debug("GlobalProperties: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
		logger.debug("GlobalProperties: config_lev_web_common_dir=${config_lev_web_common_dir}")
		logger.debug("GlobalProperties: config_lev_web_staging_dir=${config_lev_web_staging_dir}")
		logger.debug("GlobalProperties: File_separator=${File_separator}")
		int i=0
		wsadminOptionsList.each { opt-> 
			logger.debug("GlobalProperties: wsadminOptionsList["+(i++)+"]=${opt}") 
		}		
		logger.debug("GlobalProperties: ============= End   All Property Values ============")	
	}	//END printAll method
		
	// Update a particular property in the global properties object 
	public void setPropertyValue(String name, Object value)  {
		globalProps.setPropertyValue(name, value)
	}	// END setProperty method
	
	// Write the properties to ensure any updates are safe
	public int writeProps() {
		globalProps.writeProps()
	}	// END writeProps method

	// Load all Global Properties
	public int loadProps() {
		return globalProps.load()
	}	// END loadProps method
		
}	// END GlobalProperties class