// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere.docparts;

import java.util.regex.*;

/* Create a list of property maps that share a property setting. */
class ResourceFinder {
    
    private HashMap context
	private ArrayList found
	
    public ResourceFinder(HashMap context) {
        this.context = context
        println "ResourceFinder: context = " + this.context
    }
	
	public ArrayList getResources(String contextKey, String propName, String propValue) { 
		found = new ArrayList()
        Object temp = context[contextKey]
        List contextList = null
        
        if (temp instanceof List) {
            contextList = (List) temp
            for (int i = 0; i < contextList.size(); i++) {
                if (contextList[i][propName].equals(propValue))
                    found.add(contextList[i])
            }
        } else {
            println "ResourceFinder: " + context[contextKey] + " is not a List."
            return null
		}
        println "ResourceFinder: found = " + found
		return found
    }
}

/* Create a property map of CustomPropertyTable input. */
class CustomProperties {
    /*
     * Each custom property name entry contains a property name and optional 
     * property display name.
     */
    private static HashMap loginModulePropertyNames = 
       [ "JaasHost":"host",
         "JaasPort":"port",
         "JaasRepository":"repository",
         "JaasDomain":"domain",
         "JaasAliasDomain":"aliasdomain",
         "JaasDebug":"debug",
         "JaasHoldOpenConnection":"holdopenconnection" ]
       
    private static HashMap dataSourcePropertyNames =     
       [ "pDsHost":"serverName",
         "pDsPort":"portNumber",
         "pDsConnectionUrl":"serverUrl",
         "pDsDataSourceName":"dataSourceName",
         "pDsOptions":"options" ]

    private HashMap customProperties = null;   
       
	public CustomProperties() {}
    
    public HashMap getCustomProperties(String resType, HashMap properties) {
        HashMap customPropertyNames = null;
        String propertyName = null;
        String propertyValue = null;
        String displayName = null;
        Set keys = null;

        // Allocate the custom properties map for output.     
        customProperties = new HashMap();
        // Select the custom property names to use.
        switch(resType) {
            case "DataSource": 
                customPropertyNames = dataSourcePropertyNames;
                break;
            case "LoginModule":
                customPropertyNames = loginModulePropertyNames;
                break;
            default: 
                return null;
        }
        // Fill the custom properties map.
        keys = customPropertyNames.keySet();
        for (key in keys) {
            // Select the key used to add the custom property. 
            displayName = customPropertyNames[key];
            if ((displayName == null)           ||
               !(displayName instanceof String) ||
                (displayName.equals(""))) {
                propertyName = key;
            } else {
                propertyName = displayName;
            }
            // Perform special processing of custom properties.
            propertyValue = properties[key];
            if ((propertyValue) == null           ||
               !(propertyValue instanceof String) ||
                (propertyValue.equals(""))) {
                propertyValue = null;
            } else {
                switch(key) {
                    case "pDsOptions":
                        addDSOptions(propertyValue);
                        break;
                    default:
                        customProperties[propertyName] = propertyValue;
                    }
            }
            println "CustomProperties: name = " + propertyName + ", value = " + propertyValue;
        }
        return customProperties;
    }
    
    public ArrayList getCustomPropertyKeys(HashMap properties) {
        // Return a sorted list of custom property keys. 
        Set keys = properties.keySet();
        ArrayList keys_arr = new ArrayList(keys).sort();
        println "Custom keys: " + keys_arr;
        return keys_arr;
    }
    
    public String getCustomPropertyValue(String key, HashMap _properties_) {
        // Get the value of a custom property.
        return _properties_[key];
    }    
 
    public setCustomPropertyValue(String key, String value, HashMap _properties_) {
        // Set the value of a custom property.
        _properties_[key] = value;
        println "_properties_[" + key + "] = " + _properties_[key];
        return;
    }
     
    private addDSOptions(String dsoptions) {
        String[] optionArray = null;
        String regex = "([\\w]|[\\w\\_]|[\\w\\-]*)(\\=)(.*)"; 
        Matcher matcher = null;
        String propertyName = null;
        String propertyValue = null;
        
        // Split into individual options.
        optionArray = dsoptions.split(",");
        for (option in optionArray) {
            // Get each option name and value.
            matcher = (option =~ regex);
            if (matcher.matches()) {
                propertyName = matcher[0][1];
                propertyValue = matcher[0][3];
                if ((propertyValue == null)           ||
                   !(propertyValue instanceof String) ||
                    (propertyValue.equals(""))) {
                } else {
                    customProperties[propertyName] = propertyValue;
                }
            }
        }   
    }
}

/* Access Java System properties. */
class SystemProperties {
	
	public SystemProperties() {}
	
	String getSystemPropertyValue(String key) {
		return System.getProperty(key)
	}
}