// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere.docparts;

import java.util.List;
import java.util.Map;

import com.sas.config.Resource;

class WebSphereResource extends Resource {
	//	public static final String TARGETS = "targets"
	//	public static final String DELETED = "deleted"
	public static final String NOT_SET = "not-set"
	
	public WebSphereResource() {
		super()
	}
	
	public WebSphereResource(Resource r) {
		super(r)
	}

	public WebSphereResource(File propertiesFile) {
		super(propertiesFile)
	}	

	@Override
	public void setProperty(String name, Object value) {
		if(value != NOT_SET) {
			this.put(replacePeriodsWithUnderscore(name), value)
		}
	}
	
	public String getScopeString(WebSphereResource globalProps) { 
		def cellname = globalProps.getProperty("cellname")
		def nodename = globalProps.getProperty("appsrvnodename")
		if (getProperty("scope") == "CELL")
			return cellname 
		if (getProperty("scope") == "NODE")
			return cellname + "." + nodename
		if (getProperty("scope") == "SERVER")
			return cellname + "." + nodename + "." + getProperty("servername")
	}
	
	public String getScopeSetting(WebSphereResource globalProps) {
		def cellname = globalProps.getProperty("cellname")
		def nodename = globalProps.getProperty("appsrvnodename")
		if (getProperty("scope") == "CELL")
			return "Cell=" + cellname
		if (getProperty("scope") == "NODE")
			return "Node=" + nodename
		if (getProperty("scope") == "SERVER")
			return "Node=" + nodename + ", Server=" + getProperty("servername")
	}
	
	public String getHeapSize(String jvmArgs, String pattern)  {
		def heapSize = (jvmArgs =~ pattern)
		return heapSize[0][1]
	}

	private String replacePeriodsWithUnderscore(String propertyName) {
		if(propertyName != null) {
			propertyName = propertyName.replaceAll('\\.', '_')
		}
		return propertyName
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder()
		sb.append("\n\n*****")
		for(String key in this.keySet()) {
			sb.append("$key=" + this.getProperty(key) + "\n")
		}
		sb.append("*****\n\n")
		return sb.toString()
	}	
}
