# *************************************************************************
# This script is used to initialize common environment to start WebLogic
# Server, as well as WebLogic development.  This is a wrapper for the
# installed version of commEnv.cmd that is needed for use with the SAS 
# automatically configured environment.
# 
# SAS provides custom versions of all BEA scripts that are used for SAS 
# configuration. In this way, the SAS installers don't have to modify your BEA 
# Weblogic installation.

JAVA_VENDOR=${global.javaVendor}
JAVA_HOME=${global.javaHome}
JAVA_OPTIONS="${global.bitwidthOption} $JAVA_OPTIONS ${global.jdkBugWorkaround} ${global.midtierMulticastIpJavaOptions}"
ANT_OPTS="${global.bitwidthOption} ${global.jdkBugWorkaround}"
PRODUCTION_MODE=true
WEBLOGIC_VERSION=${global.weblogicSelectedVersion}
OS_LINUX_VENDOR=${global.osLinuxVendor}
OS_LINUX_VERSION=${global.osLinuxVersion}

# commEnv.sh always limits the number of open file descriptors to a finite number - never unlimited.
# The default value in commEnv.sh is too small so we need to set it to a larger value before calling commEnv.sh
resetFd() {
  if [ ! -n "`uname -s |grep -i cygwin || uname -s |grep -i windows_nt || \
              uname -s |grep -i HP-UX`" ]
  then
    maxfiles=`ulimit -H -n`
    if [ "$?" = "0" -a  `expr ${maxfiles} : '[0-9][0-9]*$'` -eq 0 ]; then
        ulimit -n ${global.minimumUlimit}
        maxfiles=${global.minimumUlimit}
    fi
    # always set the soft limit to match the hard limit
    ulimit -S -n ${maxfiles}
  fi
}

resetFd

# Remember initial library path on AIX and Solaris on Sparc
case `uname -s` in
AIX)
  if [ -n "${LIBPATH}" ]; then
    SAVE_LIBPATH=${LIBPATH}
  fi
;;
SunOS)
  arch=`uname -m`
  if [ -n "${LD_LIBRARY_PATH}" ]; then
    SAVE_LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
  fi
;;
LINUX|Linux)
  if [ "${WEBLOGIC_VERSION}" = "10.0" ]; then
    arch=`uname -m`
    if [ -n "${LD_LIBRARY_PATH}" ]; then
      if [ "${arch}" != "i86pc" ]; then
        SAVE_LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
      fi
    fi
  fi
  if [ "${WEBLOGIC_VERSION}" = "10.3" ]; then
     SAVE_LD_LIBRARY_PATH=${LD_LIBRARY_PATH}
  fi
;;
HP-UX)
  if [ "${WEBLOGIC_VERSION}" = "10.0" -o "${WEBLOGIC_VERSION}" = "10.3" ]; then
    arch=`uname -m`
    if [ "${arch}" = "ia64" ]; then
      SAVE_SHLIB_PATH=${SHLIB_PATH}
    fi
  fi
;;
*)
  echo "$0: Don't know how to set the shared library path for `uname -s`.  "
;;
esac

. "${WL_HOME}/common/bin/commEnv.sh"

WEBLOGIC_CLASSPATH="${WEBLOGIC_CLASSPATH}"
export WEBLOGIC_CLASSPATH

JAVA_VM="${JAVA_VM} ${global.bitwidthOption}"
export JAVA_VM

# Set 64-bit java native libraries for Solaris on Sparc and AIX
case `uname -s` in
AIX)
  LIBPATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/aix/ppc64;${SAVE_LIBPATH}
  export LIBPATH
;;
SunOS)
  arch=`uname -m`
  if [ "${arch}" = "i86pc" ]; then
   	 LD_LIBRARY_PATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/solaris/x64:${SAVE_LD_LIBRARY_PATH}
  else
   	 LD_LIBRARY_PATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/solaris/sparc64:${WL_HOME}/server/native/solaris/sparc64/oci920_8:${SAVE_LD_LIBRARY_PATH}
     export LD_LIBRARY_PATH
  fi
;;
LINUX|Linux)
  if [ "${WEBLOGIC_VERSION}" = "10.0" ]; then
    if [ "${OS_LINUX_VENDOR}" = "SuSE" ]; then
      arch=`uname -m`
      if [ "${arch}" != "i86pc" ]; then
   	    LD_LIBRARY_PATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/linux/i686:${WL_HOME}/server/native/linux/i686/oci920_8:${SAVE_LD_LIBRARY_PATH}
        export LD_LIBRARY_PATH
      fi
    fi
    if [ "${OS_LINUX_VENDOR}" = "redhat" -a "${OS_LINUX_VERSION}" = "4" ]; then
      arch=`uname -m`
      if [ "${arch}" != "i86pc" ]; then
   	    LD_LIBRARY_PATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/linux/i686:${WL_HOME}/server/native/linux/i686/oci920_8:${SAVE_LD_LIBRARY_PATH}
        export LD_LIBRARY_PATH
      fi
    fi    
  fi
  if [ "${WEBLOGIC_VERSION}" = "10.3" ]; then
   	 LD_LIBRARY_PATH=${PATCH_LIBPATH}:${WL_HOME}/server/native/linux/x86_64:${WL_HOME}/server/native/linux/x86_64/oci920_8:${SAVE_LD_LIBRARY_PATH}
  fi
;;
HP-UX)
  if [ "${WEBLOGIC_VERSION}" = "10.0" -o "${WEBLOGIC_VERSION}" = "10.3" ]; then
    arch=`uname -m`
    if [ "${arch}" = "ia64" ]; then
        SHLIB_PATH=${WL_HOME}/server/native/hpux11/IPF64:${WL_HOME}/server/native/hpux11/IPF64/oci920_8:${SAVE_SHLIB_PATH}
    fi
    SHLIB_PATH=${PATCH_LIBPATH}:${SHLIB_PATH}
    export SHLIB_PATH
  fi
;;
*)
  echo "$0: Don't know how to set the shared library path for `uname -s`.  "
;;
esac

# *********** Web Authentication Configuration *****************************
# If you are configuring your domain to use WebLogic Embedded LDAP Authentication as your
# authentication provider, copy the authentication provider jars into the webauth directory under
# your current domain and uncomment the following three lines. 
${global.webauthIsComment} WEBAUTH="${global.domainDir}/webauth"
${global.webauthIsComment} WEBAUTH_CLASSPATH="${WEBAUTH}/sas.core.jar:${WEBAUTH}/sas.core.nls.jar:${WEBAUTH}/sas.oma.omi.jar:${WEBAUTH}/sas.security.sspi.jar:${WEBAUTH}/sas.svc.connection.jar:${WEBAUTH}/sas.svc.connection.nls.jar:${WEBAUTH}/sas.svc.sec.login.jar:${WEBAUTH}/sas.svc.sec.login.nls.jar:${WEBAUTH}/sas.svc.sec.login.weblogic.jar:${WEBAUTH}/sas.svc.sec.login.weblogic.mbean.jar:${WEBAUTH}/sas.svc.sec.login.weblogic.mbean.nls.jar:${WEBAUTH}/sas.svc.sec.login.weblogic.nls.jar:${WEBAUTH}/sas.svcs.security.weblogic.jar:${WEBAUTH}/log4j.jar"
${global.webauthIsComment} WEBLOGIC_CLASSPATH="${WEBAUTH_CLASSPATH}:${WEBLOGIC_CLASSPATH}"
