# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

from com.sas.config import TemplateEngine,VelocityTemplateEngine
from com.sas.config.weblogic import WebLogicContextBuilder
from com.sas.config.util import PropertyUtils
from org.apache.commons.io import FileUtils

#**************************************************************************************************
# Name: SASaddLoginModule
# Purpose: Create or update the JAAS login module used by the managed servers. 
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this module (in velocity template):
#@@ credentials          domain.createloginmodule_SASTrust_userid: The SAS Trusted User. This identity is used to configure the JAAS login module.
#@@ credentials          domain.createloginmodule_SASTrust_passwd: The SAS Trusted User password.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#                        For the login module, this file is not used. All loginpolicy and loginmodule
#                        properties files are located and loaded into the velocity engine and a
#                        login module is generated.
#
# loginpolicy properties:
#@@ loginpolicy resourceDescription: JAAS login modules are configured in a flat file as documented at this URL: <a href="http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html">http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html</a>. The login policy configuration properties are stored in properties files that are named in the pattern <tt>loginpolicy.<em>resource</em>.properties</tt>. Modules are grouped together under an application name that is based on these policy names.
#@@ loginpolicy applicationPolicy:  The login policy name.  Typical values for a SAS deployment are <tt>PFS</tt>, <tt>SCS</tt>, and <tt>UsernamePassword</tt>.
#
# loginmodule properties: (descriptions for aliasdomain, debug, domain, host, port, repository come from javadocs for com.sas.services.security.login.OMILoginModule
#@@ loginmodule resourceDescription: JAAS login modules are configured in a flat file as documented at this URL: <a href="http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html">http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html</a>. The module configuration properties are stored in properties files that are named in the pattern <tt>loginmodule.<em>resource</em>.properties</tt>. Modules are grouped together under an application name.
#@@ loginmodule aliasdomain:        Another domain that this module responds to. This is used when this module runs in a remote JVM and receives generated credentials from an environment that uses the trusted authentication module.
#@@ loginmodule applicationPolicy:  The application policy that these properties apply to.
#@@ loginmodule debug:              Set this property to <tt>true</tt> to generate debug information to the System.out stream.
#@@ loginmodule domain:             The authentication domain that this module authenticates to.  Requests to authenticate users outside this domain are ignored.
#@@ loginmodule hasCredentials:     Set to <tt>true</tt> if this module incorporates credentials found in the credentials property file.
#@@ loginmodule holdOpenConnection: 
#@@ loginmodule host:               The host name for the SAS Metadata Server.
#@@ loginmodule loginModuleCode:    The login module class.
#@@ loginmodule loginModuleFlag:    Valid values: <tt>required</tt>, <tt>requisite</tt>, <tt>sufficient</tt>, <tt>optional</tt>. For more infomration, see <a href="http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html">http://download.oracle.com/javase/6/docs/api/javax/security/auth/login/Configuration.html</a>.
#@@ loginmodule port:               The port number that the SAS Metadata Server is listening on for new connections.
#@@ loginmodule repository:         The name of the repository that is set as the default repository. For the primary authentication, the typical value is Foundation.
# 
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! loginModuleFile
#!! templateDirectoryName
#!! webappsrvScriptingCacheCredentials
#
# Output:
#   Generates the weblogicConfig.login file passed to the -Djava.security.auth.login.config option to a managed server.
#   Creates a return code file with a return code indicating success or failure.
#**************************************************************************************************

def SASaddLoginModule(credentialsFile, resourcePropertiesFile):
    global templateEngine, pcb, encoding, templateDirectory, outputDirectory

    sasTrustUserPropName = "domain.createloginmodule_SASTrust"
    logger.debug('templateDirectoryName=' + templateDirectoryName)
    templateDirectory = File(templateDirectoryName)
    
    outputDirectory = File(loginModuleFile).getParentFile()
    encoding = 'UTF-8'
    if outputDirectory.exists() != true:
        outputDirectory.mkdir()
    templateEngine = VelocityTemplateEngine(templateDirectory,outputDirectory)

    pcb = WebLogicContextBuilder()

    resourceDir = File(domainPropertiesFile).getParentFile()
    logger.debug("resourceDir:" + `resourceDir`)
    
    # If a temporary credentials file is used during SDW run, real credentials file may have empty values.  
    # We need to put the credentials in the real file for velocity.  We need to clear them if they were not previously set. 

    tempCredFile = File(credentialsFile)
    realCredFile = File(resourceDir, 'credentials.properties')
    
    if getCredentialPassword(credentialsFile, sasTrustUserPropName) == "":
        print "sastrust password not set"
        logger.debug("sastrust password not set") 
    
    if `tempCredFile` == `realCredFile`:
        logger.debug("using credentials file")
    else:
        logger.debug("using temporary credentials")
        sasTrustUser = getCredentialUser(credentialsFile, sasTrustUserPropName)
        sasTrustPasswd = getCredentialPassword(credentialsFile, sasTrustUserPropName)
        PropertyUtils.setProperty(sasTrustUserPropName + "_userid", sasTrustUser, realCredFile)
        PropertyUtils.setProperty(sasTrustUserPropName + "_passwd", sasTrustPasswd, realCredFile)

    pcb.loadResources(resourceDir)

    generateFile(File(loginModuleFile).getName())
    if osType == 'unx':
        Chmod.file("600", File(loginModuleFile))

    if webappsrvScriptingCacheCredentials == "false":
        logger.debug("clearing credentials")
        PropertyUtils.setProperty(sasTrustUserPropName + "_userid", "", realCredFile)
        PropertyUtils.setProperty(sasTrustUserPropName + "_passwd", "", realCredFile)
        
    
    writeRC(RC_OK)
    return RC_OK
