# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys

#**************************************************************************************************
# Name: SAScheckServer
# Purpose: Monitor startup of a server.  It waits up to 40 minutes for a server start to 
#          either succeed or fail.  It polls the state of the server about every 30 seconds.  This
#          operation is useful to synchronize subsequent operations that require the server to be
#          started. 
#
# Input:
#   credentialsFile - The full path name of the file containing credentials.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#                     A server resource property file is expected.  The following properties are
#                     utilized by this operations:
#!! managedServerName = Name of the managed server to check.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!!	adminUrl
#
# Output:
#   Creates a return code file.  The return code reflects success or failure of the command, not
#   the success or failure of the started server.
#**************************************************************************************************

def SAScheckServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements 
    logger.debug("managedServerName=" + managedServerName)
    logger.debug("adminUrl=" + adminUrl)

    try:
	    # Connect to supplied administration server
    	connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        logger.error('Administration server not running.')
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    try:
        # Use runtime beans to get managed server state 
        domainRuntime()							

        mb = getMBean('ServerLifeCycleRuntimes/' + managedServerName)
        if mb == None:
            logger.error('Server ' + managedServerName + ' is not found.')
            writeRC(RC_RESOURCE_NOT_CONFIGURED)
            disconnect(); 
            return RC_RESOURCE_NOT_CONFIGURED

        serverState = mb.getState()

        import time
        i = 0
        while serverState != 'RUNNING':
            i = i + 1
            logger.debug(serverState)
            if serverState == 'SHUTTING_DOWN' or serverState == 'FORCE_SHUTTING_DOWN' or serverState == 'SUSPENDING' or serverState == 'FORCE_SUSPENDING':
                logger.info('Server State: ')
                logger.info(mb.getState())
            # Don't let it stay in UNKNOWN, SHUTDOWN, or FAILED state for very long
            if ( serverState == 'SHUTDOWN' or serverState == 'UNKNOWN' or serverState == 'FAILED') and i > 4:
                logger.info('Server down')
                break 
            # break out if this takes more than 80 iterations or about 40 minutes
            if i > 80:
                break
            time.sleep(30)
            serverState = mb.getState()
        logger.info(serverState)
        writeRC(RC_OK)
        disconnect()
        return RC_OK
    except WLSTException, e:
        logger.error('Failed to check status for server ' + SERVERNAME  + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
        disconnect()
        return RC_FAILURE
 