# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os.path
from com.sas.util import SasPasswordString
from com.sas.config.weblogic import MailSession

#**************************************************************************************************
# Name: SAScreateMailSession
# Purpose: Create or update a MailSession service and target it to a managed server.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this module:
#@@ credentials             mailsession.create_SASMailSession_userid: The mail session user name.  This credential is only used if the mailsession property <tt>mailsrvRequiresAuthentication</tt> is set to <tt>true</tt>.
#@@ credentials             mailsession.create_SASMailSession_passwd: The mail session user password
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ mailsession resourceDescription: Resources are stored in properties files that are named in the pattern <tt>mailsession.<em>resource</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation that is available from the Oracle Web site.  For example, the WebLogic Server 10.3.3 documentation is available at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/mail/CreateMailSessions.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/mail/CreateMailSessions.html</a>.
#@@ mailsession mailSessionJNDIName: The JNDI name for the mail session.   This name is configured in application configuration files and should not be changed without corresponding changes to the applications that use this mail session.
#@@ mailsession mailSessionName: The name of the mail session resource.
#@@ mailsession mailSessionSMTPHost: The host name of SMTP server for the mail session.
#@@ mailsession mailsrvRequiresAuthentication: This is a boolean value.  If set to <tt>true</tt> credentials are required to access this mail server.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************

def SAScreateMailSession(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug("adminUrl=" + adminUrl)
    logger.debug("targets=" + targets)
    logger.debug("mailSessionName=" + mailSessionName)
    logger.debug("mailSessionJNDIName=" + mailSessionJNDIName)
    logger.debug("mailSessionSMTPHost=" + mailSessionSMTPHost)
    logger.debug("mailsrvRequiresAuthentication=" + mailsrvRequiresAuthentication)
    logger.debug("credentialsFile=" + credentialsFile)
    
    editing = None

    try:
        if mailsrvRequiresAuthentication == "true":
            wlstMailSessionUser = getCredentialUser(credentialsFile, "mailsession.create_SASMailSession")
            wlstMailSessionPassword = getCredentialPassword(credentialsFile, "mailsession.create_SASMailSession")
            try:
                from com.sas.util import SasPasswordString
                wlstMailSessionPassword = SasPasswordString.decode(wlstMailSessionPassword)
            except ImportError, e:
                logger.error("Can't find sas.core.jar")
                writeRC(RC_NO_SAS_CORE_JAR)
                return RC_NO_SAS_CORE_JAR

        ms = MailSession()
        
        ms.loadProperties(File(resourcePropertiesFile))
        
        if mailsrvRequiresAuthentication == "true":
            ms.setUser(wlstMailSessionUser)
            
        if mailsrvRequiresAuthentication == "true":
            ms.setPassword(wlstMailSessionPassword)  
 
        pMailSessionProperties = ms.buildMailSessionProperties(false)
        
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)      
   
        edit()
        startEdit()
        editing = true

        # check if mail session already exists, if not create it
        try:
            cd('/MailSessions/' + mailSessionName)
            logger.debug('The MailSession with name ' + mailSessionName + ' already exists.')
            pMailSession = getMBean('/MailSessions/' + mailSessionName)
        except  WLSTException:
            pMailSession=create(mailSessionName, 'MailSession')
            pMailSession.setJNDIName(mailSessionJNDIName)
 
        logger.debug('MailSessionProperties:' + ms.buildMailSessionProperties(true))
        pMailSession.setProperties(makePropertiesObject(pMailSessionProperties))
    
        targetList = String.split(targets, ',')
        for t in targetList:
            pTargetServer = getMBean('/Servers/' + t)
            if pTargetServer == None:
                logger.error("target " + t + "does not exist")
                writeRC(RC_TARGET_DOES_NOT_EXIST)
                return RC_TARGET_DOES_NOT_EXIST
            pMailSession.addTarget(pTargetServer)
        save()
        activate(block="true") # Activate changes
        disconnect()
  
        logger.debug('The mail session has been created.')
        writeRC(RC_OK_NEW)
        return RC_OK_NEW
    except Exception, e:
        logger.error('Failed to create MailSession ' + mailSessionName + '.')
        logger.error(e)

        writeRC(RC_FAILURE)
    
        # Cancel edit.
        try:
            if editing is not None:
                logger.debug("Canceling edit...")
                cancelEdit('y')
        except WLSTException:
            pass
        return RC_FAILURE
