# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

from com.sas.util import SasPasswordString
#**************************************************************************************************
# Name: SAScreateUser
# Purpose: Create or update a user in the default realm of the WebLogic Embedded LDAP server.
#          
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this module:
#@@ credentials          user.create_<em>resource</em>_userid: The WebLogic Server default realm user.
#@@ credentials          user.create_<em>resource</em>_passwd: The WebLogic Server user's initial password.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ user resourceDescription: These resources represent WebLogic Server users in the default realm.  The resource files are named in the pattern <tt>user.<em>servername</em>.properties</tt>.  The credentials are stored in the credentials.properties file. For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/security/DefineUsers.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/security/DefineUsers.html</a>.
#@@ user description: A description of the user.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************

def SAScreateUser(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug( "adminUrl=" + adminUrl)
    logger.debug("credentialsFile=" + credentialsFile)

    try:
        print resourcePropertiesFile
        f = resourcePropertiesFile.split('.')
        userType = f[len(f)-2]
        logger.debug('userType: ' + userType)
        newUsername = getCredentialUser(credentialsFile, "user.create_" + userType)
        newPassword = getCredentialPassword(credentialsFile, "user.create_" + userType)
        newPassword = SasPasswordString.decode(newPassword)

        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)

        logger.debug('Creating new user.')
        authenticator=cmo.getSecurityConfiguration().getDefaultRealm().lookupAuthenticationProvider("DefaultAuthenticator")
        if not authenticator.userExists(newUsername):
            authenticator.createUser(newUsername,newPassword,description)
            logger.debug("Created user successfully")
        else:
            authenticator.resetUserPassword(newUsername,newPassword)
            logger.debug("User exists - resetting password")
  
        disconnect()
  
        logger.debug('Exiting User creation.')
        writeRC(RC_OK_NEW)
        return RC_OK_NEW
    except Exception, e:
        logger.error('Failed to create new user')
        logger.debug(e)
        writeRC(RC_FAILURE)
        return RC_FAILURE
