# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

from com.sas.config import TemplateEngine,VelocityTemplateEngine
from com.sas.config.weblogic import WebLogicContextBuilder
from com.sas.config.util import Chmod
from org.apache.commons.io import FileUtils
from org.apache.commons.io.filefilter import FileFilterUtils

from java.util import Date, Properties
from java.text import SimpleDateFormat
from java.io import FileInputStream,FileOutputStream

def generateFile(fileName):
    templateFile = File(templateDirectory,'/' + fileName)
    outputFile = File(outputDirectory,'/' + fileName)
    templateEngine.merge(pcb.build(templateFile), templateFile, encoding, outputFile)
    return

#**************************************************************************************************
# Name: SAScustomizeDomain
# Purpose: Adds SAS customizations to the Domain.  In particular a number of additional scripts
#          are added to the bin directory.
# 
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     There are no credentials used by this operation
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#                        For the domain, this file is not used. All domain properties are 
#                        found in global.properties and loaded before invoking this function.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! backupBinDir
#!! domainDirectory
#!! isAdminHost - used in velocity template for installSvc.cmd
#!! jdkBugWorkaround - used in velocity templates for commEnvSAS.cmd and commEnvSAS.sh
#!! midtierMulticastIpJavaOptions - used in velocity templates for commEnvSAS.cmd, commEnvSAS.sh, and installNoteMgrSvc.cmd
#!! mwHomeDir - used in velocity templates for setDomainEnv.cmd and setDomainEnv.sh
#!! osType
#!! pythonCachedirOption - used in velocity templates for stopWebLogic.cmd and stopWebLogic.sh
#!! SASWlstScriptHome
#!! templateDirectoryName
#!! webauthIsComment - used in velocity templates for setDomainEnv.cmd and setDomainEnv.sh
#!! weblogicBarName - used in velocity templates for installNodeMgrSvc.cmd and uninstallNodeMgrSvc.cmd
#!! weblogicProdName - used in velocity templates for installNodeMgrSvc.cmd and uninstallNodeMgrSvc.cmd
#**************************************************************************************************

def SAScustomizeDomain(credentialsFile, resourcePropertiesFile):
    global templateEngine, pcb, encoding, templateDirectory, outputDirectory

    logger.debug('resourcePropertiesFile:' + resourcePropertiesFile)

    loadProperties(resourcePropertiesFile)

    logger.debug('templateDirectoryName=' + templateDirectoryName)
    templateDirectory = File(templateDirectoryName)
    
    outputDirectory = File(domainDir + '/bldbin')
    encoding = 'UTF-8'
 
    if outputDirectory.exists() != true:
        outputDirectory.mkdir()
    
    templateEngine = VelocityTemplateEngine(templateDirectory,outputDirectory)

    pcb = WebLogicContextBuilder()
    resourceDir = File(domainPropertiesFile).getParentFile()
    logger.debug("resourceDir:" + `resourceDir`)
    pcb.loadResources(resourceDir)
    
    if osType == 'win':
        scriptExtension = 'cmd'
    else:
        scriptExtension = 'sh'
    generateFile('commEnvSAS.' + scriptExtension)
    generateFile('setDomainEnv.' + scriptExtension)
    generateFile('setWLSEnv.' + scriptExtension)
    generateFile('wlst.' + scriptExtension)
    generateFile('startManagedWebLogic.' + scriptExtension)
    generateFile('stopManagedWebLogic.' + scriptExtension)
    generateFile('stopWebLogic.' + scriptExtension)
    generateFile('startNodeManager.' + scriptExtension)
    generateFile('stopNodeManager.' + scriptExtension)
    if osType == 'win':
        generateFile('installNodeMgrSvc.' + scriptExtension)
        generateFile('uninstallNodeMgrSvc.' + scriptExtension)
        generateFile('ant.bat')

    # Move previous files to backup location and move working directory to 'bin'
    # df = SimpleDateFormat("yyyy-MM-dd-HH.mm.ss")
    # timestamp = df.format(Date())
    # backupDirectory = File(domainDir + '/back/bin_' + timestamp) 
    backupDirectory = File(backupBinDir)
    binDirectory = File(domainDir + '/bin')
    FileUtils.copyDirectory(binDirectory, backupDirectory)
    FileUtils.copyDirectory(outputDirectory, binDirectory, FileFilterUtils.suffixFileFilter(scriptExtension))

    if osType == 'unx':
        Chmod.file("700", File(binDirectory, "commEnvSAS.sh"))
        Chmod.file("700", File(binDirectory, "setDomainEnv.sh"))
        Chmod.file("700", File(binDirectory, "setWLSEnv.sh"))
        Chmod.file("700", File(binDirectory, "wlst.sh"))
        Chmod.file("700", File(binDirectory, "startManagedWebLogic.sh"))
        Chmod.file("700", File(binDirectory, "stopManagedWebLogic.sh"))
        Chmod.file("700", File(binDirectory, "stopWebLogic.sh"))
        Chmod.file("700", File(binDirectory, "startNodeManager.sh"))
        Chmod.file("700", File(binDirectory, "stopNodeManager.sh"))
 
    if string.lower(host) == string.lower(adminHost):
        adminServerResourceFile = File(resourceDir, 'server.AdminServer.properties')
        loadProperties(`adminServerResourceFile`)
        if osType == 'win':
            generateServiceScript('AdminServer', `adminServerResourceFile`)
    
    # Setup endorsed directory (S0708550)
    endorsedDir = File(domainDir, "endorsed/")
    endorsedDir.mkdir()
    endorsedJars = File(SASWlstScriptHome, "/endorsed").listFiles()
    for j in endorsedJars:
        logger.debug('copying jar file ' + j.getAbsolutePath() + ' to ' + endorsedDir.getAbsolutePath())
        FileUtils.copyFileToDirectory(j, endorsedDir)  

    Chmod.file("700", File(domainDir))
    Chmod.directory("go=", File(domainDir))
            
    writeRC(RC_OK)
    return RC_OK


