# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
   
def destroyResource(module, resourceType, resourceBean):
    if resourceType == 'ConnectionFactory':
        return module.destroyConnectionFactory(resourceBean)
    elif resourceType == 'Queue':
        return module.destroyQueue(resourceBean)
    elif resourceType == 'Topic':
        return module.destroyTopic(resourceBean)
    else:
        logger.error('Unsupported resource Type: ' + resourceType)
        return

#***********************************************************************************
# Name: SASdeleteJMSResource
# Purpose - delete a JMS Resource from a WebLogic Domain
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# resource properties used - see createJMSResource.py for descriptions of these resources:
#!! jmsresource deletedTargets
#!! jmsresource jmsModuleName
#!! jmsresource jmsServerName
#!! jmsresource resourceName
#!! jmsresource resourceType
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output: 
#     None
#***********************************************************************************
def SASdeleteJMSResource(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    logger.debug('adminUrl=' + adminUrl)
    logger.debug('resourceName=' + resourceName)
    logger.debug('deletedTargets=' + deletedTargets)
    logger.debug('jmsServerName=' + jmsServerName)
    logger.debug('jmsModuleName=' + jmsModuleName)
    logger.debug('resourceType=' + resourceType)

    rc = RC_FAILURE

    # Connect to administration server
    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    try:
        print 'Deleting '+ resourceType + ' named ' + resourceName + '.'
        edit()
        startEdit()
        cd('/')

        # get the JMS System Resource
        jmsSASSystemResource = getMBean('/JMSSystemResources/' + jmsModuleName)
        if jmsSASSystemResource is None:
            print 'No ' + jmsModuleName + ' found.'
            writeRC(RC_JMSMODULE_NOT_EXIST)
            rc = RC_JMSMODULE_NOT_EXIST
            cancelEdit('y')
            disconnect()
            return rc
        
        print 'The ' + jmsModuleName + ' exists.'
        theJMSResource = jmsSASSystemResource.getJMSResource()
        resourceBean = lookupResource(theJMSResource, resourceType, resourceName)
        if resourceBean is None:
            print 'the JMS Resource ' + resourceName + ' does not exist'
            writeRC(RC_RESOURCE_NOT_CONFIGURED)
            rc = RC_RESOURCE_NOT_CONFIGURED
            cancelEdit('y')
            disconnect()
            return rc
        
        destroyResource(theJMSResource, resourceType, resourceBean)
        if moreJMSResources(theJMSResource):
            print  'There are more resources remaining in this module.'
        else:
            print  'This module is empty.  Deleting ' + jmsModuleName
            cd('/SystemResources')
            delete(jmsModuleName, 'JMSSystemResources')
            
        writeRC(RC_OK)
        rc = RC_OK
        save()

        # Activate changes
        try:
            activate(block='true')
        except WLSTException:
            print 'Exception deleting JMS Resource, canceling Edit session to release lock.'
            writeRC(RC_FAILED_ACTIVATE)
            rc = RC_FAILED_ACTIVATE
            cancelEdit('y')
    
        disconnect()
        return rc
    except WLSTException, e:
        print 'Failed to delete JMS Resource ' + resourceName  + '.'
        print e
        writeRC(RC_FAILURE)
        cancelEdit('y')
        disconnect()
        return RC_FAILURE
