# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
#***********************************************************************************
# Name: SASdeleteServer
# Purpose - delete a WebLogic Managed Server
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# resource properties used - see createServer.py for descriptions of these resources:
#!! server  managedServerName
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output: 
#     None
#***********************************************************************************
def SASdeleteServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug('managedServerName ' + managedServerName)
    logger.debug('adminUrl ' + adminUrl)

    rc = RC_FAILURE

    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        logger.error('Failed to connect to the Admin Server.')
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    # Check if server exists
    try:
        cd('/Servers/' + managedServerName)
        logger.debug('The Server with name ' + managedServerName + ' exists.')
    except WLSTException:
        logger.error('The Server with name ' + managedServerName + ' does not exist.')
        writeRC(RC_OK_NOT_CONFIGURED)
        disconnect()
        return RC_OK_NOT_CONFIGURED

        # Remove server
    try:
        edit()
        startEdit()
  
        # "un-target" MailSessions referencing this server.
        cd('/MailSessions')
        mailSessionList = cmo.getMailSessions()
        if mailSessionList is not None:
            for ms in mailSessionList:
                targets = ms.getTargets()
                for t in targets:
                    if t.getName() == managedServerName:
                        logger.debug('Removing ' + managedServerName + ' target from MailSession: ' + ms.getName())
                        ms.removeTarget(t) 

        # "un-target" JDBC DataSources referencing this server.
        cd('/JDBCSystemResources')
        jdbcResourceList = cmo.getJDBCSystemResources()
        if jdbcResourceList is not None:
            for js in jdbcResourceList:
                targets = js.getTargets()
                for t in targets:
                    if t.getName() == managedServerName:
                        logger.debug('Removing ' + managedServerName + ' target from JDBC Resource: ' + js.getName())
                        js.removeTarget(t) 
          
        # "A JMS Server can have only one target
        # Remove any JMS Servers referencing this server.
        cd('/JMSServers')
        jmsServerList = cmo.getJMSServers()
        if jmsServerList is not None:
            for js in jmsServerList:
                targets = js.getTargets()
                for t in targets:
                    if t.getName() == managedServerName:
                        logger.debug('Removing JMS Server: ' + js.getName())
                        delete(js.getName(),'JMSServer')
 
        # "un-target" JMS System Resources referencing this server.
        cd('/JMSSystemResources')
        jmsSystemResourceList = cmo.getJMSSystemResources()
        if jmsSystemResourceList is not None:
            for js in jmsSystemResourceList:
                targets = js.getTargets()
                for t in targets:
                    if t.getName() == managedServerName:
                        logger.debug('Removing ' + managedServerName + ' target from JMS System Resource: ' + js.getName())
                        js.removeTarget(t) 
                        
        # Destroy JDBCStores referencing this server.
        cd('/JDBCStores')
        jdbcStoreList = cmo.getJDBCStores()
        if jdbcStoreList is not None:
          for js in jdbcStoreList:
            targets = js.getTargets()
            for t in targets:
              if t.getName() == managedServerName:
                logger.debug('Removing ' + managedServerName + ' target from JDBCStore Resource: ' + js.getName())
                cmo.destroyJDBCStore(getMBean('/JDBCStores/' + js.getName()))                        
            
        cd('/')
        delete(managedServerName,'Server')
        save()
        activate()
        writeRC(RC_OK)
        rc = RC_OK
        disconnect()  
    except WLSTException, e:
        logger.error('Failed to delete server ' + managedServerName  + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
        rc = RC_FAILURE
        cancelEdit('y')
        disconnect()
        
    return rc
