# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import string
import os.path
from com.sas.util import SasPasswordString

#**************************************************************************************************
# Name: SASlistServers
# Purpose: List servers in domain.
# Input:
#   credentialsFile - A properties file containing the key/value pairs.
#                     Credentials used in this module:
#                        domain.create_WeblogicAdmin_userid - Weblogic Admin Username.
#                        domain.create_WeblogicAdmin_passwd - Encoded Weblogic Admin Password.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   Writes a comma seperated list of <serverName>/<machine> to the return code file in the property 
#   "weblogic.servername.list".
#**************************************************************************************************

def SASlistServers(credentialsFile, resourcePropertiesFile):
 
    # Debug Statements
    logger.debug('adminUrl:' + adminUrl)

    serverList = None
    try:
  
        if os.path.exists(nmUserConfigFile) and os.path.exists(nmUserKeyFile):
            try:
                connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)      
            except WLSTException:
                logger.error('Failed to connect to the Admin Server.')
                writeRC(RC_FAIL_ADM_CONNECT)
                return RC_FAIL_ADM_CONNECT
        else:
            wlstAdminUsername = getCredentialUser(credentialsFile, "domain.create_WeblogicAdmin")
            wlstAdminPassword = getCredentialPassword(credentialsFile, "domain.create_WeblogicAdmin")
            wlstAdminPassword = SasPasswordString.decode(wlstAdminPassword)
            try:
                connect(url=adminUrl, username=wlstAdminUsername, password=wlstAdminPassword)  
            except WLSTException:
                logger.error('Failed to connect to the Admin Server.')
                writeRC(RC_FAIL_ADM_CONNECT)
                return RC_FAIL_ADM_CONNECT

        cd('/Servers')

        label = ""
        serverNameList = None
        serverList = cmo.getServers()
        if serverList is not None:
            for i in cmo.getServers():
                label = i.getName() + "/" 
                if i.getMachine() is not None:
                    machine_name = i.getMachine().getName()      
                    label = label + machine_name
                else:
                    label = label + "NO_MACHINE_DEFINED"
      
                logger.debug(label)
                if serverNameList is None:
                    serverNameList = label
                else:
                    serverNameList = serverNameList + "," + label  
        
        writeReturnProperty("weblogic.servername.list=", serverNameList)
        writeRC(RC_OK)
        disconnect() 
        return RC_OK   
    except Exception, e:
        logger.error('Unable to list servers.')
        logger.error(e)
        writeRC(RC_FAILURE)
        return RC_FAILURE
      