# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

#**************************************************************************************************
# Name: SASsetPolicy
# Purpose: set a policy for a resource in the default XACMLAuthorizer for WebLogic.
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#@@ policy resourceDescription: These resources represent a WebLogic default realm policy that is configured in a WebLogic domain.  The resource files are named in the pattern <tt>policy.<em>policyname</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/security/UseRolesAndPoliciesToSecureResources.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/security/UseRolesAndPoliciesToSecureResources.html</a>.
#@@ policy policyExpression: The policy expression to set for the resource.
#@@ policy policyName: This field is set but is not used.
#@@ policy securedResource: The resource ID.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************
def SASsetPolicy(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug("adminUrl=" + adminUrl)
    logger.debug("securedResource=" + securedResource)
    logger.debug("policyExpression=" + policyExpression) 

    try:
        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)

        authorizer=cmo.getSecurityConfiguration().getDefaultRealm().lookupAuthorizer("XACMLAuthorizer")
        if authorizer.policyExists(securedResource):
            logger.debug('previous policy: ' + authorizer.getPolicyExpression(securedResource))
            authorizer.setPolicyExpression(securedResource, policyExpression)
        else:
            logger.debug('No policy was previously set for resource.')
            authorizer.createPolicy(securedResource, policyExpression)
  
        logger.debug('The policy has been set to: ' + authorizer.getPolicyExpression(securedResource))
        disconnect()
  
        writeRC(RC_OK_NEW)
        return RC_OK_NEW
    except Exception, e:
        logger.error('Failed to set policy')
        logger.debug(e)
        writeRC(RC_FAILURE)
        return RC_FAILURE
