# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
import shutil

#***********************************************************************************
# Name: SASstartServer
# Purpose - start the managed server with nodemanager
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#     domainName - Name of domain
#     managedServerName - Name of managed server to start.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminHost
#!! adminUrl
#!! domainDir
#!! host
#!! nodeManagerPort
#
# Output: 
#     None
#***********************************************************************************

def SASstartServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    logger.debug('start server')
    logger.debug('host:' + host)
    logger.debug('nodeManagerPort:' + nodeManagerPort)
    logger.debug('domainDir:' + domainDir)
    logger.debug('managedServerName:' + managedServerName)
    logger.debug('adminUrl:' + adminUrl)
    logger.debug('adminHost:' + adminHost)
 
    setWLSCredentials()    
    
    if managedServerName == 'AdminServer':
        try:
            nmConnectHost = adminHost
            nmConnect(userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile, host=nmConnectHost, port=nodeManagerPort, domainName=os.path.basename(domainDir))
        except WLSTException:
            logger.error('Failed to connect to the node manager.')
            writeRC(RC_FAIL_NM_CONNECT) 
            return RC_FAIL_NM_CONNECT

    if managedServerName != 'AdminServer':
        try:
            connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
        except WLSTException:
            logger.error('Failed to connect to the Admin Server.')
            writeRC(RC_FAIL_ADM_CONNECT)
            nmDisconnect(); 
            return RC_FAIL_ADM_CONNECT
        try:  
            nmGenBootStartupProps(managedServerName)
        except WLSTException:
            logger.error('Failed to generate boot startup properties.  Server may not exist.')
            writeRC(RC_START_SRV_GEN_BOOT)
            disconnect();
            return RC_START_SRV_GEN_BOOT

    if managedServerName == 'AdminServer':
        try:
            serverStatus = nmServerStatus(managedServerName)
            if serverStatus == 'RUNNING':
                logger.info(managedServerName + ' is already running.')
                rc = RC_OK_ALREADY_STARTED
            else:
                logger.debug('Calling nmStart')
                nmStart(managedServerName)
                rc = RC_OK
                nmGenBootStartupProps('AdminServer')
        except weblogic.nodemanager.NMException:
            logger.error('Node manager failed to start server.')
            writeRC(RC_NM_FAILED_STARTSRV)
            nmDisconnect();
            if connected == 'true': 
                disconnect();
            return RC_NM_FAILED_STARTSRV
    else:
        # Use runtime beans to get managed server state 
        domainRuntime()
                                  
        mb = getMBean('ServerLifeCycleRuntimes/' + managedServerName)
        if mb == None:
            logger.error('Server ' + managedServerName + ' is not found.')
            writeRC(RC_RESOURCE_NOT_CONFIGURED)
            disconnect(); 
            return RC_RESOURCE_NOT_CONFIGURED
        serverState = mb.getState()
        if ( serverState == 'RUNNING' or serverState == 'STARTING' ):
            logger.info('Server ' + managedServerName + ' is already running or STARTING.')
            rc = RC_OK_ALREADY_STARTED
        else:    
            # start() dynamically pulls down configuration files from Admin Server.
            try:
                start(managedServerName, 'Server')
                rc = RC_OK
            except Exception, e:
                logger.error('Admin Server failed to start managed server')
                rc = RC_NM_FAILED_STARTSRV

    writeRC(rc)
    
    if connected == "true":
        disconnect()
    
    if managedServerName == 'AdminServer':  
        nmDisconnect()
    return rc