# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
from java.io import InputStreamReader, BufferedReader, IOException

#***********************************************************************************
# Name: execScript
# Purpose - launch a script command line in another process.
# Input:
#        cmd - the command line to execute.
#     
# Global properties used - see createDomain.py for descriptions of global resources:
#!! osType
#
#***********************************************************************************

def execScript(cmd):
    try:
        tempFile = File.createTempFile("execScript", ".out")
        logger.debug("tempFile=" + `tempFile`)
        if osType == 'win':
            cmdline = "cmd /c \"" + cmd + "\" >" + `tempFile` + " 2>&1"
        else:
            cmdline = cmd
        logger.debug("cmdline=" + cmdline)
        proc = Runtime.getRuntime().exec(cmdline)
        proc.waitFor()
        printStream(BufferedReader(InputStreamReader(proc.getInputStream())))
        printStream(BufferedReader(InputStreamReader(proc.getErrorStream())))
        printFile(`tempFile`)
        tempFile.delete()
        rc = proc.exitValue()
        logger.debug("return code=" + `rc`)
        return `rc`
    except Exception, e:
        logger.error("Failed to execute script")
        logger.error(e)
        return "-999"
    
def printStream(stream):
    try:
        line = stream.readLine()
        while line != None:
            logger.debug(line)
            line = stream.readLine()
    except IOException:
        print 'IO Exception in ' + `stream`


#***********************************************************************************
# Name: SASstartNodeManager
# Purpose - start the nodemanager, if its not already active
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials not used in this module.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#     nodeManagerHome - home directory for nodemanager
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! domainDir
#!! host
#!! nodeManagerPort
#!! weblogicNodeManagerServiceName
#
# Output: 
#     None
#***********************************************************************************
           
def SASstartNodeManager(credentialsFile, resourcePropertiesFile):
    
    nodeManagerHomeFile = File(domainDir, "nodemanager")
    nodeManagerHome = `nodeManagerHomeFile`

    # Debug Statements
    logger.debug('host:' + host)
    logger.debug('nodeManagerPort:' + nodeManagerPort)
    logger.debug('nodeManagerHome:' + nodeManagerHome)

    if osType == 'win':
        rc = execScript(domainDir + "\\bin\\uninstallNodeMgrSvc.cmd")
        rc = execScript(domainDir + "\\bin\\installNodeMgrSvc.cmd")
        # Note: beasvc doesn't return a proper error code, so this check is not valid.  Keeping this
        # code here in case this situation should change. 
        if (rc != "0"):
            logger.error('Failed to install Nodemanager service.  return code=' + rc)
            return RC_NM_FAILED_TO_INSTALL_SVC
        # We can rely on sc to return a failure if the previous step failed.
        rc = execScript("sc start " + weblogicNodeManagerServiceName)
        if (rc != "0"):
            logger.error('Failed to start Nodemanager service.  return code=' + rc)
            return RC_NM_FAILED_TO_START
    else:
        startNodeManager(ListenPort=nodeManagerPort, ListenAddress=host, NodeManagerHome=nodeManagerHome)
    
    if not waitUntilServerIsAvailable(host, nodeManagerPort):
        writeRC(RC_NM_TIMED_OUT_STARTING)
        return RC_NM_TIMED_OUT_STARTING
        
    writeRC(RC_OK)
    return RC_OK
