# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
#***********************************************************************************
# Name: SASstopServer
# Purpose - stop the managed server from nodemanager
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#     managedServerName - Name of managed server to stop.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output: 
#     None
#***********************************************************************************

def SASstopServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug('adminUrl:' + adminUrl)
    logger.debug('managedServerName:' + managedServerName)
 
    rc = RC_FAILURE

    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        logger.error('Failed to connect to the Admin Server.')
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    try: 
        # Use runtime beans to get managed server state 
        domainRuntime()
                                  
        mb = getMBean('ServerLifeCycleRuntimes/' + managedServerName)
        if mb == None:
            logger.error('Server ' + managedServerName + ' is not found.')
            writeRC(RC_RESOURCE_NOT_CONFIGURED)
            disconnect(); 
            return RC_RESOURCE_NOT_CONFIGURED
        serverState = mb.getState()
        if ( serverState == 'SHUTDOWN' or serverState == 'UNKNOWN' or serverState == 'FAILED' or serverState == 'SHUTTING_DOWN' or serverState == 'FORCE_SHUTTING_DOWN' or serverState == 'SUSPENDING' or serverState == 'FORCE_SUSPENDING'):
            logger.info('Server ' + managedServerName + ' is already down or in the process of shutting down.')
            writeRC(RC_OK_ALREADY_SHUTDOWN)
            rc = RC_OK_ALREADY_SHUTDOWN
        else:    
            shutdown(name=managedServerName,entityType='Server',ignoreSessions='true',timeOut=300, force='true', block='true')
            writeRC(RC_OK)
            rc = RC_OK
    except WLSTException:
        logger.error('Failed to shutdown managed server.')
        writeRC(RC_FAILED_STOP_SERVER)
        rc = RC_FAILED_STOP_SERVER

    disconnect()
    return rc