# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os

#***********************************************************************************
# Name: SASstopNodeManager
# Purpose - stop the nodemanager
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials not used in this module.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#     nodeManagerHome - home directory for nodemanager
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! domainDir
#!! host
#!! nodeManagerPort
#!! osType
#!! weblogicNodeManagerServiceName
#
# Output: 
#     None
#***********************************************************************************

def SASstopNodeManager(credentialsFile, resourcePropertiesFile):
    # Debug Statements
    logger.debug('host:' + host)
    logger.debug('nodeManagerPort:' + `nodeManagerPort`)

    if osType == 'win':
        execScript("sc stop " + weblogicNodeManagerServiceName)
        execScript(domainDir + "\\bin\\uninstallNodeMgrSvc.cmd")
    else:
        if os.path.exists(nmUserConfigFile) and os.path.exists(nmUserKeyFile):
            nmConnect(userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile, host=host, port=nodeManagerPort, domainName=os.path.basename(domainDir), domainDir=domainDir)
        else:
            adminUser = getCredentialUser(credentialsFile, "domain.create_WeblogicAdmin")
            wlstAdminPassword = getCredentialPassword(credentialsFile, "domain.create_WeblogicAdmin")
            try:
                from com.sas.util import SasPasswordString
                adminPasswd = SasPasswordString.decode(wlstAdminPassword)
            except ImportError, e:
                logger.error("Can't find sas.core.jar")
                writeRC(RC_NO_SAS_CORE_JAR)
                return RC_NO_SAS_CORE_JAR

            domain_name = os.path.basename(domainDir)
            logger.debug('domain_name:' + domain_name)
            nmConnect(adminUser, adminPasswd, host=host, port=nodeManagerPort, domainName=domain_name, domainDir=domainDir)
            logger.debug('nmConnect completed')
            storeUserConfig(nmUserConfigFile,  nmUserKeyFile, nm="true")
            logger.debug('storeUserConfig completed')
        stopNodeManager()

    writeRC(RC_OK)
    return RC_OK
        