# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import string

#**************************************************************************************************
# Name: SASundeployApplication
# Purpose: Undeploy application from specified targets
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#   appName - Name of the application.
#   adminUrl - Weblogic Admin Console URL
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
#**************************************************************************************************
def SASundeployApplication(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
    
    # Debug Statements
    logger.debug("applicationName=" + applicationName)
    logger.debug("adminUrl=" + adminUrl)

    contexts = None

    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
  
        appMBean = getMBean('/AppDeployments/' + applicationName)
        if appMBean is None:
            logger.debug('The application ' + applicationName + ' is not deployed.')
        else:
            undeploy(applicationName)
        disconnect()
  
        logger.debug('Undeployed application successfully.')
        writeRC(RC_OK)
        return RC_OK
    except Exception, e:
        errorMsg = 'Failed to undeploy application: ' + applicationName + '.'
        logger.error(errorMsg)
        logger.error(e)
  
        if rcf is not None:
            writeRC(RC_FAILURE)
 
    return RC_FAILURE
