::*****************************************************************************************************
:: Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
::*****************************************************************************************************
::
:: Name:     WASDriver.bat
::
:: Purpose:  Run the WASDriver.groovy program to perform the configuration task(s)
::           that prepare the WAS environment for SAS webapps.
:: 
:: Command-line options:
:: ---------------------
::    -e = Execution type: "RUN" - Run a single task as specified by command-line options
::                         "FILE" - Run all tasks specified in the taskFile command-line option
::    -m = Execution mode: "AUTO" - Run task(s) automatically
::                         "NOAUTO" = Syntax check all task(s)
::    -p = Prompt mode: "GRAPHIC" - Display prompts for required credentials using pop-window
::                      "TEXT" - Display prompts for required credentials using text-mode
::    -t = Task file: <Fully-qualified path to task file> (-e FILE required)
::    -d = Scripting directory: <Fully-qualified path to directory containing all scripting components>
::    -o = Task operation: (-e RUN required)
::             create, delete, start, stop, deploy, undeploy, update, updateProps, 
::             validate, updateProviderEndpoints, reset, syncNode, getPort, getStatus, 
::             checkAppStatus, clear, encodeAll, removeDuplicates
::    -r = Resource type: (-e RUN required)  
::             ActivationSpec, Application, Credentials, DataSource, DmgrProfile,
::             JDBCProvider, JMSConnectionFactory, JMSQueue, JMSTopic, LoginModule,
::             MailSession, MultiCast, Node, NodeProfile, ObjectCache, Server, SIBus
::    -n = Resource name: (-e RUN required)
::             <Name of resource to be configured>
::    -s = Scope: (-e RUN required)
::             CELL, NODE, SERVER
::***************************************************************************************************** 
						       
@ECHO off
setlocal enableextensions

::********************************************************
:: Defaults for common variables for the WASDriver program
set WD_JAVA=@javaLocation@
set WD_CLASSPATH=@groovyClasspath@
set WD_TASKFILE=@taskFile@
set WD_SCRIPTINGDIR=@scriptingDirectory@
set WD_EXECMODE=AUTO
set WD_EXECTYPE=FILE
set WD_PROMPT=TEXT

::***************************************
:: Examine each input command line option
echo ** Process command-line options %* 
set WD_HELP=0
set OPTIONS.=
set /a OPTNUM=1
for %%O in (%*) do call :CHECKOPTS %%O

::***********************************************************
:: Determine if help for WASDriver program should be produced 
if %WD_HELP% == 1 (
    "%WD_JAVA%\java" -cp "%WD_CLASSPATH%"  com.sas.config.websphere.WASDriver --help
    exit /b
)

::****************************************
:: Set script variables from option values
for /F "tokens=2* delims=.=" %%A IN ('SET OPTIONS.') DO call :SET_WDPARM %%A %%B

::***************************************************************************
:: Run WASDriver Groovy program after determining type of execution requested
echo ** Run the WASDriver program with the following options 
echo ** WD_EXECMODE="%WD_EXECMODE%"
echo ** WD_EXECTYPE="%WD_EXECTYPE%"
echo ** WD_PROMPT="%WD_PROMPT%"
echo ** WD_SCRIPTINGDIR="%WD_SCRIPTINGDIR%"
if NOT EXIST "%WD_SCRIPTINGDIR%\." (
    echo **ERROR** Scripting directory "%WD_SCRIPTINGDIR%" does not exist.
    exit /b
)
if "%WD_EXECTYPE%" == "FILE" goto :TASKFILE
if "%WD_EXECTYPE%" == "RUN"  goto :SINGLETASK
echo **ERROR** Unknown -e option "%WD_EXECTYPE%", must be "RUN" or "FILE".
exit /b

::********************************************
:: Save each command line argument in an array
:CHECKOPTS
    if "%1" == "-h"     set WD_HELP=1
    if "%1" == "-help"  set WD_HELP=1
    if "%1" == "--help"  set WD_HELP=1
    set OPTIONS.%OPTNUM%=%1
    set /a OPTNUM=%OPTNUM%+1
    goto :EOF

::***********************************************************
:: Set the variables used as values for the WASDriver options
:SET_WDPARM
    set /a VALUE_INDEX=%1+1
    if "%2" == "-e" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_EXECTYPE=%%S
    if "%2" == "-m" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_EXECMODE=%%S
    if "%2" == "-t" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_TASKFILE=%%S
    if "%2" == "-d" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_SCRIPTINGDIR=%%S
    if "%2" == "-p" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_PROMPT=%%S
    if "%2" == "-o" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_OPERATION=%%S
    if "%2" == "-r" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_RESOURCETYPE=%%S
    if "%2" == "-n" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_RESOURCENAME=%%S
    if "%2" == "-s" for /F "tokens=2* delims=.=" %%R in ('SET OPTIONS.%VALUE_INDEX%') do set WD_SCOPE=%%S
    goto :EOF

::*******************************
:: Run WASDriver with a task file  
:TASKFILE
    echo ** WD_TASKFILE="%WD_TASKFILE%"
    if NOT EXIST "%WD_SCRIPTINGDIR%\tasks\%WD_TASKFILE%" goto :TASKFILE_ERROR
    echo "%WD_JAVA%\java" -cp "%WD_CLASSPATH%" com.sas.config.websphere.WASDriver -e %WD_EXECTYPE% -m %WD_EXECMODE% -p %WD_PROMPT% -d %WD_SCRIPTINGDIR% -t %WD_TASKFILE%
    "%WD_JAVA%\java" -cp "%WD_CLASSPATH%" com.sas.config.websphere.WASDriver -e %WD_EXECTYPE% -m %WD_EXECMODE% -p %WD_PROMPT% -d %WD_SCRIPTINGDIR% -t %WD_TASKFILE%
    exit /b
    
:TASKFILE_ERROR
    echo **ERROR** Task file "%WD_SCRIPTINGDIR%\tasks\%WD_TASKFILE%" does not exist.   
    exit /b
    
::********************************
:: Run WASDriver for a single task
:SINGLETASK 
    echo ** WD_OPERATION="%WD_OPERATION%"
    echo ** WD_RESOURCETYPE="%WD_RESOURCETYPE%"
    echo ** WD_RESOURCENAME="%WD_RESOURCENAME%"
    echo ** WD_SCOPE="%WD_SCOPE%"
    set WD_PROPERTYFILE=%WD_SCRIPTINGDIR%\props\%WD_RESOURCETYPE%.%WD_SCOPE%.%WD_RESOURCENAME%.properties
    if NOT EXIST %WD_PROPERTYFILE% goto :SINGLETASK_ERROR
    echo "%WD_JAVA%\java" -cp "%WD_CLASSPATH%" com.sas.config.websphere.WASDriver -e %WD_EXECTYPE% -m %WD_EXECMODE% -p %WD_PROMPT% -d %WD_SCRIPTINGDIR% -o %WD_OPERATION% -r %WD_RESOURCETYPE% -n %WD_RESOURCENAME% -s %WD_SCOPE%
    "%WD_JAVA%\java" -cp "%WD_CLASSPATH%" com.sas.config.websphere.WASDriver -e %WD_EXECTYPE% -m %WD_EXECMODE% -p %WD_PROMPT% -d %WD_SCRIPTINGDIR% -o %WD_OPERATION% -r %WD_RESOURCETYPE% -n %WD_RESOURCENAME% -s %WD_SCOPE%
    exit /b  
    
:SINGLETASK_ERROR:
    echo **ERROR** Required property file "%WD_PROPERTYFILE%" does not exist.
    exit /b

endlocal